'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _dialog = require('react-toolbox/lib/dialog');

var _dialog2 = _interopRequireDefault(_dialog);

var _PureComponent2 = require('./PureComponent');

var _PureComponent3 = _interopRequireDefault(_PureComponent2);

var _link = require('react-toolbox/lib/link');

var _link2 = _interopRequireDefault(_link);

var _LaunchHelperModule = require('./LaunchHelper.module.css');

var _LaunchHelperModule2 = _interopRequireDefault(_LaunchHelperModule);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('./underscore_ext'),
    map = _require.map,
    pick = _require.pick,
    mapObject = _require.mapObject,
    getIn = _require.getIn,
    get = _require.get;

var platform = require('platform');
var Rx = require('./rx');


var osFiles = {
	osxJre: {
		pattern: "ucsc_xena_macos_[_0-9]*_with_jre.dmg",
		description: "OSX installer, bundled JRE",
		help: "Recommended for OSX 10.7 and above"
	},
	win32: {
		pattern: "ucsc_xena_windows_[_0-9]*.exe",
		description: "Windows 32 bit installer, bundled JRE",
		help: "Recommended for all 32 bit versions of Windows"
	},
	win64: {
		pattern: "ucsc_xena_windows-x64_[_0-9]*.exe",
		description: "Windows 64 bit installer, bundled JRE",
		help: "Recommended for all 64 bit versions of Windows"
	},
	tar: {
		pattern: "ucsc_xena_[_0-9]*.tar.gz",
		description: "Tar archive, no updater or JRE",
		help: "Recommended for linux server deployments"
	}
},
    defaults = {
	'OS X': { 64: 'osxJre' },
	'Windows': { 32: 'win32', 64: 'win64' },
	'Linux': { 32: 'tar', 64: 'tar' }
};

function findMatch(pattern, list) {
	var matches = list.filter(function (s) {
		return s.match(pattern);
	});
	return matches.length > 0 ? matches[0] : undefined;
}

var matchPaths = function matchPaths(serverFiles) {
	return mapObject(osFiles, function (obj, key) {
		return findMatch(osFiles[key].pattern, serverFiles);
	});
};

var parseInt10 = function parseInt10(s) {
	return parseInt(s, 10);
};

function osxArch() {
	var version = getIn(platform, ['os', 'version']),
	    parts = version && version.match(/(\d*)\.(\d*)/).slice(1).map(parseInt10); // [maj, min, patch]
	return parts && parts[0] === 10 && parts[1] <= 6 ? 32 : 64;
}

function getOs() {
	var family = getIn(platform, ['os', 'family']);
	if (family) {
		if (family.indexOf('Windows') !== -1) {
			return 'Windows';
		}
		if (family.indexOf('OS X') !== -1) {
			return 'OS X';
		}
		if (family.indexOf('Linux') !== -1) {
			return 'Linux';
		}
	}
}

var downloadDir = 'https://genome-cancer.ucsc.edu/download/public/get-xena';
var i4jLogo = downloadDir + '/install4j_small.png';
var updatesPath = downloadDir + '/updates.xml';
var downloadPath = function downloadPath(name) {
	return downloadDir + '/' + name;
};
var getFileName = function getFileName(e) {
	return downloadPath(e.getAttribute('fileName'));
};

var XenaDownload = function (_React$Component) {
	(0, _inherits3.default)(XenaDownload, _React$Component);

	function XenaDownload() {
		var _ref;

		var _temp, _this, _ret;

		(0, _classCallCheck3.default)(this, XenaDownload);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref = XenaDownload.__proto__ || (0, _getPrototypeOf2.default)(XenaDownload)).call.apply(_ref, [this].concat(args))), _this), _this.state = {}, _this.onShowAdvanced = function () {
			_this.props.onShowAdvanced(!_this.props.advanced);
		}, _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
	}

	(0, _createClass3.default)(XenaDownload, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			var _this2 = this;

			// Haven't been able to get the browsers to use the etags from
			// the server. It might need some other header to indicate that
			// the doc should be considered expired. This doesn't help in
			// the current deployment, because the headers for the doc have already
			// been served. A tacky work-around is to append a random query string,
			// forcing the client to bypass its cache.
			// Note that the etags do appear to work for some files, like the
			// image files.
			this.sub = Rx.Observable.ajax({ method: 'GET', responseType: 'xml', url: updatesPath + '?x=' + Math.random().toString(), crossDomain: true }).subscribe(function (xml) {
				var files = matchPaths([].concat((0, _toConsumableArray3.default)(xml.response.getElementsByTagName('entry'))).map(getFileName));

				_this2.setState({ files: files }); //eslint-disable-line react/no-did-mount-set-state
			});
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.sub.unsubscribe();
		}
	}, {
		key: 'render',
		value: function render() {
			var files = this.state.files,
			    _props = this.props,
			    isFirst = _props.isFirst,
			    advanced = _props.advanced,
			    os = getOs(),
			    arch = os === 'OS X' ? osxArch() : getIn(platform, ['os', 'architecture']),
			    defaultTarget = getIn(defaults, [os, arch]),
			    defaultInstall = get(files, defaultTarget);


			return _react2.default.createElement(
				'div',
				{ className: _LaunchHelperModule2.default.download },
				_react2.default.createElement(
					'div',
					{ className: _LaunchHelperModule2.default.i4j },
					_react2.default.createElement(
						'p',
						null,
						'Supported by ',
						_react2.default.createElement(
							'a',
							{ href: 'https://www.ej-technologies.com/products/install4j/overview.html' },
							_react2.default.createElement('img', { src: i4jLogo })
						)
					)
				),
				_react2.default.createElement(
					'p',
					null,
					_react2.default.createElement('br', null),
					_react2.default.createElement(
						'span',
						{ className: _LaunchHelperModule2.default.largeFont },
						isFirst ? 'If this is your first time, ' : 'If nothing prompts from browser, '
					),
					defaultInstall ? _react2.default.createElement(_link2.default, { className: _LaunchHelperModule2.default.downloadLink, href: defaultInstall, label: 'download & run a Local Xena hub.' }) : 'download & run a Local Xena hub, from the list below:'
				),
				defaultInstall ? _react2.default.createElement(
					'span',
					{ className: _LaunchHelperModule2.default.advancedLink, onClick: this.onShowAdvanced },
					advanced ? 'Fewer options...' : 'More options...'
				) : null,
				defaultInstall ? _react2.default.createElement('br', null) : null,
				_react2.default.createElement(
					'div',
					{ className: advanced || !defaultInstall ? _LaunchHelperModule2.default.tableShow : _LaunchHelperModule2.default.table },
					map(files ? pick(osFiles, function (_, k) {
						return files[k];
					}) : [], function (info, key) {
						return _react2.default.createElement(_link2.default, { className: _LaunchHelperModule2.default.downloadList, href: files[key], title: info.help, label: info.description });
					})
				),
				_react2.default.createElement('br', null)
			);
		}
	}]);
	return XenaDownload;
}(_react2.default.Component);

var launchingHelp = ['Launching...', _react2.default.createElement(
	'p',
	null,
	'Please click ',
	_react2.default.createElement(
		'b',
		null,
		'Open UCSC xena'
	),
	' if you see the system dialog.'
)];

var statusHelp = {
	undefined: [],
	down: launchingHelp,
	started: launchingHelp,
	up: ['Your local Xena Hub is running.', _react2.default.createElement(
		'p',
		null,
		'To view your data, use the "Visualization" button'
	)],
	old: ['Your local Xena Hub is out of date.', _react2.default.createElement('p', null)],
	lost: ['We have lost contact with your Local Xena hub.', _react2.default.createElement(
		'p',
		null,
		'To re-start it, you may reload this page.'
	)]
};

var launch = function launch() {
	var i = document.getElementById('xenaLauncher');
	if (i) {
		document.body.removeChild(i);
	}
	i = document.createElement('iframe');
	i.id = 'xenaLauncher';
	document.body.appendChild(i);
	i.src = 'ucscxena://';
};

// XXX Note that we re-render on every change of the wrapped
// component. Maybe should put dialog in a separate component to
// avoid re-rendering it.
var wrap = function wrap(Comp) {
	var _class, _temp2;

	return _temp2 = _class = function (_PureComponent) {
		(0, _inherits3.default)(_class, _PureComponent);

		function _class(props) {
			(0, _classCallCheck3.default)(this, _class);

			// If we just arrived on a page that requires the hub, and status is
			// 'down' but not 'lost' (i.e. user shutdown), try to start.
			var _this3 = (0, _possibleConstructorReturn3.default)(this, (_class.__proto__ || (0, _getPrototypeOf2.default)(_class)).call(this, props));

			_this3.onShowAdvanced = function (advanced) {
				_this3.setState({ advanced: advanced });
			};

			_this3.onHide = function () {
				_this3.setState({ show: false });
			};

			_this3.onShow = function () {
				_this3.setState({ show: true });
			};

			_this3.setCompRef = function (ref) {
				_this3.compRef = ref;
			};

			_this3.actions = [{ label: 'Help', target: '_blank', href: 'https://ucsc-xena.gitbook.io/project/local-xena-hub' }, { label: 'Close', onClick: _this3.onHide }];
			_this3.state = { advanced: false, show: props.state.localStatus === 'down' || props.state.localStatus === 'old' };
			return _this3;
		}

		(0, _createClass3.default)(_class, [{
			key: 'componentDidMount',
			value: function componentDidMount() {
				if (this.props.state.localStatus === 'down') {
					launch();
				}
			}
		}, {
			key: 'componentWillReceiveProps',
			value: function componentWillReceiveProps(props) {
				// If localStatus was not set (no ping yet, from server) and changes to 'down',
				// show dialog & try to launch.
				if (!this.props.state.localStatus && props.state.localStatus === 'down') {
					this.setState({ show: true });
					launch();
				} else if (props.state.localStatus === 'up') {
					this.setState({ show: false });
				}
			}
		}, {
			key: 'render',
			value: function render() {
				var _state = this.state,
				    advanced = _state.advanced,
				    show = _state.show,
				    status = this.props.state.localStatus,
				    statusBadge = status === 'up' ? _react2.default.createElement(
					'i',
					{ title: 'Connected to local Xena Hub',
						className: 'material-icons ' + _LaunchHelperModule2.default.badgeConnected,
						onClick: this.onShow },
					'lens'
				) : _react2.default.createElement(
					'i',
					{ title: 'Not connected to local Xena Hub. Click for details.',
						className: 'material-icons ' + _LaunchHelperModule2.default.badgeDisconnected,
						onClick: this.onShow },
					'lens'
				),
				    unsupported = platform.name.match(/safari/i) && status === 'down',
				    _ref2 = unsupported ? [platform.name + ' does not support viewing your own data. Please use Chrome or Firefox', _react2.default.createElement('p', null)] : statusHelp[status],
				    _ref3 = (0, _slicedToArray3.default)(_ref2, 2),
				    header = _ref3[0],
				    help = _ref3[1];

				return _react2.default.createElement(
					Comp,
					(0, _extends3.default)({ ref: this.setCompRef }, this.props, { badge: statusBadge }),
					_react2.default.createElement(
						_dialog2.default,
						{ theme: { body: _LaunchHelperModule2.default.body }, active: show, actions: this.actions, className: _LaunchHelperModule2.default.dialog },
						_react2.default.createElement('div', { className: _LaunchHelperModule2.default.padding }),
						_react2.default.createElement(
							'h2',
							{ className: _LaunchHelperModule2.default.header },
							header
						),
						_react2.default.createElement('div', { className: _LaunchHelperModule2.default.padding }),
						_react2.default.createElement(
							'div',
							{ className: _LaunchHelperModule2.default.status },
							help,
							status !== 'up' && status !== 'lost' ? _react2.default.createElement(XenaDownload, { isFirst: status !== 'old', advanced: advanced, onShowAdvanced: this.onShowAdvanced }) : null
						),
						_react2.default.createElement(
							'p',
							{ className: _LaunchHelperModule2.default.footer },
							'A Local Xena Hub is an application on your computer for loading and storing data.'
						)
					)
				);
			}
		}]);
		return _class;
	}(_PureComponent3.default), _class.displayName = 'LaunchHelperWrapper', _temp2;
};

var wrapLaunchHelper = function wrapLaunchHelper(shouldMount, Comp) {
	var Wrapper = wrap(Comp),
	    LaunchHelperMounter = function LaunchHelperMounter(props) {
		return shouldMount(props) ? _react2.default.createElement(Wrapper, props) : _react2.default.createElement(Comp, props);
	};
	return LaunchHelperMounter;
};

exports.default = wrapLaunchHelper;