'use strict';

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// deprecated. Was used to draw ga4gh annotations.
/*global require: false, module: false */

var _ = require('underscore');

var _require = require('static-interval-tree'),
    matches = _require.matches;

var _require2 = require('./layoutPlot'),
    pxTransformEach = _require2.pxTransformEach;

// Draw annotations on one or more evenly sized horizontal bands,
// each band representing one set of values. E.g. two bands
// [[2, 3], [7, 8]]
// will render data points with value 2 and 3 on one band, and data points
// with value 7 and 8 on another band. The data points are drawn in the
// order given, so 3 will overwrite 2, and 8 will overwrite 7.
// 'data' should be in the format [{start: i, end: i, val: i}, ...]

function drawBands(vg, bands, color, layout, indx) {
	var count = bands.length,
	    height = vg.height() / count;

	pxTransformEach(layout, function (toPx, _ref) {
		var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
		    start = _ref2[0],
		    end = _ref2[1];

		var matching = matches(indx, { start: start, end: end });
		var dbg = _.groupBy(matching, 'val');

		_.each(bands, function (band, i) {
			return _.each(band, function (curval) {
				return _.each(dbg[curval], function (_ref3) {
					var start = _ref3.start,
					    end = _ref3.end,
					    val = _ref3.val;

					var _toPx = toPx([start, end]),
					    _toPx2 = (0, _slicedToArray3.default)(_toPx, 2),
					    pstart = _toPx2[0],
					    pend = _toPx2[1];

					vg.box(pstart, height * i, pend - pstart || 1, height, color(val));
				});
			});
		});
	});
}

var vcmp = function vcmp(_ref4, _ref5) {
	var v1 = _ref4.val;
	var v2 = _ref5.val;
	return v1 === v2 ? 0 : v1 > v2 ? 1 : -1;
};

// Draw annotations on one or more evenly sized horizontal bands,
// each band representing a different sequence of floating point values.

function drawFloatBands(vg, bands, color, layout) {
	var count = bands.length,
	    height = vg.height() / count;

	pxTransformEach(layout, function (toPx, _ref6) {
		var _ref7 = (0, _slicedToArray3.default)(_ref6, 2),
		    start = _ref7[0],
		    end = _ref7[1];

		_.each(bands, function (band, i) {
			var matching = matches(band, { start: start, end: end });
			_.each(matching.slice(0).sort(vcmp), function (_ref8) {
				var start = _ref8.start,
				    end = _ref8.end,
				    val = _ref8.val;

				// sort, so higher vals draw last
				var _toPx3 = toPx([start, end]),
				    _toPx4 = (0, _slicedToArray3.default)(_toPx3, 2),
				    pstart = _toPx4[0],
				    pend = _toPx4[1];

				vg.box(pstart, height * i, pend - pstart || 1, height, color(val)); // minimum 1 px
			});
		});
	});
}

module.exports = {
	drawBands: drawBands,
	drawFloatBands: drawFloatBands
};