'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _ui = require('./controllers/ui');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('./underscore_ext');

var _require = require('reselect'),
    createSelectorCreator = _require.createSelectorCreator,
    defaultMemoize = _require.defaultMemoize;

var _require2 = require('./selectors'),
    createFmapSelector = _require2.createFmapSelector;

var widgets = require('./columnWidgets');
var km = require('./models/km');

var _require3 = require('./models/searchSamples'),
    searchSamples = _require3.searchSamples;

var isPublicSelector = require('./isPublicSelector');
// XXX should move userServers, or maybe put it in a selector

var _require4 = require('./controllers/common'),
    userServers = _require4.userServers;

var minWidth = (0, _ui.defaultWidth)(0);

var createSelector = createSelectorCreator(defaultMemoize, _.isEqual);

var indexSelector = createFmapSelector(function (state) {
	return _.fmap(state.columns, function (column, key) {
		return [_.getIn(column, ['fieldType']), state.data[key]];
	});
}, function (args) {
	return widgets.index.apply(widgets, (0, _toConsumableArray3.default)(args));
});

function cmpString(s1, s2) {
	if (s1 > s2) {
		return 1;
	} else if (s2 > s1) {
		return -1;
	}
	return 0;
}

var invert = function invert(dir, fn) {
	return dir === 'reverse' ? function (s1, s2) {
		return fn(s2, s1);
	} : fn;
};

var sortSelector = createSelector(function (state) {
	return state.cohortSamples;
}, function (state) {
	return _.fmap(state.columns, function (c) {
		return _.pick(c, 'fieldType', 'fields', 'xzoom', 'sortVisible', 'sortDirection');
	});
}, function (state) {
	return state.columnOrder;
}, function (state) {
	return state.data;
}, function (state) {
	return state.index;
}, function (cohortSamples, columns, columnOrder, data, index) {
	var getSampleID = function getSampleID(i) {
		return _.get(cohortSamples, i);
	},
	    order = columnOrder.slice(1),
	    // skip 'samples' in sort
	cmpFns = _.fmap(columns, function (c, id) {
		return invert(c.sortDirection, widgets.cmp(columns[id], data[id], index[id]));
	}),

	// XXX should further profile this to see how much it's costing us
	// to create a findValue callback on every cmpFn call.
	cmpFn = function cmpFn(s1, s2) {
		return _.findValue(order, function (id) {
			return cmpFns[id](s1, s2);
		}) || cmpString(getSampleID(s1), getSampleID(s2));
	};

	return _.range((cohortSamples || []).length).sort(cmpFn);
});

// This could be further optimized to eliminate the mergeKeys calls, below, which will
// re-create every column object on every state update, regardless of whether the column
// has changed. We never have more than about ten columns, so it's probably pointless to do
// so.
var transformSelector = createFmapSelector(function (state) {
	return _.fmap(state.columns, function (column, key) {
		return [_.omit(column, 'user'), // ugh. Review column schema + widget.transform.
		_.getIn(column, ['vizSettings']), state.data[key], state.samples, state.index[key]];
	});
}, _.apply(widgets.transform));

var avgSelector = createFmapSelector(function (state) {
	return _.fmap(state.columns, function (column, key) {
		return [_.omit(column, 'user'), // Review column schema + widget.avg.
		state.data[key], _.get(state.cohortSamples, 'length', 0), state.index[key]];
	});
}, _.apply(widgets.avg));

var matchSelector = createSelector(function (state) {
	return state.sampleSearch;
}, function (state) {
	return state.columns;
}, function (state) {
	return state.columnOrder;
}, function (state) {
	return state.data;
}, function (state) {
	return state.cohortSamples;
}, searchSamples);

var mergeKeys = function mergeKeys(a, b) {
	return _.mapObject(a, function (v, k) {
		return _.merge(v, b[k]);
	});
};

var kmSelector = createSelector(function (state) {
	return state.samples;
}, function (state) {
	return _.getIn(state, ['columns', _.getIn(state, ['km', 'id'])]);
}, function (state) {
	return _.getIn(state, ['data', _.getIn(state, ['km', 'id'])]);
}, function (state) {
	return _.getIn(state, ['index', _.getIn(state, ['km', 'id'])]);
}, function (state) {
	return _.getIn(state, ['km', 'cutoff']);
}, function (state) {
	return _.getIn(state, ['km', 'splits']);
}, function (state) {
	return _.getIn(state, ['km', 'survivalType']);
}, function (state) {
	return state.survival;
}, function (samples, column, data, index, cutoff, splits, survivalType, survival) {
	return column && survival && km.makeGroups(column, data, index, cutoff, splits, survivalType, survival, samples);
});

// Enforce default width in wizardMode
var ammedWidthSelector = createFmapSelector(function (_ref) {
	var columns = _ref.columns,
	    wizardMode = _ref.wizardMode,
	    _ref$defaultWidth = _ref.defaultWidth,
	    defaultWidth = _ref$defaultWidth === undefined ? minWidth : _ref$defaultWidth;
	return _.fmap(columns, function (column) {
		return { column: column, wizardMode: wizardMode, defaultWidth: defaultWidth };
	});
}, function (_ref2) {
	var column = _ref2.column,
	    wizardMode = _ref2.wizardMode,
	    defaultWidth = _ref2.defaultWidth;
	return wizardMode ? _.assoc(column, 'width', defaultWidth) : column;
});

var index = function index(state) {
	return (0, _extends3.default)({}, state, { index: indexSelector(state) });
};
var avg = function avg(state) {
	return (0, _extends3.default)({}, state, { data: mergeKeys(state.data, avgSelector(state)) });
};
var match = function match(state) {
	return _.Let(function () {
		var m = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : matchSelector(state);
		var hl = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : state.highlightSelect;
		return (0, _extends3.default)({}, state, {
			samplesMatched: !m.matches ? null : hl == null ? _.last(m.matches) : m.matches[hl],
			allMatches: m });
	});
};
var sort = function sort(state) {
	return (0, _extends3.default)({}, state, { samples: sortSelector(state) });
};
var transform = function transform(state) {
	return (0, _extends3.default)({}, state, { columns: mergeKeys(state.columns, transformSelector(state)) });
};
var ammedWidth = function ammedWidth(state) {
	return (0, _extends3.default)({}, state, { columns: ammedWidthSelector(state) });
};
var setPublic = function setPublic(state) {
	return (0, _extends3.default)({}, state, { isPublic: isPublicSelector(state) });
};

// kmGroups transform calculates the km data, and merges it into the state.km object.

var kmGroups = function kmGroups(state) {
	return (0, _extends3.default)({}, state, { km: (0, _extends3.default)({}, state.km, { groups: kmSelector(state) }) });
};

var spreadsheetSelector = function spreadsheetSelector(selector) {
	return function (state) {
		return _.updateIn(state, ['spreadsheet'], selector);
	};
};

//

var supportsTies = function supportsTies(state) {
	return _.getIn(state, ['cohort', 'name'], '').indexOf('TCGA') === 0;
};

var tiesSelector = function tiesSelector(state) {
	return _.assoc(state, 'tiesEnabled', supportsTies(state));
};

var pickUserServers = function pickUserServers(obj, servers) {
	return _.pick(obj, userServers({ servers: servers }));
};

var cohortsSelector = createSelector(function (state) {
	return state.wizard.serverCohorts;
}, function (state) {
	return state.spreadsheet.servers;
}, function (serverCohorts, servers) {
	return _.uniq(_.flatten(_.values(pickUserServers(serverCohorts, servers))));
});

var indexBy = function indexBy(key, list) {
	return _.object(_.pluck(list, key), list);
};

var datasetsSelector = createSelector(function (state) {
	return _.get(state.wizard.cohortDatasets, _.get(state.spreadsheet.cohort, 'name'), {});
}, function (state) {
	return state.spreadsheet.servers;
}, function (cohortDatasets, servers) {
	return indexBy('dsID', _.flatten(_.values(pickUserServers(cohortDatasets, servers))));
});

// XXX should try to deprecate this & use cohortFeatures
var featuresByDsID = function featuresByDsID(cohortFeatures) {
	return _.object(_.flatmap(cohortFeatures, function (datasets, host) {
		return _.map(datasets, function (features, dataset) {
			return [(0, _stringify2.default)({ host: host, name: dataset }), features];
		});
	}));
};

var featuresSelector = createSelector(function (state) {
	return _.get(state.wizard.cohortFeatures, _.get(state.spreadsheet.cohort, 'name'), {});
}, function (state) {
	return state.spreadsheet.servers;
}, function (cohortFeatures, servers) {
	return featuresByDsID(pickUserServers(cohortFeatures, servers));
});

var setWizardProps = function setWizardProps(selector) {
	return function (state) {
		return selector(_.updateIn(state, ['wizard'], function (wizard) {
			return _.merge(wizard, {
				cohorts: cohortsSelector(state),
				datasets: datasetsSelector(state),
				features: featuresSelector(state)
			});
		}));
	};
};

///////
// This is the main transform ('selector') of the application state, before passing to the view.
// We build indexes of the column data, sort samples by the column data, transform
// the data for display (e.g. map to colors), and calculate km if a km plot is requested.
//
// The result of the transforms is a state object with the calculated values merged.
// The transforms are memoized for performance.

var selector = function selector(state) {
	return tiesSelector(kmGroups(transform(sort(match(avg(index(ammedWidth(setPublic(state)))))))));
};

// This seems odd. Surely there's a better test?
var hasSurvival = function hasSurvival(survival) {
	return !!(_.some(_.values(km.survivalOptions), function (option) {
		return _.get(survival, option.ev) && _.get(survival, option.tte);
	}) && _.get(survival, 'patient'));
};

var survivalSelector = createSelector(function (state) {
	return state.wizard.cohortFeatures;
}, function (state) {
	return state.spreadsheet.cohort;
}, function (state) {
	return state.spreadsheet.km;
}, function (cohortFeatures, cohort, user) {
	return cohort && cohortFeatures && hasSurvival(km.pickSurvivalVars(cohortFeatures[cohort.name], user));
});

var setSurvival = function setSurvival(selector) {
	return function (state) {
		return selector(_.assocIn(state, ['spreadsheet', 'hasSurvival'], survivalSelector(state)));
	};
};

module.exports = setWizardProps(setSurvival(spreadsheetSelector(selector)));