'use strict';

// browser method for parsing a binary buffer with
// list of file hashes and bloom filters.


var Bloom = require('ucsc-bloomfilter.js');

var hashLen = 20; // sha1 hash length
var hexToStr = function hexToStr(v) {
	return (v < 16 ? '0' : '') + v.toString(16);
};
function hashFromBuff(buff, offset) {
	var v = '';
	for (var i = 0; i < hashLen; ++i) {
		v += hexToStr(buff[offset + i]);
	}
	return v;
}

// buff is ArrayBuffer with hash/filter pairs
function loadFilters(buff) {
	var uint8 = new Uint8Array(buff),
	    filters = {},
	    i = 0,
	    filter;

	while (i < buff.byteLength) {
		filter = new Bloom(buff, i + hashLen);
		filters[hashFromBuff(uint8, i)] = filter;
		i += hashLen + filter.bufferSize;
	}

	return filters;
}

module.exports = loadFilters;