'use strict';

// node method for writing a set of bloom filters and
// file hashes to a file.


var fs = require('fs');

var times = function times(n, fn) {
	for (var i = 0; i < n; ++i) {
		fn(i);
	}
};

function hashToBuffer(hash) {
	var buff = Buffer.alloc(hash.length / 2);
	times(hash.length / 2, function (i) {
		buff[i] = parseInt(hash.slice(i * 2, i * 2 + 2), 16);
	});
	return buff;
}

function writeFilters(fileName, filters) {
	var f = fs.openSync(fileName, 'w');

	filters.forEach(function (_ref) {
		var hash = _ref.hash,
		    filter = _ref.filter;

		var _filter$buffer = filter.buffer(),
		    buffer = _filter$buffer.buffer,
		    start = _filter$buffer.start,
		    size = _filter$buffer.size;

		fs.writeSync(f, hashToBuffer(hash));

		var buff = new Buffer(buffer);
		fs.writeSync(f, buff, start, start + size);
	});
}

module.exports = writeFilters;