'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getColumns = exports.dataset = exports.cohort = exports.getOpts = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.columnsParam = columnsParam;

var _util = require('./util');

var _underscore_ext = require('./underscore_ext');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var columnOptPaths = {
	width: ['width'],
	columnLabel: ['user', 'columnLabel'],
	fieldLabel: ['user', 'fieldLabel'],
	sortDirection: ['sortDirection'],
	normalize: ['vizSettings', 'colNormalization'],
	geneAverage: ['fieldType'],
	showIntrons: ['showIntrons'],
	sortVisible: ['sortVisible']
};

// maybe should move the cleaning to models/columns.js, so we can enforce things that vary with column type.
var invalid = {}; // use reference equality to tag invalid values.
var columnOptCleaner = {
	width: function width(v) {
		return !(0, _underscore_ext.isNumber)(v) ? invalid : v < 10 ? 10 : v > 500 ? 500 : v;
	},
	sortDirection: function sortDirection(s) {
		return s === 'reverse' ? s : s === 'forward' ? s : invalid;
	},
	normalize: function normalize(v) {
		return v === 'none' ? 'none' : v === 'mean' ? 'subset' : v === 'log2' ? 'log2(x)' : v === 'normal2' ? 'normal2' : invalid;
	},
	geneAverage: function geneAverage(v) {
		return !(0, _underscore_ext.isBoolean)(v) ? invalid : v ? 'genes' : invalid;
	}, // not so much invalid, as 'use default': 'geneProbes', or 'probes'.
	showIntrons: function showIntrons(v) {
		return (0, _underscore_ext.isBoolean)(v) ? v : invalid;
	},
	sortVisible: function sortVisible(v) {
		return (0, _underscore_ext.isBoolean)(v) ? v : invalid;
	}
};
var columnOptClean = function columnOptClean(opt, v) {
	return (columnOptCleaner[opt] || _underscore_ext.identity)(v);
};
var columnOpts = (0, _keys2.default)(columnOptPaths);

// invert columnOptCleaner. For creating links.
var columnOptSetter = {
	sortDirection: function sortDirection(s) {
		return s ? s : invalid;
	},
	normalize: function normalize(v) {
		return v === 'none' ? 'none' : v === 'mean' ? 'subset' : v === 'log2(x)' ? 'log2' : v === 'normal2' ? 'normal2' : invalid;
	}, // default
	geneAverage: function geneAverage(v) {
		return v === 'genes' ? true : invalid;
	} // default
};
var columnOptSet = function columnOptSet(opt, v) {
	return (columnOptSetter[opt] || _underscore_ext.identity)(v);
};

var columnRequired = ['name', 'host', 'fields'];

var columnAllowed = [].concat(columnRequired, (0, _toConsumableArray3.default)(columnOpts));

var mergeOpts = function mergeOpts(c) {
	return (0, _underscore_ext.merge)((0, _underscore_ext.omit)(c, columnOpts), { opts: (0, _underscore_ext.flatmap)((0, _underscore_ext.pick)(c, columnOpts), function (v, k) {
			return (0, _underscore_ext.Let)(function () {
				var cleaned = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : columnOptClean(k, v);
				return cleaned === invalid ? [] : [[columnOptPaths[k], cleaned]];
			});
		}) });
};

var pickAllowed = function pickAllowed(c) {
	return (0, _underscore_ext.pick)(c, columnAllowed);
};

var columnSchema = function columnSchema(list) {
	return (0, _underscore_ext.isArray)(list) && list.length > 0 && list.every(function (column) {
		return (0, _underscore_ext.isObject)(column) && ((0, _underscore_ext.has)(column, 'name') && (0, _underscore_ext.has)(column, 'host') || (0, _underscore_ext.has)(column, 'cohort') && (0, _underscore_ext.has)(column, 'type')) && (0, _underscore_ext.has)(column, 'fields');
	});
};

function columnsParam() {
	var _allParameters = (0, _util.allParameters)(),
	    columns = _allParameters.columns,
	    filterColumns = _allParameters.filterColumns;

	if (columns) {
		try {
			var visibleColumns = JSON.parse(columns);
			var filterOnly = filterColumns ? JSON.parse(filterColumns) : [];
			var list = visibleColumns.concat(filterOnly).map(function (c) {
				return mergeOpts(pickAllowed(c));
			});
			if (columnSchema(list)) {
				return { columns: list, visible: visibleColumns.length + 1 }; // +1 for sampleID
			}
		} catch (e) {
			console.log('Failed to parse columns ' + columns);
		}
	}
	return {};
}

var getOpts = exports.getOpts = function getOpts(c) {
	return mergeOpts(pickAllowed(c)).opts;
};

// XXX not really implemented. The idea was to allow 'basic' data types
// via cohort + type.
var cohort = exports.cohort = function cohort(columns) {
	return (0, _underscore_ext.find)(columns, function (c) {
		return (0, _underscore_ext.has)(c, 'cohort');
	});
};

// XXX currently unused
var dataset = exports.dataset = function dataset(columns) {
	return columns[0].name;
};

var columnDatasetFields = function columnDatasetFields(column) {
	return (0, _extends3.default)({}, JSON.parse(column.dsID), {
		fields: column.fields.join(' ')
	});
};

var columnOptions = function columnOptions(column) {
	return (0, _underscore_ext.pick)((0, _underscore_ext.mapObject)(columnOptPaths, function (path, key) {
		return columnOptSet(key, (0, _underscore_ext.getIn)(column, path));
	}), function (v) {
		return v !== invalid;
	});
};

var encode = function encode(x) {
	return encodeURIComponent((0, _stringify2.default)(x));
};

// extract from state
var getColumns = exports.getColumns = function getColumns(state) {
	return encode((0, _underscore_ext.map)((0, _underscore_ext.omit)((0, _underscore_ext.getIn)(state, ['spreadsheet', 'columns'], {}), 'samples'), function (column) {
		return (0, _underscore_ext.merge)(columnOptions(column), columnDatasetFields(column));
	}));
};