'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _PureComponent2 = require('../PureComponent');

var _PureComponent3 = _interopRequireDefault(_PureComponent2);

var _columns = require('../models/columns');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var React = require('react');
var _ = require('../underscore_ext');
var CohortOrDisease = require('../views/CohortOrDisease');
var VariableSelect = require('../views/VariableSelect');
var GhostVariableSelect = require('../views/GhostVariableSelect');
var getStepperState = require('./getStepperState');
var uuid = require('../uuid');
var Rx = require('../rx');


// Configuration for first and second variable select cards that are displayed during wizard.
var variableSelectConfig = {
	'FIRST_COLUMN': {
		helpText: {
			'Genotypic': 'Add a gene (e.g. RB1) or position (e.g. chr19p), and select a dataset.',
			'Phenotypic': 'Add a phenotype (e.g. sample type, age).'
		},
		pos: 1,
		title: 'First Variable'
	},
	'SECOND_COLUMN': {
		helpText: {
			'Genotypic': 'Add a gene (e.g. RB1) or position (e.g. chr19p), and select a dataset.',
			'Phenotypic': 'Add a phenotype (e.g. sample type, age).'
		},
		pos: 2,
		title: 'Second Variable'
	}
};

function wizardColumns(wizardMode, stepperState, cohortSelectProps, datasetSelectProps, width) {
	if (wizardMode) {
		if (stepperState === 'COHORT') {
			return [React.createElement(CohortOrDisease, (0, _extends3.default)({ key: 'c1' }, cohortSelectProps)), React.createElement(GhostVariableSelect, (0, _extends3.default)({ key: 'c2', width: width }, variableSelectConfig.FIRST_COLUMN)), React.createElement(GhostVariableSelect, (0, _extends3.default)({ key: 'c3', width: width }, variableSelectConfig.SECOND_COLUMN))];
		}
		if (stepperState === 'FIRST_COLUMN') {
			return [React.createElement(VariableSelect, (0, _extends3.default)({ key: 'c2' }, variableSelectConfig[stepperState], datasetSelectProps)), React.createElement(GhostVariableSelect, (0, _extends3.default)({ key: 'c3', width: width }, variableSelectConfig.SECOND_COLUMN))];
		}
		if (stepperState === 'SECOND_COLUMN') {
			return [React.createElement(VariableSelect, (0, _extends3.default)({ key: 'c3' }, variableSelectConfig[stepperState], datasetSelectProps))];
		}
	}
	return [];
}

var preferredLabels = {
	'gene expression': 'Gene Expression',
	'copy number': 'Copy Number',
	'simple somatic mutation': 'Somatic Mutation'
};

var activeHubs = function activeHubs(hubs) {
	return _.keys(hubs).filter(function (hub) {
		return hubs[hub].user;
	});
};
var cohortName = function cohortName(cohort) {
	return _.get(cohort, 'name');
};
var getByCohort = function getByCohort(table, cohort, def) {
	return _.get(table, cohortName(cohort), def);
};

function getPreferedDatasets(cohort, cohortPreferred, hubs, datasets) {
	if (!cohortPreferred) {
		return; // signify 'not loaded'
	}
	var active = activeHubs(hubs),

	// Only include datasets on active hubs & real existing datasets (more
	// reliable against mistakes in the .json file)
	avail = function avail(ds) {
		return _.contains(active, JSON.parse(ds).host) && _.has(datasets, ds);
	},

	// filter out key used to support geneset/pathway view
	notPathway = function notPathway(ds, key) {
		return key !== "copy number for pathway view";
	},
	    preferred = _.pick(getByCohort(cohortPreferred, cohort, {}), function (ds, key) {
		return avail(ds) && notPathway(ds, key);
	});

	// Use isEmpty to handle 1) no configured preferred datasets or 2)
	// preferred dataset list is empty after filtering by active hubs.
	return _.isEmpty(preferred) ? [] : _.keys(preferred).map(function (type) {
		return { dsID: preferred[type], label: preferredLabels[type] };
	});
}

function getAnalytic(cohort, cohortAnalytic, hubs, datasets) {
	if (!cohortAnalytic) {
		return; // signify 'not loaded'
	}
	var active = activeHubs(hubs),

	// Only include datasets on active hubs & real existing datasets (more reliable against mistakes in the .json file)
	analytic = _.filter(getByCohort(cohortAnalytic, cohort, []), function (ds) {
		return _.Let(function () {
			var dsID = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _stringify2.default)(_.pick(ds, 'host', 'name'));
			return _.contains(active, ds.host) && _.has(datasets, dsID);
		});
	});

	return _.isEmpty(analytic) ? [] : analytic;
}

function getPreferredPhenotypes(cohort, cohortPreferredPhenotypes, hubs) {
	if (!cohortPreferredPhenotypes) {
		return; // signify 'not loaded'
	}
	var active = activeHubs(hubs),
	    preferred = _.filter(getByCohort(cohortPreferredPhenotypes, cohort, []), function (_ref) {
		var dsID = _ref.dsID;
		return _.contains(active, JSON.parse(dsID).host);
	});

	return _.isEmpty(preferred) ? [] : preferred;
}

var consolidateFeatures = function consolidateFeatures(featureSet) {
	return _.reduce(featureSet, function (all, features, dsID) {
		var strippedFeatures = _.toArray(_.mapObject(features, function (f) {
			return _.merge(f, { dsID: dsID, label: f.longtitle || f.name });
		}));
		return all.concat(strippedFeatures);
	}, []);
};

var sortFeatures = function sortFeatures(features) {
	return _.sortBy(features, function (f) {
		return f.label.toUpperCase();
	});
};

var removeSampleID = function removeSampleID(features) {
	return _.filter(features, function (f) {
		return f.name !== "sampleID";
	});
};

// 1) if appState.editing, then set editing state, and render editor.
// 2) if wizard mode
//      add cohort editor, or
//      add 1st column editor, or
//      add 2nd column editor
function addWizardColumns(Component) {
	var _class, _temp;

	return _temp = _class = function (_PureComponent) {
		(0, _inherits3.default)(_class, _PureComponent);

		function _class(props) {
			(0, _classCallCheck3.default)(this, _class);

			var _this = (0, _possibleConstructorReturn3.default)(this, (_class.__proto__ || (0, _getPrototypeOf2.default)(_class)).call(this, props));

			_this.onCancel = function () {
				_this.props.callback(['edit-column', null]);
			};

			_this.onCohortSelect = function (cohort) {
				_this.props.callback(['cohort', cohort, _columns.typeWidth.matrix]);
			};

			_this.onDatasetSelect = function (posOrId, selected) {
				var _this$props$wizard = _this.props.wizard,
				    datasets = _this$props$wizard.datasets,
				    features = _this$props$wizard.features,
				    isPos = _.isNumber(posOrId),
				    settingsList = _.map(selected, function (s) {
					return (0, _columns.computeSettings)(datasets, features, s.opts, s.dataset.dsID, s);
				});

				_this.props.callback(['add-column', posOrId].concat((0, _toConsumableArray3.default)(settingsList.map(function (settings, i) {
					return { id: !i && !isPos ? posOrId : uuid(), settings: settings };
				}))));
			};

			var editing = props.editing;

			_this.state = { editing: editing };
			return _this;
		}

		(0, _createClass3.default)(_class, [{
			key: 'componentWillMount',
			value: function componentWillMount() {
				var callback = this.props.callback;

				this.sub = Rx.Observable.of(true).concat(Rx.Observable.fromEvent(window, 'resize')).filter(function () {
					return !window.cypressScreenshot;
				}) // cypress work-around
				.debounceTime(200).subscribe(function () {
					return callback(['viewportWidth', document.documentElement.clientWidth]);
				});
			}
		}, {
			key: 'componentWillUnmount',
			value: function componentWillUnmount() {
				this.sub.unsubscribe();
			}
		}, {
			key: 'componentWillReceiveProps',
			value: function componentWillReceiveProps(newProps) {
				var editing = newProps.editing;

				this.setState({ editing: editing });
			}
		}, {
			key: 'addColumns',
			value: function addColumns() {
				var _props = this.props,
				    children = _props.children,
				    appState = _props.appState,
				    wizard = _props.wizard,
				    cohort = appState.cohort,
				    wizardMode = appState.wizardMode,
				    defaultWidth = appState.defaultWidth,
				    servers = appState.servers,
				    cohorts = wizard.cohorts,
				    cohortPreferred = wizard.cohortPreferred,
				    cohortAnalytic = wizard.cohortAnalytic,
				    cohortMeta = wizard.cohortMeta,
				    cohortPhenotype = wizard.cohortPhenotype,
				    datasets = wizard.datasets,
				    features = wizard.features,
				    stepperState = getStepperState(appState),
				    editing = appState.editing,
				    analytic = getAnalytic(cohort, cohortAnalytic, servers, datasets),
				    preferred = getPreferedDatasets(cohort, cohortPreferred, servers, datasets),
				    preferredPhenotypes = getPreferredPhenotypes(cohort, cohortPhenotype, servers),
				    width = defaultWidth,
				    cohortSelectProps = {
					cohorts: cohorts,
					cohortMeta: cohortMeta,
					onSelect: this.onCohortSelect,
					width: width },
				    datasetSelectProps = {
					datasets: datasets,
					features: features && sortFeatures(removeSampleID(consolidateFeatures(features))),
					preferred: preferred,
					analytic: analytic,
					basicFeatures: preferredPhenotypes,
					onSelect: this.onDatasetSelect,
					width: width },
				    columns = React.Children.toArray(children),
				    cancelIcon = React.createElement(
					'i',
					{ className: 'material-icons', onClick: this.onCancel },
					'cancel'
				),
				    withEditor = columns.map(function (el) {
					return editing === el.props.id ? React.createElement(VariableSelect, (0, _extends3.default)({
						key: editing,
						actionKey: editing,
						pos: editing,
						text: appState.columns[editing].value,
						fields: appState.columns[editing].fieldList || appState.columns[editing].fields,
						dataset: appState.columns[editing].dsID,
						title: 'Edit Variable'
					}, datasetSelectProps, {
						colId: el.props.label,
						controls: cancelIcon })) : el;
				}),
				    withNewColumns = _.flatmap(withEditor, function (el, i) {
					return editing === i ? [el, React.createElement(VariableSelect, (0, _extends3.default)({ key: i, actionKey: i, pos: i, title: 'Add Variable'
					}, datasetSelectProps, { controls: cancelIcon }))] : [el];
				});

				return withNewColumns.concat(wizardColumns(wizardMode, stepperState, cohortSelectProps, datasetSelectProps, width));
			}
		}, {
			key: 'render',
			value: function render() {
				var _props2 = this.props,
				    children = _props2.children,
				    _props2$appState = _props2.appState,
				    editing = _props2$appState.editing,
				    wizardMode = _props2$appState.wizardMode,
				    columns = editing != null || wizardMode ? this.addColumns() :
				// This looks like a noop, but toArray changes element keys. If
				// we don't do this, there's a mismatch in keys during editing,
				// which causes expensive re-mounts.
				React.Children.toArray(children);

				return React.createElement(
					Component,
					this.props,
					columns
				);
			}
		}]);
		return _class;
	}(_PureComponent3.default), _class.displayName = 'SpreadsheetWizardColumns', _temp;
}

module.exports = addWizardColumns;