'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _rx = require('../rx');

var _rx2 = _interopRequireDefault(_rx);

var _underscore_ext = require('../underscore_ext');

var _defaultServers = require('../defaultServers');

var _xenaQuery = require('../xenaQuery');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var localHub = _defaultServers.servers.localHub;
var _Rx$Observable = _rx2.default.Observable,
    interval = _Rx$Observable.interval,
    never = _Rx$Observable.never,
    of = _Rx$Observable.of;


var backoffInterval = function backoffInterval(low, high, multiple) {
	return _rx2.default.Observable.of(low).expand(function (t) {
		return _rx2.default.Observable.of(Math.min(multiple * t, high)).delay(t);
	});
};

var oneSecond = 1000,
    twentySeconds = 20 * oneSecond,
    tenMinutes = 10 * 60 * oneSecond;

var contactTimeout = 200;

var nextState = function nextState(state, _ref) {
	var status = _ref.status;
	return status !== 'down' ? status : state === 'up' ? 'lost' : state;
};

exports.default = function (bus) {
	var needHub = new _rx2.default.Subject(),
	    uiActive = _rx2.default.Observable.fromEvent(document.body, 'mousemove').startWith(true).switchMap(function () {
		return of(false).delay(2000).startWith(true);
	});

	var status = needHub.combineLatest(uiActive, function (needHub, uiActive) {
		return { needHub: needHub, uiActive: uiActive };
	}).distinctUntilChanged(_underscore_ext.isEqual).switchMap(function (_ref2) {
		var needHub = _ref2.needHub,
		    uiActive = _ref2.uiActive;
		return (
			// probably more involved than we need. ping at some base rate that's high when
			// we explicitly need a hub, and low when we don't. If user stops interacting
			// with the page, back off exponentially to a slow ping rate. 1.2 == 20% each interval.
			(0, _underscore_ext.Let)(function () {
				var base = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : needHub ? oneSecond : twentySeconds;
				return (uiActive ? interval(base) : interval(base).take(60).concat(backoffInterval(base, tenMinutes, 1.2))).startWith(true);
			})
		);
	}).switchMapTo((0, _xenaQuery.testStatus)(localHub, contactTimeout)).scan(nextState, 'down').distinctUntilChanged().share();

	status.subscribe(function (status) {
		bus.next(['localStatus', status]);
	});

	status.switchMap(function (status) {
		return (
			// Trying to distinguish websocket closed due to shut-down, vs. socket timeout:
			// retry the connection every second, unless the http port goes down.
			status === 'up' ? _rx2.default.Observable.webSocket("ws://127.0.0.1:7222/load-events").retryWhen(function (errors) {
				return errors.delay(oneSecond);
			}) : never()
		);
	}).subscribe(function (resp) {
		return bus.next(['localQueue', resp.queue]);
	});

	return {
		action: function action(state) {
			return state;
		},
		postAction: function postAction(serverBus, state, newState) {
			// Other pages? /heatmap/? /hub/?
			// Watch more closely for localHub on
			// a) hub page, if localHub selected
			// b) import page,
			// c) datapages, if on a localHub-specific page, e.g. hub or dataset
			if (newState.page === 'hub' && (0, _underscore_ext.getIn)(newState, ['spreadsheet', 'servers', localHub, 'user']) || newState.page === 'import' || newState.page === 'datapages' && (newState.params.host === localHub || (0, _underscore_ext.get)(newState.params.hub, 0) === localHub)) {
				needHub.next(true);
			} else {
				needHub.next(false);
			}
		}
	};
};