'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Adapted from redux-devtools, so we can use redux state monitors
// with xena state.
//


var _ = require('../underscore_ext');
var INIT_ACTION = { type: '@@INIT' };
var identity = function identity(state) {
	return state;
};
var notImplemented = function notImplemented(state, ac) {
	console.log('No support for ' + ac.type);
	return state;
};

var liftState = function liftState(state) {
	return { state: state };
};

function logError(err) {
	if ((typeof window === 'undefined' ? 'undefined' : (0, _typeof3.default)(window)) === 'object' && typeof window.chrome !== 'undefined') {
		// In Chrome, rethrowing provides better source map support
		setTimeout(function () {
			throw err;
		});
	} else {
		console.error(err.stack || err);
	}
}

function liftReducer(reducer) {
	return function (state, action) {
		if (state.error) {
			return _.assoc(state, 'error', 'Interrupted by an error up the chain');
		}
		try {
			return liftState(reducer(state.state, action.action.payload));
		} catch (err) {
			logError(err);
			return _.assoc(state, 'error', err.toString());
		}
	};
}

function recomputeStates(liftedReducer, devState) {
	var committedState = devState.committedState,
	    stagedActionIds = devState.stagedActionIds,
	    skippedActionIds = devState.skippedActionIds,
	    actionsById = devState.actionsById,
	    computedStates = _.reduce(stagedActionIds.slice(1), // skip INIT_ACTION
	function (acc, id) {
		return skippedActionIds.indexOf(id) !== -1 ? _.conj(acc, acc[acc.length - 1]) : _.conj(acc, liftedReducer(acc[acc.length - 1], actionsById[id]));
	}, [liftState(committedState)]);

	return (0, _extends3.default)({}, devState, { computedStates: computedStates });
}

var liftAction = function liftAction(action) {
	return { action: { type: action[0], payload: action } };
};

function controls(liftedReducer, initialCommittedState) {
	return {
		'PERFORM_ACTION': function PERFORM_ACTION(devState, _ref) {
			var action = _ref.action;
			var nextActionId = devState.nextActionId,
			    actionsById = devState.actionsById,
			    stagedActionIds = devState.stagedActionIds,
			    computedStates = devState.computedStates,
			    id = nextActionId + 1,
			    curr = computedStates[computedStates.length - 1],
			    liftedAction = liftAction(action),
			    next = liftedReducer(curr, liftedAction);


			return _.assoc(devState, 'stagedActionIds', _.conj(stagedActionIds, id), 'actionsById', _.assoc(actionsById, id, liftedAction), 'computedStates', _.conj(computedStates, next), 'nextActionId', id);
		},
		'RESET': function RESET(devState) {
			return (0, _extends3.default)({}, devState, {
				actionsById: { 0: { action: INIT_ACTION } },
				nextActionId: 1,
				stagedActionIds: [0],
				skippedActionIds: [],
				committedState: initialCommittedState,
				computedStates: [liftState(initialCommittedState)]
			});
		},
		'ROLLBACK': function ROLLBACK(devState) {
			return (0, _extends3.default)({}, devState, {
				actionsById: { 0: { action: INIT_ACTION } },
				nextActionId: 1,
				stagedActionIds: [0],
				skippedActionIds: [],
				computedStates: [devState.computedStates[0]]
			});
		},
		'COMMIT': function COMMIT(_ref2) {
			var computedStates = _ref2.computedStates,
			    devState = (0, _objectWithoutProperties3.default)(_ref2, ['computedStates']);
			return (0, _extends3.default)({}, devState, {
				actionsById: { 0: { action: INIT_ACTION } },
				nextActionId: 1,
				stagedActionIds: [0],
				skippedActionIds: [],
				committedState: computedStates[computedStates.length - 1].state,
				computedStates: [_.last(computedStates)]
			});
		},
		'SWEEP': function SWEEP(_ref3) {
			var stagedActionIds = _ref3.stagedActionIds,
			    skippedActionIds = _ref3.skippedActionIds,
			    devState = (0, _objectWithoutProperties3.default)(_ref3, ['stagedActionIds', 'skippedActionIds']);
			return recomputeStates(liftedReducer, (0, _extends3.default)({}, devState, {
				stagedActionIds: _.difference(stagedActionIds, skippedActionIds),
				skippedActionIds: []
			}));
		},
		'TOGGLE_ACTION': function TOGGLE_ACTION(_ref4, _ref5) {
			var id = _ref5.id;
			var skippedActionIds = _ref4.skippedActionIds,
			    devState = (0, _objectWithoutProperties3.default)(_ref4, ['skippedActionIds']);
			return recomputeStates(liftedReducer, skippedActionIds.indexOf(id) === -1 ? (0, _extends3.default)({}, devState, {
				skippedActionIds: _.conj(skippedActionIds, id)
			}) : (0, _extends3.default)({}, devState, {
				skippedActionIds: _.filter(skippedActionIds, function (i) {
					return i !== id;
				})
			}));
		},
		'JUMP_TO_STATE': notImplemented,
		'IMPORT_STATE': function IMPORT_STATE(devState, _ref6) {
			var nextLiftedState = _ref6.nextLiftedState;
			return recomputeStates(liftedReducer, nextLiftedState);
		}
	};
}

function instrument(controller, monitorReducer, initialState) {
	var initialDevState = {
		nextActionId: 1,
		actionsById: { 0: { action: INIT_ACTION } },
		skippedActionIds: [],
		stagedActionIds: [0],
		computedStates: [{ state: initialState }],
		committedState: initialState,
		select: function select(s) {
			return s;
		},
		monitorState: monitorReducer(undefined, {})
	},
	    liftedReducer = liftReducer(controller.action),
	    ctrls = controls(liftedReducer, initialState);

	return function (state, action) {
		var s = (ctrls[action.type] || identity)(state || initialDevState, action);
		return (0, _extends3.default)({}, s, {
			monitorState: monitorReducer(s.monitorState, action)
		});
	};
}

function createDevTools(children) {
	var monitorElement = _react.Children.only(children);
	var monitorProps = monitorElement.props;
	var Monitor = monitorElement.type;
	var enhancer = function enhancer(controller, initialState) {
		return instrument(controller, function (state, action) {
			return Monitor.update(monitorProps, state, action);
		}, initialState);
	};

	var DevTools = function (_React$Component) {
		(0, _inherits3.default)(DevTools, _React$Component);

		function DevTools() {
			(0, _classCallCheck3.default)(this, DevTools);
			return (0, _possibleConstructorReturn3.default)(this, (DevTools.__proto__ || (0, _getPrototypeOf2.default)(DevTools)).apply(this, arguments));
		}

		(0, _createClass3.default)(DevTools, [{
			key: 'render',
			value: function render() {
				return _react2.default.createElement(Monitor, (0, _extends3.default)({}, this.props, monitorProps));
			}
		}]);
		return DevTools;
	}(_react2.default.Component);

	DevTools.instrument = enhancer;
	return DevTools;
}

module.exports = {
	instrument: instrument,
	createDevTools: createDevTools
};