'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _toArray2 = require('babel-runtime/helpers/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.getLines = getLines;

require('babel-polyfill');

var _defaultServers = require('../defaultServers');

var _rx = require('../rx');

var _rx2 = _interopRequireDefault(_rx);

var _errorChecking = require('../import/errorChecking');

var _errorChecking2 = _interopRequireDefault(_errorChecking);

var _infer = require('../import/infer.js');

var _infer2 = _interopRequireDefault(_infer);

var _constants = require('../import/constants');

var _underscore_ext = require('../underscore_ext');

var _backPressure = require('../import/backPressure');

var _backPressure2 = _interopRequireDefault(_backPressure);

var _chunkReader = require('../import/chunkReader');

var _chunkReader2 = _interopRequireDefault(_chunkReader);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _Rx$Observable = _rx2.default.Observable,
    ajax = _Rx$Observable.ajax,
    create = _Rx$Observable.create,
    defer = _Rx$Observable.defer,
    empty = _Rx$Observable.empty,
    from = _Rx$Observable.from,
    fromEvent = _Rx$Observable.fromEvent,
    of = _Rx$Observable.of;

var EMPTY = empty();
var asap = _rx2.default.Scheduler.asap;
var GENOMIC_MATRIX = _constants.FILE_FORMAT.GENOMIC_MATRIX,
    CLINICAL_MATRIX = _constants.FILE_FORMAT.CLINICAL_MATRIX;

require('./html5-formdata-polyfill'); // for safari

var referenceHost = 'https://reference.xenahubs.net';

var getBaseProbemapName = function getBaseProbemapName(name) {
	return name ? name.replace(/(.*[/])/, '') : null;
};

var createMetaDataFile = function createMetaDataFile(state) {
	var probemap = getBaseProbemapName((0, _underscore_ext.getIn)(state, ['probemap', 'name']));
	return (0, _stringify2.default)({
		cohort: state[state.cohortRadio],
		dataSubType: _constants.dataSubType[state.dataType],
		type: state.fileFormat,
		assembly: state.assembly || void 0,
		probemap: probemap ? probemap : void 0
	}, null, 4);
};

var postFile = function postFile(file) {
	var payload = {
		url: _defaultServers.servers.localHub + '/upload/',
		body: file,
		responseType: 'text',
		method: 'POST',
		crossDomain: true
	};

	return ajax(payload).map(function (r) {
		return r.response;
	});
};

var updateFile = function updateFile(fileName) {
	var payload = {
		url: _defaultServers.servers.localHub + '/update/',
		body: { 'file': fileName },
		responseType: 'text',
		method: 'POST',
		crossDomain: true
	};
	return ajax(payload).map(function (r) {
		return JSON.parse(r.response);
	});
};

var readFileObs = function readFileObs(fileHandle) {
	return create(function (obs) {
		var reading = true;
		var reader = new FileReader();
		reader.onload = function (e) {
			reading = false;
			obs.next(e.target.result);
			obs.complete();
		};
		reader.onerror = function (ev) {
			return obs.error(ev.target.error);
		};
		reader.readAsBinaryString(fileHandle);
		return function () {
			return reading && reader.abort();
		};
	});
};

var matchFormat = function matchFormat(fileInfo) {
	var _fileInfo$form = fileInfo.form,
	    fileFormat = _fileInfo$form.fileFormat,
	    dataType = _fileInfo$form.dataType;

	return fileFormat === GENOMIC_MATRIX && dataType === 'phenotype' ? (0, _underscore_ext.assocIn)(fileInfo, ['form', 'fileFormat'], CLINICAL_MATRIX, ['isTransposed'], true) : fileFormat === CLINICAL_MATRIX && dataType !== 'phenotype' ? (0, _underscore_ext.assocIn)(fileInfo, ['form', 'fileFormat'], GENOMIC_MATRIX, ['isTransposed'], true) : fileInfo;
};

// Note that this will emit lines including the end-of-line markers,
// and will append empty lines to the end of the previous non-empty
// line, like 'foo\n\nbar\n' -> ['foo\n\n', 'bar\n']
//
// Have tried a dozen different ways of doing this with regexes, via split,
// match, and exec. split and match are hard to get right, without a lot of
// irrelevant complexity. exec works, but the chrome regex engine has wildly
// variable performance characteristics depending on the input.
function getLines(chunk) {
	var lines = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
	var _repeat = true;

	var _chunk, _lines;

	while (_repeat) {
		_repeat = false;

		var i = chunk.search(/[\n\r]/);
		if (i === -1) {
			return { lines: lines, remainder: chunk };
		}

		do {
			i += 1;
		} while (i < chunk.length && (chunk[i] === '\n' || chunk[i] === '\r'));

		lines.push(chunk.slice(0, i));

		_chunk = chunk.slice(i);
		_lines = lines;
		chunk = _chunk;
		lines = _lines;
		_repeat = true;
		continue;
	}
}

// Takes a stream of text buffers.
// Returns a stream of lines (synchronous).
var toLines = function toLines(chunks) {
	var rem = '';
	return chunks.concatMap(function (chunk) {
		var _getLines = getLines(chunk),
		    lines = _getLines.lines,
		    remainder = _getLines.remainder,
		    ret = [rem + lines[0]].concat((0, _toConsumableArray3.default)(lines.slice(1)));

		rem = remainder;
		return from(ret);
	}).concat(defer(function () {
		return rem.length ? of(rem) : EMPTY;
	}));
};

//var chunkSize = 40 * 1000 * 1000;
var chunkSize = 10 * 1000 * 1000;

// Take a stream of strings, returning a stream of buffers
// of size chunkSize.
var fillBuffer = function fillBuffer(seq) {
	var buff = '';
	return seq.concatMap(function (s) {
		var over = buff.length + s.length - chunkSize,
		    ret = over > 0 ? of(buff + s.slice(0, -over)) : EMPTY;
		buff = over > 0 ? s.slice(-over) : buff + s;
		return ret;
	}).concat(defer(function () {
		return buff.length ? of(buff) : EMPTY;
	}));
};

var sendFn = function sendFn(fileName) {
	return function (buffer) {
		return buffer.concatMap(function (blob, i) {
			var formData = new FormData();

			formData.append("file", blob, fileName);
			if (i !== 0) {
				formData.append('append', 'true');
			}
			return postFile(formData);
		});
	};
};

function bufferTransposedFile(fileName, file, dataType, result, metrics) {
	var count = Math.ceil(file.size / chunkSize),
	    cols = metrics.header.length,
	    rows = metrics.c0.length + 1,
	    caat = Math.min(cols, ~~(chunkSize / (file.size / cols))),
	    // # columns at a time, to read
	passes = Math.ceil(cols / caat),
	    bufferFn = function bufferFn(chunks) {
		var lines = chunks.windowCount(count)
		// should probably use a toLines method that cuts the newlines
		.concatMap(function (columnChunks) {
			return toLines(columnChunks);
		}).map(function (l, i) {
			return (0, _underscore_ext.Let)(function () {
				var pass = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : ~~(i / rows);
				return l.split(/\t/).slice(pass * caat, (pass + 1) * caat);
			});
		}).bufferCount(rows).concatMap(function (arr) {
			return arr[0].map(function (_, i) {
				return arr.map(function (a) {
					return a[i].replace(/[\r\n]+$/, '');
				}).join('\t') + '\n';
			});
		});
		// kind of silly that we join the lines here, then split them in getErrors.
		// Need to ditch the running checks in getErrors. Or expand
		// them to something more useful.
		return fillBuffer(lines.do((0, _errorChecking2.default)(dataType, result))).map(function (buff) {
			return new Blob([buff]);
		});
	};
	return (0, _backPressure2.default)(_underscore_ext.iterable.repeat(passes, (0, _chunkReader2.default)(file, chunkSize)), 2, readFileObs, bufferFn, sendFn(fileName));
}
// If sending to server is slower than reading from disk, it's better to
// compute the Blob in bufferFn (vs. sendFn), so it's immediately ready
// to send when the previous query completes.
function bufferFile(fileName, file, dataType, result) {
	var bufferFn = function bufferFn(chunks) {
		return fillBuffer(toLines(chunks).do((0, _errorChecking2.default)(dataType, result))).map(function (buff) {
			return new Blob([buff]);
		});
	};
	return (0, _backPressure2.default)((0, _chunkReader2.default)(file, chunkSize), 2, readFileObs, bufferFn, sendFn(fileName));
}

function serverError(err) {
	console.log(err);
	return {
		serverError: err.message
	};
}

// XXX fix the inference/snippet on 1st read
var importFile = function importFile(fileInfo, ignoreWarnings, metrics) {
	var _matchFormat = matchFormat(fileInfo),
	    fileName = _matchFormat.fileName,
	    file = _matchFormat.file,
	    form = _matchFormat.form,
	    isTransposed = _matchFormat.isTransposed,
	    dataType = form.dataType,
	    result = {},
	    bufferFn = isTransposed ? bufferTransposedFile : bufferFile,
	    upload = bufferFn(fileName, file, dataType, result, metrics).last().concatMap(function () {
		var formData = new FormData();
		formData.append("file", new Blob([createMetaDataFile(form)]), fileName + '.json');
		return postFile(formData);
	});

	return upload.concatMap(function () {
		return (0, _underscore_ext.Let)(function () {
			var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : result,
			    errors = _ref.errors,
			    warnings = _ref.warnings,
			    snippets = _ref.snippets;

			return errors.length ? of({ errors: errors, snippets: snippets }) : warnings.length && !ignoreWarnings ? of({ warnings: warnings }) : updateFile(fileName);
		});
	}).catch(function (error) {
		return _rx2.default.Observable.of(serverError(error));
	});
};

var download = function download(host, name) {
	return _rx2.default.Observable.ajax({
		url: host + '/download/' + name,
		responseType: 'text',
		method: 'GET',
		crossDomain: true
	});
};

var uploadProbemapFile = function uploadProbemapFile(_ref2) {
	var name = _ref2.name;

	return _rx2.default.Observable.zip(download(referenceHost, name), download(referenceHost, name + '.json')).flatMap(function (_ref3) {
		var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
		    data = _ref4[0],
		    json = _ref4[1];

		var formData = new FormData();
		var fileName = getBaseProbemapName(name);

		formData.append("file", new Blob([data.response]), fileName);
		formData.append("file", new Blob([json.response]), fileName + '.json');

		return postFile(formData).switchMapTo(updateFile(fileName));
	}).catch(function (error) {
		return _rx2.default.Observable.of({ probemapError: error.message });
	});
};

// XXX This is a bit rough, just demonstrating how we might approach incorporating
// the inferences on data format. We may get a suggestion of where the samples are
// (1st column or 1st row), an ordered list of matching probemaps (or undefined),
// and an ordered list of matching cohorts (or undefined). We stash all of these,
// and pre-populate some form fields, like cohortRadio and fileFormat. We could
// pre-poulate probemap, but due to the current probemap wizard screen it would
// require first identifying the correct radio button (gene/transcript or probe).
// We probably want to refactor that screen, instead. There's more we might
// infer, e.g. if we match probemaps, it probably isn't a segmented or mutation
// dataset, and we could drop those from the data type selection.
var inferForm = function inferForm(knownProbemaps, firstRow, firstColumn) {
	var inference = (0, _infer2.default)(firstRow, firstColumn),
	    _ref5 = inference || {},
	    samples = _ref5.samples,
	    probemaps = _ref5.probemaps,
	    cohorts = _ref5.cohorts,
	    inferredOrientation = samples === 'row' ? GENOMIC_MATRIX : undefined,
	    cohort = (0, _underscore_ext.getIn)(cohorts, [0, 'name']),
	    probemap = (0, _underscore_ext.getIn)(probemaps, [0, 'name']);


	probemap = (0, _underscore_ext.getIn)(knownProbemaps.find(function (p) {
		return (0, _underscore_ext.getIn)(p, ['value', 'name']) === probemap;
	}), ['value']);

	return {
		recommended: inference,
		form: {
			fileFormat: inferredOrientation,
			cohortRadio: cohort ? 'publicCohort' : undefined,
			publicCohort: cohort,
			probemap: probemap
		}
	};
};

// XXX copied from staticCompoents. Put in a common place.
var maxColumns = 6,
    maxNumRows = 20;

var file; // XXX local state
var metrics;

var cmds = {
	loadFile: function loadFile(probemaps, fileHandle) {
		file = fileHandle;
		return toLines(from((0, _chunkReader2.default)(fileHandle, chunkSize), asap).concatMap(readFileObs)).reduce(function (acc, line, i) {
			if (i === 0) {
				acc.header = line.replace(/[\n\r]+$/, '').split(/\t/);
				acc.snippet = [acc.header.slice(0, maxColumns)];
				acc.c0 = [];
			} else if (i < maxNumRows) {
				var c = line.split(/\t/, maxColumns);
				acc.snippet.push(c);
				acc.c0.push((0, _underscore_ext.copyStr)(c[0]));
			} else {
				var _c = line.split(/\t/, 1);
				acc.c0.push((0, _underscore_ext.copyStr)(_c[0]));
			}
			return acc;
		}, {}).do(function (acc) {
			return metrics = acc;
		}) // XXX stash for later use when loading
		.map(function (_ref6) {
			var header = _ref6.header,
			    c0 = _ref6.c0,
			    snippet = _ref6.snippet;
			return [inferForm(probemaps, header, c0), snippet];
		});
	},
	postFile: function postFile(fileName, form, localProbemaps, ignoreWarnings) {
		var params = { form: form, fileName: fileName, file: file },
		    doImport = importFile(params, ignoreWarnings, metrics);
		return form.probemap && (!(0, _underscore_ext.has)(localProbemaps, form.probemap.hash) || (0, _underscore_ext.has)(localProbemaps[form.probemap.hash], 'error')) ? uploadProbemapFile(form.probemap).switchMap(function (pm) {
			return (0, _underscore_ext.has)(pm, 'probemapError') ? _rx2.default.Observable.of(pm) : doImport;
		}) : doImport;
	}
};

// XXX catch errors here? would need to flag them somehow &
// extract enough info to be serializable.
// esp. need to handle file read errors.
var wrapSlotRequest = function wrapSlotRequest(_ref7) {
	var _ref7$msg = (0, _toArray3.default)(_ref7.msg),
	    tag = _ref7$msg[0],
	    args = _ref7$msg.slice(1),
	    id = _ref7.id;

	return (
		// using defer so we can capture any errors thrown in the handlers
		defer(function () {
			return cmds[tag].apply(cmds, (0, _toConsumableArray3.default)(args));
		}).catch(function (err) {
			return of(serverError(err));
		}).map(function (msg) {
			return { msg: msg, id: id };
		})
	);
};

var recvQ = fromEvent(self, 'message').map(function (_ref8) {
	var data = _ref8.data;
	return data;
});

recvQ.groupBy(function (_ref9) {
	var _ref9$msg = (0, _slicedToArray3.default)(_ref9.msg, 1),
	    tag = _ref9$msg[0];

	return tag;
}).map(function (g) {
	return g.switchMap(wrapSlotRequest);
}) // cancel old requests on same method
.mergeAll().subscribe(function (ev) {
	return postMessage(ev);
});