'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _rx = require('../rx');

var _rx2 = _interopRequireDefault(_rx);

var _utils = require('./utils');

var _defaultServers = require('../defaultServers');

var _xenaQuery = require('../xenaQuery');

var _underscore_ext = require('../underscore_ext');

var _importWorker = require('worker-loader!./import-worker');

var _importWorker2 = _interopRequireDefault(_importWorker);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var loaderSocket = _rx2.default.Observable.webSocket("ws://localhost:7222/load-events");

var watchQueueFor = function watchQueueFor(file) {
    return loaderSocket.takeWhile(function (_ref) {
        var queue = _ref.queue;
        return queue.find(function (_ref2) {
            var _ref3 = (0, _slicedToArray3.default)(_ref2, 1),
                name = _ref3[0];

            return name === file;
        });
    }).last(null, null, 'done');
};

var worker = new _importWorker2.default();

// XXX error handling? What do we do with errors in the worker?
// For import, we collect & report, so in theory there aren't any.
// For read file, unclear.. what if there's a permissions error,
// or something? Is this possible?
var workerObs = _rx2.default.Observable.fromEvent(worker, 'message').share();
var msgId = 0;

// sendMessage wraps worker messages in ajax-like observables, by assigning
// unique ids to each request, and waiting for a single response with the
// same id. The worker must echo the id in the response.
var sendMessage = function sendMessage(msg) {
    var id = msgId++;
    worker.postMessage({ msg: msg, id: id });
    return workerObs.filter(function (ev) {
        return ev.data.id === id;
    }).take(1).map(function (ev) {
        return ev.data.msg;
    });
};

var referenceHost = 'https://reference.xenahubs.net',
    defaultStudyName = 'My Study';

var getDefaultState = function getDefaultState() {
    return {
        wizardPage: 0,
        wizardHistory: []
    };
};

var getDefaultCustomCohort = function getDefaultCustomCohort(localCohorts) {
    var name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultStudyName;
    var number = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
    var _repeat = true;

    var _localCohorts, _name, _number;

    while (_repeat) {
        _repeat = false;

        if (!localCohorts.includes(name)) {
            return name;
        } else {
            _localCohorts = localCohorts;
            _name = defaultStudyName + ' (' + number + ')';
            _number = ++number;
            localCohorts = _localCohorts;
            name = _name;
            number = _number;
            _repeat = true;
            continue;
        }
    }
};

var importFileDone = function importFileDone(state, result) {
    return (0, _underscore_ext.updateIn)(state, ['form'], function (form) {
        return (0, _underscore_ext.assoc)(form, 'errorCheckInprogress', false, 'errors', (0, _underscore_ext.get)(result, 'errors', []), 'warnings', (0, _underscore_ext.get)(result, 'warnings', []), 'errorSnippets', (0, _underscore_ext.get)(result, 'snippets', []), 'serverError', (0, _underscore_ext.get)(result, 'serverError', undefined), 'probemapError', (0, _underscore_ext.get)(result, 'probemapError', undefined));
    });
};

var fileQueued = function fileQueued(fileName, queue) {
    return (0, _underscore_ext.isArray)(queue) && (0, _underscore_ext.get)((0, _underscore_ext.last)(queue), 0) === fileName;
};

var readFileDone = function readFileDone(state, result) {
    return (0, _underscore_ext.has)(result, 'serverError') ? (0, _underscore_ext.assocIn)(state, ['form', 'serverError'], result.serverError) : (0, _underscore_ext.Let)(function () {
        var _ref4 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : result,
            _ref5 = (0, _slicedToArray3.default)(_ref4, 2),
            _ref5$ = _ref5[0],
            recommended = _ref5$.recommended,
            form = _ref5$.form,
            fileContent = _ref5[1];

        return (0, _extends3.default)({}, state, { fileContent: fileContent, recommended: recommended, fileReadInProgress: false, form: (0, _extends3.default)({}, state.form, form) });
    });
};

// XXX handle 'inference'
var retryFileDone = function retryFileDone(state, _ref6) {
    var _ref7 = (0, _slicedToArray3.default)(_ref6, 2),
        _ref7$ = (0, _slicedToArray3.default)(_ref7[0], 2),
        _ref7$$ = _ref7$[0],
        recommended = _ref7$$.recommended,
        form = _ref7$$.form,
        fileContent = _ref7$[1],
        errors = _ref7[1];

    var stateWithInference = (0, _underscore_ext.assocInAll)(state, ['recommended'], recommended, ['retryForm'], form);
    return importFileDone((0, _underscore_ext.assocInAll)(stateWithInference, ['fileContent'], fileContent), errors);
};

var onRetryMetaData = function onRetryMetaData(state) {
    var retryForm = (0, _underscore_ext.getIn)(state, ['retryForm']) || {};
    retryForm = (0, _extends3.default)({}, retryForm, {
        newCohort: getDefaultCustomCohort((0, _underscore_ext.getIn)(state, ['localCohorts']))
    });
    return (0, _underscore_ext.assocInAll)(state, ['form'], retryForm, ['retryForm'], undefined);
};

function parseServerErrors(resp) {
    var _resp = (0, _slicedToArray3.default)(resp, 1),
        _resp$ = _resp[0],
        status = _resp$.status,
        text = _resp$.text,
        _JSON$parse = JSON.parse(text),
        error = _JSON$parse.error;

    return status === 'loaded' ? {} : {
        serverError: error || 'Unknown server error'
    };
}

// emits error result in form {error, serverError, ...}
var postFile = function postFile(_ref8, ignoreWarnings) {
    var fileName = _ref8.fileName,
        form = _ref8.form,
        localProbemaps = _ref8.localProbemaps;
    return sendMessage(['postFile', fileName, form, localProbemaps, ignoreWarnings]).flatMap(function (result) {
        return fileQueued(fileName, result) ? watchQueueFor(fileName).switchMapTo((0, _xenaQuery.datasetStatus)(_defaultServers.servers.localHub, fileName)).map(parseServerErrors) : _rx2.default.Observable.of(result);
    });
};

var getCohortArray = function getCohortArray(cohorts) {
    return cohorts.map(function (c) {
        return c.cohort;
    });
};
var getValueLabelList = function getValueLabelList(items) {
    return items.map(function (item) {
        return { label: item.label, value: { name: item.name, hash: item.hash } };
    });
};

var fetchLocalProbemaps = function fetchLocalProbemaps(serverBus) {
    return serverBus.next(['set-local-probemaps', (0, _xenaQuery.probemapList)(_defaultServers.servers.localHub)]);
};

//var t0;
var importControls = {
    // XXX Important: this controller is stateful, due to maintaining a
    // web worker.
    'file': function file(state, fileHandle) {
        return (0, _underscore_ext.assocInAll)(state, ['file'], fileHandle, ['fileName'], fileHandle.name);
    },
    'import-file-post!': function importFilePost(serverBus, state, newState) {
        //		t0 = Date.now();
        serverBus.next(['import-file-done', postFile(newState)]);
    },

    // Merge in any errors & status from the file save.
    'import-file-done': importFileDone,
    // on dataset error, we may have loaded a probemap
    'import-file-done-post!': function importFileDonePost(serverBus, state, newState) {
        fetchLocalProbemaps(serverBus, state, newState);
        //		console.log(`load time ${(Date.now() - t0) / 1000} sec`);
    },
    'read-file': function readFile(state) {
        return (0, _underscore_ext.assocIn)(state, ['form', 'serverError'], undefined);
    },
    'read-file-post!': function readFilePost(serverBus, state, newState, fileHandle) {
        return serverBus.next(['read-file-done', sendMessage(['loadFile', newState.probemaps, fileHandle])]);
    },
    'file-read-inprogress': function fileReadInprogress(state) {
        return (0, _underscore_ext.assocIn)(state, ['fileReadInProgress'], true);
    },
    // XXX add spinner to file snippet & make sure back/forward make
    // sense while waiting on this.
    'read-file-done': readFileDone,
    'set-status': function setStatus(state, status) {
        return (0, _underscore_ext.assocIn)(state, ['status'], status);
    },
    'wizard-page': function wizardPage(state, newPage) {
        return (0, _underscore_ext.assocIn)(state, ['wizardPage'], newPage);
    },
    'wizard-page-history': function wizardPageHistory(state, newHistory) {
        return (0, _underscore_ext.assocIn)(state, ['wizardHistory'], newHistory);
    },
    'file-content': function fileContent(state, content) {
        return (0, _underscore_ext.assocIn)(state, ['fileContent'], content);
    },
    'clear-metadata': function clearMetadata(state) {
        return (0, _underscore_ext.assocIn)(state, ['form'], {});
    },
    // XXX form state is probably messed up at this point. Need a way
    // to merge in new inference when file changes.
    'retry-file-post!': function retryFilePost(serverBus, state, newState, fileHandle) {
        return serverBus.next(['retry-file-done', sendMessage(['loadFile', newState.probemaps, fileHandle]).flatMap(function (newFile) {
            return postFile(newState).map(function (errors) {
                return [newFile, errors];
            });
        })]);
    },
    'retry-file-done': retryFileDone,
    // on dataset error, we may have loaded a probemap
    'retry-file-done-post!': fetchLocalProbemaps,
    'retry-meta-data': onRetryMetaData,
    'reset-import-state': function resetImportState(state) {
        return getDefaultState(state);
    },
    'load-with-warnings-post!': function loadWithWarningsPost(serverBus, state, newState) {
        return serverBus.next(['import-file-done', postFile(newState, true)]);
    }
};

var query = {
    'get-local-cohorts-post!': function getLocalCohortsPost(serverBus) {
        return serverBus.next(['local-cohorts', (0, _xenaQuery.cohortSummary)(_defaultServers.servers.localHub, [])]);
    },
    'local-cohorts': function localCohorts(state, cohorts) {
        var localCohorts = getCohortArray(cohorts);
        return (0, _underscore_ext.assocInAll)(state, ['localCohorts'], localCohorts, ['form', 'newCohort'], getDefaultCustomCohort(localCohorts));
    },
    'get-probemaps-post!': function getProbemapsPost(serverBus) {
        serverBus.next(['set-probemaps', (0, _xenaQuery.probemapList)(referenceHost)]);
        fetchLocalProbemaps(serverBus);
    },
    'set-probemaps': function setProbemaps(state, probemaps) {
        return (0, _underscore_ext.assocIn)(state, ['probemaps'], getValueLabelList(probemaps));
    },
    'set-local-probemaps': function setLocalProbemaps(state, probemaps) {
        return (0, _underscore_ext.assoc)(state, 'localProbemaps', (0, _underscore_ext.object)((0, _underscore_ext.pluck)(probemaps, 'hash'), probemaps));
    }
};

var changeFormProp = function changeFormProp(propName) {
    return function (state, propValue) {
        return (0, _underscore_ext.assocIn)(state, ['form', propName], propValue);
    };
};

var formControls = {
    'file-format': changeFormProp('fileFormat'),
    'data-type': changeFormProp('dataType'),
    'cohort-radio': changeFormProp('cohortRadio'),
    'import-publicCohort': changeFormProp('publicCohort'),
    'import-localCohort': changeFormProp('localCohort'),
    'import-newCohort': changeFormProp('newCohort'),
    'errors': changeFormProp('errors'),
    'probemap': changeFormProp('probemap'),
    'assembly': changeFormProp('assembly'),
    // We can only set this 'true' from the UI. It is reset from an action handler, after import.
    'error-check-inprogress': function errorCheckInprogress(state) {
        return (
            // importFileDone will clear the error state if no errors are passed for the second arg.
            (0, _underscore_ext.assocIn)(importFileDone(state), ['form', 'errorCheckInprogress'], true)
        );
    }
};

exports.default = (0, _utils.mount)((0, _utils.compose)((0, _utils.make)(importControls), (0, _utils.make)(query), (0, _utils.make)(formControls)), ['import']);