'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _toArray2 = require('babel-runtime/helpers/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

exports.default = source;

var _underscore_ext = require('../underscore_ext');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// syncs data in memory with data on server, as a function of app state.
// dataRequest is a fn of state that returns a set of paths that should be
// synced.
// fetchMethods is an object with fetch method per top-level path key.
// cachePolicy is an object with transform per top-level path key, that
// can be used to drop unused state. It is applied after merging newly
// synced data.
// mount is a top-level key under which to merge the synced data in the
// application state.
//
// returns a controller, and an imperative cache invalidation method.
function source(fetchMethods, dataRequest, cachePolicy, mount) {
	var _controls;

	// To make this more general, would need to follow the path until
	// we hit a dispatch method, then pass args.
	// XXX should this inject a common 'error' value, if the query fails?
	// We would need to update some views where we currently map to empty
	// array on error.
	var fetchData = function fetchData(_ref) {
		var _ref2 = (0, _toArray3.default)(_ref),
		    type = _ref2[0],
		    args = _ref2.slice(1);

		return fetchMethods[type].apply(fetchMethods, (0, _toConsumableArray3.default)(args));
	};
	var dispatchKey = mount + '-merge-data';

	// XXX Local mutatable state. The effects controller is stateful wrt
	// data queries.
	var queue = [];
	var outOfDate = {};

	var clearCache = function clearCache(fn) {
		return function (state, path, data) {
			return (cachePolicy[path[0]] || cachePolicy.default)(fn(state, path, data), path);
		};
	};

	var enforceValue = function enforceValue(path, val) {
		if (val == null) {
			// Fetch methods must return a value besides null or undefined. Otherwise
			// this will create a request loop, where we fetch again because we can't
			// tell the data has already been fetched.
			console.error('Received invalid response for path ' + path);
			return { error: 'invalid value' };
		}
		return val;
	};

	function invalidatePath(state, pattern) {
		(0, _underscore_ext.matchKeys)(state, pattern).forEach(function (path) {
			outOfDate = (0, _underscore_ext.assocIn)(outOfDate, path, true);
		});
	}

	function validatePath(path) {
		outOfDate = (0, _underscore_ext.updateIn)(outOfDate, (0, _underscore_ext.initial)(path), function (p) {
			return p && (0, _underscore_ext.dissoc)(p, (0, _underscore_ext.last)(path));
		});
	}

	var controls = (_controls = {}, (0, _defineProperty3.default)(_controls, dispatchKey, clearCache(function (state, path, data) {
		return (0, _underscore_ext.assocIn)(state, [mount].concat((0, _toConsumableArray3.default)(path)), enforceValue(path, data));
	})), (0, _defineProperty3.default)(_controls, dispatchKey + '-post!', function undefined(serverBus, state, newState, path) {
		var i = queue.findIndex(function (p) {
			return (0, _underscore_ext.isEqual)(p, path);
		});
		queue.splice(i, 1);
		validatePath(path);
	}), _controls);

	var postAction = function postAction(serverBus, state, newState) {
		var toFetch = dataRequest(newState).filter(function (path) {
			return ((0, _underscore_ext.getIn)(newState[mount], path) == null || (0, _underscore_ext.getIn)(outOfDate, path)) && !(0, _underscore_ext.find)(queue, function (p) {
				return (0, _underscore_ext.isEqual)(p, path);
			});
		});

		toFetch.forEach(function (path) {
			queue.push(path);
			serverBus.next([[dispatchKey, path], fetchData(path)]);
		});
	};

	var fetchController = {
		action: _underscore_ext.identity,
		postAction: postAction
	};

	return {
		controller: (0, _utils.compose)(fetchController, (0, _utils.make)(controls)),
		invalidatePath: invalidatePath
	};
}