'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _columns = require('../models/columns');

var _searchSamples = require('../models/searchSamples');

var _rx = require('../rx');

var Rx = _interopRequireWildcard(_rx);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('../underscore_ext');

var _require = require('./common'),
    resetZoom = _require.resetZoom,
    fetchColumnData = _require.fetchColumnData,
    fetchCohortData = _require.fetchCohortData,
    setCohort = _require.setCohort,
    fetchClustering = _require.fetchClustering;

var _require2 = require('./utils'),
    compose = _require2.compose,
    make = _require2.make,
    mount = _require2.mount;

var uuid = require('../uuid');

var columnOpen = function columnOpen(state, id) {
	return _.has(_.get(state, 'columns'), id);
};

var resetLoadPending = function resetLoadPending(state) {
	return _.dissoc(state, 'loadPending');
};

var setSamples = function setSamples(state, _ref) {
	var samples = _ref.samples,
	    over = _ref.over,
	    hasPrivateSamples = _ref.hasPrivateSamples;
	return _.assoc(state, 'cohortSamples', samples, 'samplesOver', over, 'hasPrivateSamples', hasPrivateSamples);
};

var setLoading = function setLoading(state) {
	return _.reduce(state.columnOrder, function (acc, id) {
		return _.assocIn(acc, ['data', id, 'status'], 'loading');
	}, state);
};

// XXX put this in a central location, ideally behind a memoizer,
// and accessible from the reducers. Would be best to ditch dsID.
var datasetsByID = function datasetsByID(state) {
	var cohort = _.get(state.spreadsheet.cohort, 'name'),
	    cohortDatasets = _.get(state.wizard.cohortDatasets, cohort, {}),
	    allDS = _.flatmap(cohortDatasets, _.identity);
	return _.object(_.pluck(allDS, 'dsID'), allDS);
};

// XXX put this in a central location, ideally behind a memoizer,
// and accessible from the reducers. Would be best to ditch dsID.
var featuresByID = function featuresByID(state) {
	var cohort = _.get(state.spreadsheet.cohort, 'name'),
	    features = _.get(state.wizard.cohortFeatures, cohort, {}),
	    all = _.flatmap(features, function (datasets, host) {
		return _.map(datasets, function (fields, name) {
			return [(0, _stringify2.default)({ host: host, name: name }), fields];
		});
	});
	return _.object(all);
};

var setHeatmapParams = function setHeatmapParams(state) {
	return state.params.heatmap ? _.updateIn(state, ['spreadsheet'], function (spreadsheet) {
		return _.assocIn.apply(_, [spreadsheet].concat((0, _toConsumableArray3.default)(state.params.heatmap.flatten())));
	}) : state;
};

var linkedColumns = function linkedColumns(state, _ref2) {
	var _ref3 = (0, _slicedToArray3.default)(_ref2, 2),
	    fieldResp = _ref3[0],
	    ids = _ref3[1];

	var columnParams = state.params.columns,
	    maybeResetLoadPending = _.getIn(state, ['params', 'filter']) ? _.identity : resetLoadPending,
	    byID = datasetsByID(state),
	    features = featuresByID(state),
	    columns = _.mmap(columnParams, fieldResp, function (column, matches) {
		return _.Let(function () {
			var dsID = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _stringify2.default)({ host: column.host, name: column.name });
			return (0, _columns.computeSettings)(byID, features, column.opts, dsID, matches);
		});
	});

	return _.reduce(columns, function (acc, spec, i) {
		return _.assocIn(acc, ['spreadsheet', 'columns', ids[i]], spec);
	}, _.updateIn(maybeResetLoadPending(setHeatmapParams(state)), ['spreadsheet'], function (s) {
		return _.dissoc(s, 'fieldMatches');
	}, ['spreadsheet', 'columnOrder'], function (o) {
		return o.concat(ids);
	}, ['spreadsheet', 'wizardMode'], function () {
		return false;
	}));
};

// true if we have a dataset list for every linked host, or every
// server in state.
var datasetsLoaded = function datasetsLoaded(state) {
	var cohortDatasets = _.get(state.wizard.cohortDatasets, _.get(state.spreadsheet.cohort, 'name'), {}),
	    servers = state.spreadsheet.servers,
	    dsServers = _.pluck(state.params.columns, 'host');
	return dsServers.every(function (s) {
		return cohortDatasets[s];
	}) || (0, _keys2.default)(servers).every(function (s) {
		return !servers[s].user || cohortDatasets[s];
	});
};

// true if we have all the features loaded. Assumes that all
// dataset meta is loaded.
var featuresLoaded = function featuresLoaded(state) {
	var cohort = _.get(state.spreadsheet.cohort, 'name'),
	    byID = datasetsByID(state),
	    required = _.flatmap(state.params.columns, function (column) {
		var ds = byID[(0, _stringify2.default)({ host: column.host, name: column.name })];
		return ds && ds.type === 'clinicalMatrix' ? [column] : [];
	}),
	    loaded = _.get(state.wizard.cohortFeatures, cohort, {});

	return _.every(required, function (d) {
		return _.has(loaded, d.host) && _.has(loaded[d.host], d.name);
	});
};

var filterDatasets = function filterDatasets(state) {
	var cohortDatasets = _.get(state.wizard.cohortDatasets, _.get(state.spreadsheet.cohort, 'name'), {});
	return _.updateIn(state, ['params', 'columns'], function (columns) {
		return columns.filter(function (_ref4) {
			var host = _ref4.host,
			    name = _ref4.name;
			return _.has(cohortDatasets, host) && // note: should be true, since we add all linked hosts
			_.find(cohortDatasets[host], function (ds) {
				return ds.name === name;
			});
		});
	});
};

var dsMsg = function dsMsg(badDataset) {
	return badDataset.length ? ' These datasets are not known: ' + badDataset.map(function (_ref5) {
		var host = _ref5.host,
		    name = _ref5.name;
		return host + '/' + name;
	}).join(', ') + '.' : '';
};

var datasetError = function datasetError(badDataset) {
	return 'We are unable to load some columns.' + dsMsg(badDataset);
};

// if loadPending & we have columns, only load when
// when we have datasets & features
var shouldLoadColumns = function shouldLoadColumns(state) {
	return state.loadPending && _.getIn(state, ['params', 'columns']) && datasetsLoaded(state) && featuresLoaded(state);
};

var setLinkedCohortError = function setLinkedCohortError(state) {
	return _.assoc(resetLoadPending(state), 'stateError', 'We are unable to find the cohort for dataset ' + state.params.columns[0].name + ' on host ' + state.params.columns[0].host + '. Please check the spelling, and your network connectivity.');
};

var dropFilter = function dropFilter(state) {
	return _.updateIn(state, ['params'], function (p) {
		return _.omit(p, 'filter', 'filterColumns', 'visible');
	});
};

var dropHidden = function dropHidden(state, visible) {
	return _.updateIn(state, ['spreadsheet'], function (spreadsheet) {
		var columns = spreadsheet.columns,
		    columnOrder = spreadsheet.columnOrder,
		    data = spreadsheet.data,
		    newOrder = columnOrder.slice(0, visible);

		return _.assoc(spreadsheet, 'columns', _.pick(columns, newOrder), 'data', _.pick(data, newOrder), 'columnOrder', newOrder);
	});
};

var mergeWidgetData = function mergeWidgetData(state, id, data) {
	return columnOpen(state, id) ? _.assocIn(state, ["data", id], _.assoc(data, 'status', 'loaded')) : state;
};

var controls = {
	// XXX was clearWizardCohort just cleaning up bad bookmarks? Do we need to handle that case?
	// How do we handle cohortSamples reloading generally, and with respect to restoring state?
	bookmark: function bookmark(state, _bookmark) {
		return resetLoadPending(_.merge(state, _bookmark));
	},
	'bookmark-error': function bookmarkError(state) {
		return resetLoadPending(_.assoc(state, 'stateError', 'bookmark'));
	},
	// Here we need to load cohort data if servers or cohort has changed,
	// *or* if we never loaded cohort data (e.g. due to waiting on bookmark).
	'manifest': function manifest(state, _ref6) {
		var cohort = _ref6.cohort,
		    samples = _ref6.samples;
		return _.updateIn(state, ['spreadsheet'], setCohort({ name: cohort, sampleFilter: samples }, null));
	},
	'manifest-post!': function manifestPost(serverBus, state, newState) {
		return (
			// just samples now
			fetchCohortData(serverBus, newState.spreadsheet)
		);
	},
	inlineState: function inlineState(state, newState) {
		return resetLoadPending(newState);
	},
	samples: function samples(state, resp) {
		var fieldMatches = _.getIn(state, ['spreadsheet', 'fieldMatches']),
		    maySetLoading = !state.loadPending || fieldMatches ? setLoading : _.identity,
		    maySetColumns = state.loadPending && fieldMatches ? linkedColumns : _.identity;
		return _.updateIn(maySetColumns(state, fieldMatches), ['spreadsheet'], function (state) {
			return maySetLoading(resetZoom(setSamples(state, resp)));
		});
	},
	'samples-post!': function samplesPost(serverBus, state, newState, _ref7) {
		var samples = _ref7.samples;

		var fieldMatches = _.getIn(state, ['spreadsheet', 'fieldMatches']);
		if (!state.loadPending || fieldMatches) {
			_.mapObject(_.get(newState.spreadsheet, 'columns', {}), function (settings, id) {
				return fetchColumnData(serverBus, samples, id, settings);
			});
		}
	},
	'wizard-merge-data': function wizardMergeData(state) {
		if (shouldLoadColumns(state)) {
			var cohortDatasets = _.get(state.wizard.cohortDatasets, _.get(state.spreadsheet.cohort, 'name'), {}),

			// Note we implicitly add all hubs in a linked column list,
			// so the hubs should all be known.
			badDataset = state.params.columns.filter(function (_ref8) {
				var host = _ref8.host,
				    name = _ref8.name;
				return _.has(cohortDatasets, host) && !_.find(cohortDatasets[host], function (ds) {
					return ds.name === name;
				});
			}),
			    next = filterDatasets(badDataset.length ? _.assoc(state, 'stateError', datasetError(badDataset)) : state);

			return next.params.columns.length > 0 ? next : resetLoadPending(state);
		}
		return state;
	},
	'wizard-merge-data-post!': function wizardMergeDataPost(serverBus, state, newState) {
		if (shouldLoadColumns(newState)) {
			var _Rx$Observable;

			// This is just unbelievable. Should be much simpler.
			var byId = datasetsByID(newState),
			    columns = newState.params.columns;
			serverBus.next(['columns-match-fields', (_Rx$Observable = Rx.Observable).zip.apply(_Rx$Observable, (0, _toConsumableArray3.default)(columns.map(function (c) {
				return _.Let(function () {
					var dsID = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _stringify2.default)({ host: c.host, name: c.name });
					return (0, _columns.matchDatasetFields)(byId, dsID, c.fields);
				});
			}))),
			// Note that uuid is a side-effect, and can't be run in a reducer.
			_.times(columns.length, uuid)]);
		}
	},
	'columns-match-fields': function columnsMatchFields(state, resp, ids) {
		return !_.isEmpty(_.getIn(state, ['spreadsheet', 'cohortSamples'])) ? _.updateIn(linkedColumns(state, [resp, ids]), ['spreadsheet'], setLoading) : _.assocIn(state, ['spreadsheet', 'fieldMatches'], [resp, ids]);
	},
	'columns-match-fields-post!': function columnsMatchFieldsPost(serverBus, state, newState) {
		if (!_.isEmpty(_.getIn(state, ['spreadsheet', 'cohortSamples']))) {
			_.mapObject(_.get(newState.spreadsheet, 'columns', {}), function (settings, id) {
				return fetchColumnData(serverBus, newState.spreadsheet.cohortSamples, id, settings);
			});
		}
	},
	'dataset-cohort': function datasetCohort(state, resp) {
		var cohort = _.getIn(resp, [0, 'cohort']);
		return cohort ? _.updateIn(state, ['spreadsheet'], function (state) {
			return setCohort({ name: cohort }, undefined, state);
		}) : setLinkedCohortError(state);
	},
	'dataset-cohort-error': setLinkedCohortError,
	'dataset-cohort-post!': function datasetCohortPost(serverBus, state, newState, resp) {
		var cohort = _.getIn(resp, [0, 'cohort']);
		if (cohort) {
			fetchCohortData(serverBus, newState.spreadsheet);
		}
	},
	'widget-data': function widgetData(state) {
		var filter = _.getIn(state, ['params', 'filter']);
		if (filter && _.every(state.spreadsheet.data, function (d) {
			return d.status === 'loaded';
		})) {
			var visible = _.getIn(state, ['params', 'visible']);
			return resetLoadPending(dropFilter(dropHidden(state, visible)));
		}
		return state;
	},
	'widget-data-post!': function widgetDataPost(serverBus, state, newState, id, widgetData) {
		var filter = _.getIn(state, ['params', 'filter']);
		// This is weird. We re-compute the merged widget data, in case we
		// discarded it as 'hidden', above.
		var merged = mergeWidgetData(state.spreadsheet, id, widgetData);
		if (filter && _.every(merged.data, function (d) {
			return d.status === 'loaded';
		})) {
			var columns = merged.columns,
			    columnOrder = merged.columnOrder,
			    data = merged.data,
			    cohortSamples = merged.cohortSamples;

			var matches = (0, _searchSamples.searchSamples)(filter, columns, columnOrder, data, cohortSamples);
			// XXX no support for cross. Always take the first.
			var nextSamples = matches.matches[0].map(function (i) {
				return cohortSamples[i];
			});
			serverBus.next(['sampleFilter', Rx.Observable.of(nextSamples, Rx.Scheduler.async)]);
		}
	}
};

var spreadsheetControls = {
	// XXX Here we drop the update if the column is no longer open.
	'widget-data': mergeWidgetData,
	'widget-data-post!': function widgetDataPost(serverBus, state, newState, id) {
		if (_.getIn(newState, ['columns', id, 'clustering']) != null) {
			fetchClustering(serverBus, newState, id);
		}
	},
	'cluster-result': function clusterResult(state, id, order) {
		return _.assocIn(state, ['data', id, 'clustering', 'probes'], order);
	},
	'widget-data-error': function widgetDataError(state, id) {
		return columnOpen(state, id) ? _.assocIn(state, ["data", id, 'status'], 'error') : state;
	},
	'km-survival-data': function kmSurvivalData(state, survival) {
		return _.assoc(state, 'survival', survival);
	},
	// XXX Here we should be updating application state. Instead we invoke a callback, because
	// chart.js can't handle passed-in state updates.
	'chart-average-data-post!': function chartAverageDataPost(serverBus, state, newState, offsets, thunk) {
		return thunk(offsets);
	}
};

module.exports = compose(make(controls), mount(make(spreadsheetControls), ['spreadsheet']));