'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _toArray2 = require('babel-runtime/helpers/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('../underscore_ext');
var xenaQuery = require('../xenaQuery');
var Rx = require('../rx');

// the expression dataset and phenodataset must be on the same host
var expressionHost = 'https://kidsfirst.xenahubs.net';
//var expressionHost = 'https://toil.xenahubs.net';

/*
//basic gencode annotataion
var transcriptDataset = {
	host: 'https://reference.xenahubs.net',
	name: 'wgEncodeGencodeBasicV23'
};
*/

// comprehensive gencode annotataion -- TOIL run was using the v23 comprehensive gencode annotataion
var transcriptDataset = {
	host: 'https://reference.xenahubs.net',
	name: 'wgEncodeGencodeCompV23'
};

var expressionDataset = {
	tpm: "TCGA_target_GTEX_KF/rsem.isoforms_TPM.txt",
	isoformPercentage: "TCGA_target_GTEX_KF/rsem.isoforms_IsoPct.txt"
	//	tpm: "TcgaTargetGtex_rsem_isoform_tpm",
	//	isoformPercentage: "TcgaTargetGtex_rsem_isopct"
};

var subtypeDataset = 'transcript_view/transcript_main_category.txt';
var subtypeField = 'transcript_view_category';
//var subtypeDataset = 'TCGA_GTEX_category.txt';
//var subtypeField = 'TCGA_GTEX_main_category';

var identity = function identity(x) {
	return x;
};

// The transcriptExpression query is a bit wonky, because we're pulling the subtypes from TCGA_GTEX_main_category,
// but in transcriptExpression we filter on _sample_type. This assumes the two are kept in sync. It would be
// better to use the same field in both places, but first we need to confirm that it will work.
var fetchExpression = function fetchExpression(transcripts, _ref) {
	var subtypeA = _ref.subtypeA,
	    subtypeB = _ref.subtypeB,
	    unit = _ref.unit;
	return xenaQuery.transcriptExpression(expressionHost,
	// adding 1 for the space after the prefix
	_.pluck(transcripts, 'name'), subtypeA, subtypeB, expressionDataset[unit], subtypeDataset, subtypeField).map(function (_ref2) {
		var _ref3 = (0, _slicedToArray3.default)(_ref2, 2),
		    expsA = _ref3[0],
		    expsB = _ref3[1];

		return _.mmap(expsA, expsB, transcripts, function (expA, expB, transcript) {
			return _.assoc(transcript, 'expA', expA, 'expB', expB);
		});
	});
};

function fetchTranscripts(serverBus, params) {
	var host = transcriptDataset.host,
	    name = transcriptDataset.name,
	    query = Rx.Observable.zip(xenaQuery.geneTranscripts(host, name, params.gene).flatMap(function (transcripts) {
		return fetchExpression(transcripts, params);
	}), xenaQuery.datasetMetadata(expressionHost, expressionDataset[params.unit]), function (transcripts, _ref4) {
		var _ref5 = (0, _slicedToArray3.default)(_ref4, 1),
		    meta = _ref5[0];

		return { transcripts: transcripts, unit: meta.unit };
	});


	serverBus.next(['geneTranscripts', query]);
}

function filterSubtypes(subtypes) {
	return subtypes[subtypeField].sort();
}

function fetchSubtypes(serverBus) {
	serverBus.next(['transcriptSampleSubtypes', xenaQuery.fieldCodes(expressionHost, subtypeDataset, [subtypeField]).map(filterSubtypes)]);
}

var controls = {
	'init-post!': function initPost(serverBus, state, newState) {
		var _ref6 = newState.transcripts || {},
		    gene = _ref6.gene,
		    subtypeA = _ref6.subtypeA,
		    status = _ref6.status;

		fetchSubtypes(serverBus);
		if ((status === 'loading' || status === 'error') && gene && subtypeA) {
			fetchTranscripts(serverBus, newState.transcripts);
		}
	},
	loadGene: function loadGene(state, gene, subtypeA, subtypeB, unit) {
		var zoom = state.transcripts && gene === state.transcripts.gene ? state.transcripts.zoom : {};
		return _.updateIn(state, ['transcripts'], function (s) {
			return _.merge(s, {
				status: gene ? 'loading' : undefined,
				gene: gene,
				subtypeA: subtypeA,
				subtypeB: subtypeB,
				unit: unit,
				zoom: zoom });
		});
	},
	'loadGene-post!': function loadGenePost(serverBus, state, newState) {
		if (newState.transcripts.gene) {
			fetchTranscripts(serverBus, newState.transcripts);
		}
	},
	geneTranscripts: function geneTranscripts(state, _ref7) {
		var transcripts = _ref7.transcripts,
		    unit = _ref7.unit;
		return _.assocIn(state, ['transcripts', 'status'], 'loaded', ['transcripts', 'genetranscripts'], transcripts, ['transcripts', 'datasetUnit'], unit);
	},
	'geneTranscripts-error': function geneTranscriptsError(state) {
		return _.assocIn(state, ['transcripts', 'status'], 'error');
	},
	transcriptSampleSubtypes: function transcriptSampleSubtypes(state, subtypes) {
		return _.assocIn(state, ['transcripts', 'subtypes'], subtypes);
	},
	units: function units(state, _units) {
		return _.assocIn(state, ['transcripts', 'units'], _units);
	},
	transcriptZoom: function transcriptZoom(state, name) {
		return _.updateIn(state, ['transcripts', 'zoom', name], function (z) {
			return !z;
		});
	}
};

module.exports = {
	action: function action(state, _ref8) {
		var _ref9 = (0, _toArray3.default)(_ref8),
		    tag = _ref9[0],
		    args = _ref9.slice(1);

		return (controls[tag] || identity).apply(undefined, [state].concat((0, _toConsumableArray3.default)(args)));
	},
	postAction: function postAction(serverBus, state, newState, _ref10) {
		var _ref11 = (0, _toArray3.default)(_ref10),
		    tag = _ref11[0],
		    args = _ref11.slice(1);

		return (controls[tag + '-post!'] || identity).apply(undefined, [serverBus, state, newState].concat((0, _toConsumableArray3.default)(args)));
	}
};