'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.defaultWidth = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _manifest = require('../manifest');

var _manifest2 = _interopRequireDefault(_manifest);

var _columnsParam = require('../columnsParam');

var columnsParam = _interopRequireWildcard(_columnsParam);

var _xenaQuery = require('../xenaQuery');

var _xenaQuery2 = _interopRequireDefault(_xenaQuery);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('../underscore_ext');
var Rx = require('../rx');

var _require = require('./common'),
    userServers = _require.userServers,
    setCohort = _require.setCohort,
    fetchSamples = _require.fetchSamples,
    fetchColumnData = _require.fetchColumnData,
    fetchCohortData = _require.fetchCohortData,
    fetchSurvival = _require.fetchSurvival,
    fetchClustering = _require.fetchClustering;

var _require2 = require('../models/fieldSpec'),
    setFieldType = _require2.setFieldType;

var _require3 = require('../notifications'),
    setNotifications = _require3.setNotifications;

var _require4 = require('../models/searchSamples'),
    remapFields = _require4.remapFields;

var _require5 = require('../inlineState'),
    fetchInlineState = _require5.fetchInlineState;

var _require6 = require('./utils'),
    compose = _require6.compose,
    make = _require6.make,
    mount = _require6.mount;

var _require7 = require('../dom_helper'),
    JSONToqueryString = _require7.JSONToqueryString;

var _require8 = require('../bookmark'),
    parseBookmark = _require8.parseBookmark;

var gaEvents = require('../gaEvents');


function fetchBookmark(serverBus, bookmark) {
	gaEvents('bookmark', 'load');
	serverBus.next(['bookmark', Rx.Observable.ajax({
		responseType: 'text',
		method: 'GET',
		url: '/api/bookmarks/bookmark?id=' + bookmark
	}).map(function (r) {
		return parseBookmark(r.response);
	})]);
}

function fetchManifest(serverBus, url) {
	serverBus.next(['manifest', Rx.Observable.ajax({
		responseType: 'text',
		method: 'GET',
		url: url
	}).map(function (r) {
		return (0, _manifest2.default)(r.response);
	})]);
}

function fetchDatasetCohort(serverBus, _ref) {
	var host = _ref.host,
	    name = _ref.name;

	serverBus.next(['dataset-cohort', _xenaQuery2.default.datasetCohort(host, name)]);
}

var warnZoom = function warnZoom(state) {
	return !_.getIn(state, ['notifications', 'zoomHelp']) ? _.assoc(state, 'zoomHelp', true) : state;
};

var zoomHelpClose = function zoomHelpClose(state) {
	return _.assocIn(_.dissoc(state, 'zoomHelp'), ['notifications', 'zoomHelp'], true);
};

var setLoadingState = function setLoadingState(state, params) {
	return _.any(['bookmark', 'inlineState', 'columns'], function (p) {
		return _.get(params, p);
	}) ? _.assoc(state, 'loadPending', true) : state;
};

function fetchState(serverBus) {
	serverBus.next(['inlineState', fetchInlineState()]);
}

function resetWizard(state) {
	return state.columnOrder.length > 2 ? _.assoc(state, 'wizardMode', false, 'showWelcome', false) : state;
}

// Use min app width (1280px) if viewport width is currently smaller than min
// app width. (App is responsive above 1280px but components are snapped at a
// minimum width of 1280px)
var defaultWidth = exports.defaultWidth = function defaultWidth(viewportWidth) {
	var width = viewportWidth < 1280 ? 1280 : viewportWidth;
	return Math.floor((width - 48) / 4) - 16; // Allow for 2 x 24px gutter on viewport, plus 16px margin for column
};

// XXX This same info appears in Datapages.js, and in various links.
var getPage = function getPage(path) {
	return path === '/transcripts/' ? 'transcripts' : path === '/hub/' ? 'hub' : path === '/datapages/' ? 'datapages' : path === '/import/' ? 'import' : 'heatmap';
};

var setPage = function setPage(state, path, params) {
	return _.assoc(state, 'page', getPage(path), 'params', params);
};

var paramList = function paramList(params) {
	return _.isEmpty(params) ? '' : '?' + JSONToqueryString(params);
};

var controls = {
	init: function init(state) {
		var pathname = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '/';
		var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

		var next = setLoadingState(state, params),
		    cohort = columnsParam.cohort(params.columns),
		    page = setPage(next, pathname, params);
		setCohort(cohort);
		return cohort ? _.updateIn(page, ['spreadsheet'], setCohort({ name: cohort })) : page;
	},
	'init-post!': function initPost(serverBus, state, newState, pathname) {
		var params = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};

		var bookmark = _.get(params, 'bookmark'),
		    cohort = columnsParam.cohort(params.columns),
		    inlineState = _.get(params, 'inlineState'),
		    manifest = _.get(params, 'manifest');
		if (inlineState) {
			fetchState(serverBus);
		} else if (bookmark) {
			fetchBookmark(serverBus, bookmark);
		} else if (cohort) {
			fetchCohortData(serverBus, newState.spreadsheet);
		} else if (params.columns) {
			// have columns but no cohort
			// All cohorts must match, so take the first one &
			// later filter any datasets not in the cohort.
			fetchDatasetCohort(serverBus, params.columns[0]);
		}
		if (manifest) {
			// not sure why this is separate. bookmark + manifest??
			fetchManifest(serverBus, manifest);
		}
	},
	navigate: function navigate(state, page) {
		var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
		return _.assoc(state, 'page', page, 'params', params);
	},
	'navigate-post!': function navigatePost(serverBus, state, newState, page, params) {
		return history.pushState({}, '', '/' + page + '/' + paramList(params));
	},
	history: function (_history) {
		function history(_x5, _x6) {
			return _history.apply(this, arguments);
		}

		history.toString = function () {
			return _history.toString();
		};

		return history;
	}(function (state, history) {
		return _.isEmpty(history) ? state : _.Let(function () {
			var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : history,
			    path = _ref2.path,
			    _ref2$params = _ref2.params,
			    params = _ref2$params === undefined ? {} : _ref2$params;

			return _.assoc(state, 'page', getPage(path), 'params', params);
		});
	}),
	cohort: function cohort(state, _cohort, width) {
		return _.updateIn(state, ['spreadsheet'], setCohort({ name: _cohort }, width));
	},
	'cohort-post!': function cohortPost(serverBus, state, newState) {
		return (
			// XXX just samples, now
			fetchCohortData(serverBus, newState.spreadsheet)
		);
	},
	cohortReset: function cohortReset(state) {
		return _.updateIn(state, ['spreadsheet'], setCohort(undefined, undefined));
	},
	'import': function _import(state, newState) {
		return _.merge(state, newState);
	},
	'import-error': function importError(state) {
		return _.assoc(state, 'stateError', 'import');
	},
	stateError: function stateError(state, error) {
		return _.assoc(state, 'stateError', error);
	},
	'km-open-post!': function kmOpenPost(serverBus, state, newState) {
		return fetchSurvival(serverBus, newState, {});
	}, // 2nd param placeholder for km.user
	'localStatus': function localStatus(state, stat) {
		return _.assoc(state, 'localStatus', stat);
	}
};

var spreadsheetControls = {
	'refresh-cohorts-post!': function refreshCohortsPost(serverBus, state, newState) {
		// reload samples
		fetchCohortData(serverBus, newState);
	},
	cluster: function cluster(state, id, value) {
		return _.assocIn(state, ['columns', id, 'clustering'], value);
	},
	'cluster-post!': function clusterPost(serverBus, state, newState, id, value, data) {
		if (value != null && _.getIn(newState, ['data', id, 'clustering', 'probes']) == null) {
			fetchClustering(serverBus, newState, id, data);
		}
	},
	sampleFilter: function sampleFilter(state, _sampleFilter) {
		return _.assoc(state, 'cohort', _.assocIn(state.cohort, ['sampleFilter'], _sampleFilter), 'survival', null);
	},
	'sampleFilter-post!': function sampleFilterPost(serverBus, state, newState) {
		return fetchSamples(serverBus, userServers(newState), newState.cohort, newState.allowOverSamples);
	},
	'addColumnAddHover': function addColumnAddHover(state, hovering) {
		return _.assoc(state, 'addColumnAddHover', hovering);
	},
	'add-column': function addColumn(state, posOrId) {
		for (var _len = arguments.length, idSettingsList = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
			idSettingsList[_key - 2] = arguments[_key];
		}

		var columnOrder = state.columnOrder,
		    columns = state.columns,
		    sampleSearch = state.sampleSearch,
		    data = state.data,
		    isPos = _.isNumber(posOrId),
		    pos = isPos ? posOrId : columnOrder.indexOf(posOrId) - 1,
		    ids = _.pluck(idSettingsList, 'id'),
		    settingsList = _.pluck(idSettingsList, 'settings'),
		    newOrder = _.splice.apply(_, [columnOrder, pos + 1, isPos ? 0 : 1].concat((0, _toConsumableArray3.default)(ids))),
		    newState = _.assoc(state, 'columns', _.merge(columns, _.object(ids, settingsList)), 'columnOrder', newOrder, 'sampleSearch', remapFields(columnOrder, newOrder, sampleSearch), 'editing', null, // is editing always off after column add?
		'data', _.merge(data, _.object(ids, ids.map(_.constant({ 'status': 'loading' })))));

		return resetWizard(newState);
	},
	'add-column-post!': function addColumnPost(serverBus, state, newState, pos) {
		for (var _len2 = arguments.length, idSettingsList = Array(_len2 > 4 ? _len2 - 4 : 0), _key2 = 4; _key2 < _len2; _key2++) {
			idSettingsList[_key2 - 4] = arguments[_key2];
		}

		idSettingsList.forEach(function (_ref3) {
			var id = _ref3.id;
			return fetchColumnData(serverBus, state.cohortSamples, id, _.getIn(newState, ['columns', id]));
		});
	},
	'enableTransition': function enableTransition(state, value) {
		return _.assoc(state, 'enableTransition', value);
	},
	'edit-column': function editColumn(state, editing) {
		return _.assoc(state, 'editing', editing);
	},
	resize: function resize(state, id, _ref4) {
		var width = _ref4.width,
		    height = _ref4.height;
		return _.assocInAll(state, ['zoom', 'height'], Math.round(height), ['columns', id, 'width'], Math.round(width));
	},
	// If 'editing' is a blank column (isNumber), we need to decrement it
	// if it is higher than id index, to preserve its position in the order.
	remove: function remove(state, id) {
		var columns = state.columns,
		    columnOrder = state.columnOrder,
		    data = state.data,
		    editing = state.editing,
		    newEditing = _.isNumber(editing) && editing >= state.columnOrder.indexOf(id) ? editing - 1 : editing,
		    ns = _.assoc(state, 'editing', newEditing, 'columns', _.dissoc(columns, id), 'columnOrder', _.without(columnOrder, id), 'data', _.dissoc(data, id));

		return _.assoc(ns, 'sampleSearch', remapFields(state.columnOrder, ns.columnOrder, state.sampleSearch));
	},
	order: function order(state, _order) {
		// Filter out 'editing' columns
		var newOrder = _order.filter(function (id) {
			return !_.isNumber(id);
		}),
		    editing = _.findIndexDefault(_order.slice(1), _.isNumber, state.editing);
		return _.assoc(state, 'columnOrder', newOrder, 'editing', editing, 'sampleSearch', remapFields(state.columnOrder, _order, state.sampleSearch));
	},
	zoom: function zoom(state, _zoom) {
		return warnZoom(_.assoc(state, "zoom", _zoom));
	},
	'zoom-help-close': zoomHelpClose,
	'zoom-help-disable': zoomHelpClose,
	'zoom-help-disable-post!': function zoomHelpDisablePost(serverBus, state, newState) {
		return setNotifications(newState.notifications);
	},
	'notifications-disable': function notificationsDisable(state, key) {
		return _.assocIn(state, ['notifications', key], true);
	},
	'notifications-disable-post!': function notificationsDisablePost(serverBus, state, newState) {
		return setNotifications(newState.notifications);
	},
	'notifications-enable': function notificationsEnable(state) {
		return _.assoc(state, 'notifications', {});
	},
	'notifications-enable-post!': function notificationsEnablePost(serverBus, state, newState) {
		return setNotifications(newState.notifications);
	},
	reload: function reload(state, id) {
		return _.assocIn(state, ['data', id, 'status'], 'loading');
	},
	'reload-post!': function reloadPost(serverBus, state, newState, id) {
		return fetchColumnData(serverBus, newState.cohortSamples, id, _.getIn(newState, ['columns', id]));
	},
	fieldType: function fieldType(state, id, _fieldType) {
		return _.updateIn(state, ['columns', id], setFieldType(_fieldType), ['data', id, 'status'], function () {
			return 'loading';
		});
	},
	'fieldType-post!': function fieldTypePost(serverBus, state, newState, id) {
		return fetchColumnData(serverBus, newState.cohortSamples, id, _.getIn(newState, ['columns', id]));
	},
	vizSettings: function vizSettings(state, column, settings) {
		return _.assocIn(state, ['columns', column, 'vizSettings'], settings);
	},
	'columnLabel': function columnLabel(state, id, value) {
		return _.assocIn(state, ['columns', id, 'user', 'columnLabel'], value);
	},
	'fieldLabel': function fieldLabel(state, id, value) {
		return _.assocIn(state, ['columns', id, 'user', 'fieldLabel'], value);
	},
	'showIntrons': function showIntrons(state, id) {
		return _.updateIn(state, ['columns', id, 'showIntrons'], function (v) {
			return !v;
		});
	},
	'sortVisible': function sortVisible(state, id, value) {
		return _.assocIn(state, ['columns', id, 'sortVisible'], value);
	},
	'km-open': function kmOpen(state, id) {
		return _.assocInAll(state, ['km', 'id'], id, ['km', 'title'], _.getIn(state, ['columns', id, 'user', 'columnLabel']), ['km', 'label'], _.getIn(state, ['columns', id, 'user', 'fieldLabel']), ['km', 'survivalType'], _.intersection([_.getIn(state, ['km', 'survivalType'])], _.keys(_.getIn(state, ['survival'])))[0]);
	},
	// see km-open-post! in controls, above. Requires wizard.cohortFeatures.
	'km-close': function kmClose(state) {
		return _.assocIn(state, ['km', 'id'], null);
	},
	'km-cutoff': function kmCutoff(state, value) {
		return _.assocIn(state, ['km', 'cutoff'], value);
	},
	'km-splits': function kmSplits(state, value) {
		return _.assocIn(state, ['km', 'splits'], value);
	},
	'km-survivalType': function kmSurvivalType(state, value) {
		return _.assocIn(state, ['km', 'survivalType'], value);
	},
	'heatmap': function heatmap(state) {
		return _.assocInAll(state, ['mode'], 'heatmap', ['chartState', 'spreadsheetColumn'], null);
	},
	'chart': function chart(state) {
		return _.assoc(state, 'mode', 'chart');
	},
	'chart-set-column': function chartSetColumn(state, id) {
		return _.assocIn(state, ['chartState', 'spreadsheetColumn'], id);
	},
	'chart-set-state': function chartSetState(state, chartState) {
		return _.assoc(state, 'chartState', chartState);
	},
	'chart-set-average-post!': function chartSetAveragePost(serverBus, state, newState, offsets, thunk) {
		return serverBus.next(['chart-average-data', Rx.Observable.of(offsets, Rx.Scheduler.async), thunk]);
	},
	'sample-search': function sampleSearch(state, text) {
		return _.assoc(state, 'sampleSearch', text);
	},
	// XXX maybe this should be transient state, instead, since it's not
	// meaningful after reload?
	'highlightSelect': function highlightSelect(state, highlight) {
		return _.assoc(state, 'highlightSelect', highlight);
	},
	'vizSettings-open': function vizSettingsOpen(state, id) {
		return _.assoc(state, 'openVizSettings', id);
	},
	'sortDirection': function sortDirection(state, id, newDir) {
		return _.assocIn(state, ['columns', id, 'sortDirection'], newDir);
	},
	'allowOverSamples': function allowOverSamples(state, aos) {
		return _.assoc(state, 'allowOverSamples', aos);
	},
	'allowOverSamples-post!': function allowOverSamplesPost(serverBus, state, newState, aos) {
		return fetchSamples(serverBus, userServers(newState), newState.cohort, aos);
	},
	showWelcome: function showWelcome(state, show) {
		return _.assoc(state, 'showWelcome', show);
	},
	wizardMode: function wizardMode(state, mode) {
		return _.assoc(state, 'wizardMode', mode);
	},
	viewportWidth: function viewportWidth(state, width) {
		return _.assoc(state, 'defaultWidth', defaultWidth(width));
	},
	// Due to wonky react-bootstrap handlers, xzoom can occur after remove, so
	// check that the column exists before updating.
	'xzoom': function xzoom(state, id, _xzoom) {
		return _.updateIn(state, ['columns', id], function (c) {
			return c ? _.assoc(c, 'xzoom', _xzoom) : c;
		});
	}
};

exports.default = compose(mount(make(spreadsheetControls), ['spreadsheet']), make(controls));