'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _underscore_ext = require('../underscore_ext');

var _xenaQuery = require('../xenaQuery');

var _xenaQuery2 = _interopRequireDefault(_xenaQuery);

var _dataType = require('../models/dataType');

var _rx = require('../rx');

var _rx2 = _interopRequireDefault(_rx);

var _utils = require('./utils');

var _query2 = require('./query');

var _query3 = _interopRequireDefault(_query2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('../defaultServers'),
    localHub = _require.servers.localHub;

var _require2 = require('./common'),
    userServers = _require2.userServers;

var of = _rx2.default.Observable.of;

// Cohort metadata looks like
// {[cohort]: [tag, tag, ...], [cohort]: [tag, tag, ...], ...}
// We want data like
// {[tag]: [cohort, cohort, ...], [tag]: [cohort, cohort, ...], ...}

function invertCohortMeta(meta) {
	return (0, _underscore_ext.fmap)((0, _underscore_ext.groupBy)((0, _underscore_ext.flatmap)(meta, function (tags, cohort) {
		return tags.map(function (tag) {
			return [cohort, tag];
		});
	}), function (_ref) {
		var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
		    tag = _ref2[1];

		return tag;
	}), function (cohortTags) {
		return cohortTags.map(function (_ref3) {
			var _ref4 = (0, _slicedToArray3.default)(_ref3, 1),
			    cohort = _ref4[0];

			return cohort;
		});
	});
}

var fetchCohortMeta = function fetchCohortMeta() {
	return _xenaQuery2.default.fetchCohortMeta.map(invertCohortMeta).catch(function () {
		return of({});
	});
};

var fetchCohortPreferred = function fetchCohortPreferred() {
	return _xenaQuery2.default.fetchCohortPreferred.map(function (cohortPreferred) {
		return (0, _underscore_ext.fmap)(cohortPreferred, function (preferred) {
			return (0, _underscore_ext.fmap)(preferred, function (_ref5) {
				var host = _ref5.host,
				    dataset = _ref5.dataset;
				return (0, _stringify2.default)({ host: host, name: dataset });
			});
		});
	}).catch(function () {});
};

var fetchCohortPhenotype = function fetchCohortPhenotype() {
	return _xenaQuery2.default.fetchCohortPhenotype.map(function (cohortPhenotype) {
		return (0, _underscore_ext.fmap)(cohortPhenotype, function (preferred) {
			return (0, _underscore_ext.map)(preferred, function (_ref6) {
				var host = _ref6.host,
				    dataset = _ref6.dataset,
				    feature = _ref6.feature;
				return { dsID: (0, _stringify2.default)({ host: host, name: dataset }), name: feature };
			});
		});
	});
};

var fetchCohortAnalytic = function fetchCohortAnalytic() {
	return _xenaQuery2.default.fetchCohortAnalytic;
};

var fetchTumorMap = function fetchTumorMap() {
	return _xenaQuery2.default.fetchTumorMap.map(function (res) {
		return res;
	}).catch(function () {});
};

var phenoPat = /^phenotype/i; // match ../viwes/VariableSelect.js definition of phenotype data
var isPhenotype = function isPhenotype(ds) {
	return ds.type === 'clinicalMatrix' && (!ds.dataSubType || ds.dataSubType.match(phenoPat));
};

// {wizard: {cohortDatasets: {[cohort]: {[server]: [dataset, ...]}}}}
var allPhenoDatasets = function allPhenoDatasets(state, cohort, servers) {
	return (0, _underscore_ext.flatmap)((0, _underscore_ext.pick)((0, _underscore_ext.getIn)(state, ['wizard', 'cohortDatasets', state.spreadsheet.cohort.name], {}), servers), function (datasets, server) {
		return datasets.filter(isPhenotype).map(function (ds) {
			return ['cohortFeatures', cohort, server, ds.name];
		});
	});
};

// Do we really have to handle linkedHub here?? Shouldn't it be set in state?
// Depends on order of our reducers? We shouldn't have to deal with it here.
var wizardData = function wizardData(state) {
	return state.page !== 'heatmap' ? [] : (0, _underscore_ext.Let)(function () {
		var _ref7 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : state,
		    spreadsheet = _ref7.spreadsheet;

		var servers = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : userServers(spreadsheet);
		return [['cohortMeta'], ['cohortPreferred'], ['cohortPhenotype'], ['cohortAnalytic'], ['cohortTumorMap']].concat((0, _toConsumableArray3.default)(servers.map(function (server) {
			return ['serverCohorts', server];
		})), (0, _toConsumableArray3.default)(spreadsheet.cohort ? servers.map(function (server) {
			return ['cohortDatasets', spreadsheet.cohort.name, server];
		}) : []), (0, _toConsumableArray3.default)(spreadsheet.cohort ? allPhenoDatasets(state, spreadsheet.cohort.name, servers) : []));
	});
};

var fetchMethods = {
	// XXX Note that this will cache lists from hubs that the user has
	// recently disabled. We only clear cache when the cohort is changed.
	// OTOH it will fetch any recently enabled hubs. So, the view should
	// iterate over the userServer list, not this cache.
	cohortDatasets: function cohortDatasets(cohort, server) {
		return (0, _xenaQuery.datasetList)(server, [cohort]).catch(function () {
			return of([]);
		});
	},
	cohortFeatures: function cohortFeatures(cohort, server, dataset) {
		return (0, _xenaQuery.allFieldMetadata)(server, dataset).catch(function () {
			return of([]);
		});
	},
	serverCohorts: function serverCohorts(server) {
		return (0, _xenaQuery.allCohorts)(server, _dataType.ignoredType).catch(function () {
			return of([]);
		});
	},
	cohortMeta: function cohortMeta() {
		return fetchCohortMeta();
	},
	cohortPreferred: function cohortPreferred() {
		return fetchCohortPreferred();
	},
	cohortPhenotype: function cohortPhenotype() {
		return fetchCohortPhenotype();
	},
	cohortAnalytic: function cohortAnalytic() {
		return fetchCohortAnalytic();
	},
	cohortTumorMap: function cohortTumorMap() {
		return fetchTumorMap();
	}
};

// XXX review this for wizard
var cachePolicy = {
	// cache all hosts
	// ['serverCohorts', server]
	serverCohorts: _underscore_ext.identity,
	cohortMeta: _underscore_ext.identity,
	cohortPreferred: _underscore_ext.identity,
	cohortPhenotype: _underscore_ext.identity,
	cohortAnalytic: _underscore_ext.identity,
	cohortTumorMap: _underscore_ext.identity,

	// cache a single key (cohort): the last cohort updated.
	// ['cohortDatasets', cohort, server]
	// ['cohortFeatures', cohort, server, dsName]
	default: function _default(state, path) {
		return (0, _underscore_ext.updateIn)(state, ['wizard', path[0]], function (item) {
			return (0, _underscore_ext.pick)(item, path[1]);
		});
	}
};

var _query = (0, _query3.default)(fetchMethods, wizardData, cachePolicy, 'wizard'),
    fetchController = _query.controller,
    invalidatePath = _query.invalidatePath;

var invalidateCohorts = (0, _underscore_ext.Let)(function () {
	var _ref8 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _underscore_ext.matchKeys,
	    any = _ref8.any;

	return function (_ref9) {
		var wizard = _ref9.wizard;

		invalidatePath(wizard, ['cohortMeta']);
		invalidatePath(wizard, ['cohortPreferred']);
		invalidatePath(wizard, ['cohortPhenotype']);
		invalidatePath(wizard, ['cohortAnalytic']);
		invalidatePath(wizard, ['serverCohorts', any]);
		invalidatePath(wizard, ['cohortDatasets', any, any]);
		invalidatePath(wizard, ['cohortFeatures', any, any, any]);
	};
});

var invalidateLocalHub = (0, _underscore_ext.Let)(function () {
	var _ref10 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _underscore_ext.matchKeys,
	    any = _ref10.any;

	return function (_, __, _ref11) {
		var wizard = _ref11.wizard;

		invalidatePath(wizard, ['serverCohorts', localHub]);
		invalidatePath(wizard, ['cohortDatasets', any, localHub]);
		invalidatePath(wizard, ['cohortFeatures', any, localHub, any]);
	};
});

var controls = {
	'localStatus-post!': invalidateLocalHub,
	'localQueue-post!': invalidateLocalHub,
	'refresh-cohorts-post!': function refreshCohortsPost(serverBus, state, newState) {
		return invalidateCohorts(newState);
	}
};

exports.default = (0, _utils.compose)(fetchController, (0, _utils.make)(controls));