'use strict';

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('./underscore_ext');

var _require = require('./color_helper'),
    contrastColor = _require.contrastColor,
    greyHEX = _require.greyHEX;

var _require2 = require('./models/mutationVector'),
    impact = _require2.impact,
    getSNVEffect = _require2.getSNVEffect;

var labelFont = 12;
var labelMargin = 1; // left & right margin

var radius = 4;
var minVariantHeight = function minVariantHeight(pixPerRow) {
	return Math.max(pixPerRow, 2);
}; // minimum draw height of 2

var toYPx = function toYPx(zoom, v) {
	var height = zoom.height,
	    count = zoom.count,
	    index = zoom.index,
	    svHeight = height / count;

	return { svHeight: svHeight, y: (v.y - index) * svHeight + svHeight / 2 };
};

var toYPxSubRow = function toYPxSubRow(zoom, v) {
	var rowCount = v.rowCount,
	    subrow = v.subrow,
	    height = zoom.height,
	    count = zoom.count,
	    svHeight = height / (count * rowCount);

	return { svHeight: svHeight, y: toYPx(zoom, v).y + svHeight * ((1 - rowCount) / 2 + subrow) };
};

var splitRows = function splitRows(count, height) {
	return height / count > labelFont;
};

function push(arr, v) {
	arr.push(v);
	return arr;
}

// XXX note this draws outside the zoom area. It could be optimized
// by considering zoom count and index.
function drawBackground(vg, width, height, pixPerRow, hasValue) {
	var _$reduce = _.reduce(_.groupByConsec(hasValue, _.identity), function (_ref, g) {
		var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
		    acc = _ref2[0],
		    sum = _ref2[1];

		return [g[0] ? acc : push(acc, [sum, g.length]), sum + g.length];
	}, [[], 0]),
	    _$reduce2 = (0, _slicedToArray3.default)(_$reduce, 1),
	    stripes = _$reduce2[0];

	vg.smoothing(false);
	vg.box(0, 0, width, height, 'white'); // white background

	var rects = stripes.map(function (_ref3) {
		var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
		    offset = _ref4[0],
		    len = _ref4[1];

		return [0, offset * pixPerRow, width, pixPerRow * len];
	});
	vg.drawRectangles(rects, { fillStyle: 'grey' });

	var nullLabels = stripes.filter(function (_ref5) {
		var _ref6 = (0, _slicedToArray3.default)(_ref5, 2),
		    len = _ref6[1];

		return len * pixPerRow > labelFont;
	});
	nullLabels.forEach(function (_ref7) {
		var _ref8 = (0, _slicedToArray3.default)(_ref7, 2),
		    offset = _ref8[0],
		    len = _ref8[1];

		vg.textCenteredPushRight(0, pixPerRow * offset, width, pixPerRow * len, 'black', labelFont, "null");
	});
}

function drawImpactNodes(vg, width, zoom, smallVariants) {
	// --------- separate variants to SV(with feet "[" , "]" or size >50bp) vs others (small) ---------
	var height = zoom.height,
	    count = zoom.count,
	    vHeight = minVariantHeight(height / count),
	    split = splitRows(count, height),
	    toY = split ? toYPxSubRow : toYPx,
	    minWidth = 2; // double that is the minimum width we draw

	// --------- small variants drawing start here ---------

	var varByImp = _.groupByConsec(smallVariants, function (v) {
		return v.color;
	});
	varByImp = _.sortBy(varByImp, function (list) {
		return impact[getSNVEffect(impact, list[0].data.effect)];
	}); // draw variants from low to high impact

	if (!split) {
		varByImp.forEach(function (vars) {
			var points = vars.map(function (v) {
				var _toYPx = toYPx(zoom, v),
				    y = _toYPx.y,
				    padding = _.max([minWidth - Math.abs(v.xEnd - v.xStart), 0]);

				return [v.xStart - padding, y, v.xEnd + padding, y];
			});
			vg.drawPoly(points, { strokeStyle: vars[0].color, lineWidth: vHeight });
		});
	} else {
		smallVariants.forEach(function (v) {
			var _toY = toY(zoom, v),
			    svHeight = _toY.svHeight,
			    y = _toY.y,
			    h = minVariantHeight(svHeight),
			    padding = _.max([minWidth - Math.abs(v.xEnd - v.xStart), 0]);

			vg.drawPoly([[v.xStart - padding, y, v.xEnd + padding, y]], { strokeStyle: v.color, lineWidth: h });

			// small variants label
			var xStart = v.xStart,
			    xEnd = v.xEnd,
			    data = v.data,
			    label = data.aminoAcid || data.alt,
			    textWidth = vg.textWidth(labelFont, label),
			    minTxtWidth = vg.textWidth(labelFont, 'WWWW');


			if (xEnd - xStart >= minTxtWidth / 5 * label.length) {
				vg.textCenteredPushRight(xStart + labelMargin, y - h / 2, xEnd - xStart - labelMargin, h, contrastColor(v.color), labelFont, label);
			} else {
				vg.textCenteredPushRight(xEnd + labelMargin, y - h / 2, textWidth, h, greyHEX, labelFont, label);
			}
		});
	}
}

function drawSVNodes(vg, width, zoom, svVariants) {
	var count = zoom.count,
	    height = zoom.height,
	    toY = splitRows(count, height) ? toYPxSubRow : toYPx,
	    varByIdMap = svVariants.map(function (v) {
		var _v$data = v.data,
		    alt = _v$data.alt,
		    altGene = _v$data.altGene,
		    _toY2 = toY(zoom, v),
		    svHeight = _toY2.svHeight,
		    y = _toY2.y;

		return (0, _extends3.default)({}, v, {
			y: y,
			h: minVariantHeight(svHeight),
			alt: alt,
			altGene: altGene
		});
	});

	//SV variants draw color background according to joining chromosome

	varByIdMap.forEach(function (variant) {
		var xStart = variant.xStart,
		    xEnd = variant.xEnd,
		    y = variant.y,
		    color = variant.color,
		    h = variant.h,
		    points = [[xStart, y, xEnd, y]];


		vg.drawPoly(points, { strokeStyle: color, lineWidth: h });
	});

	varByIdMap.forEach(function (variant) {
		var xStart = variant.xStart,
		    xEnd = variant.xEnd,
		    y = variant.y,
		    h = variant.h,
		    endMark = xEnd < width - 1 ? [[xEnd, y, xEnd + 1, y]] : [],
		    startMark = xStart > 0 ? [[xStart, y, xStart + 1, y]] : [],
		    points = [].concat(startMark, endMark);


		vg.drawPoly(points, { strokeStyle: 'black', lineWidth: h });
	});

	//feet variants show text label when there is only one variant for this sample, otherwise, text might overlap
	if (height / count > labelFont) {
		var margin = 2 * labelMargin,
		    //more labelMargin due to drawing of breakpoint
		minTxtWidth = vg.textWidth(labelFont, 'WWWW');

		varByIdMap.forEach(function (variant) {
			var xStart = variant.xStart,
			    xEnd = variant.xEnd,
			    y = variant.y,
			    alt = variant.alt,
			    altGene = variant.altGene,
			    h = variant.h;


			if (h > labelFont && xEnd - xStart - margin >= minTxtWidth) {
				vg.clip(xStart + margin, y - h / 2, xEnd - xStart - 2 * margin, h, function () {
					return vg.textCenteredPushRight(xStart + margin, y - h / 2, xEnd - xStart - margin, h, 'black', labelFont, altGene ? altGene + " " + alt : alt);
				});
			}
		});
	};
}

var drawWithBackground = _.curry(function (draw, vg, props) {
	var width = props.width,
	    zoom = props.zoom,
	    nodes = props.nodes,
	    count = zoom.count,
	    height = zoom.height,
	    index = zoom.index;

	if (!nodes) {
		vg.box(0, 0, width, height, "gray");
		return;
	}

	var samples = props.samples,
	    samplesInDS = props.index.bySample,
	    last = index + count,
	    toDraw = nodes.filter(function (v) {
		return v.y >= index && v.y < last;
	}),
	    pixPerRow = height / count,
	    hasValue = samples.slice(index, index + count).map(function (s) {
		return samplesInDS[s];
	});


	vg.labels(function () {
		drawBackground(vg, width, height, pixPerRow, hasValue);
		draw(vg, width, zoom, toDraw);
	});
});

var drawMutations = drawWithBackground(drawImpactNodes);
var drawSV = drawWithBackground(drawSVNodes);

module.exports = { drawMutations: drawMutations, drawSV: drawSV, splitRows: splitRows, radius: radius, minVariantHeight: minVariantHeight, toYPx: toYPx, toYPxSubRow: toYPxSubRow, labelFont: labelFont };