'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getHeatmap = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.heatmapParam = heatmapParam;

var _util = require('./util');

var _underscore_ext = require('./underscore_ext');

var _searchParser = require('./models/searchParser');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// This is pretty much cut & paste from columnsParam, so might be
// more complex than necessary.

var heatmapOptPaths = {
	showWelcome: ['showWelcome'],
	search: ['sampleSearch'],
	searchSampleList: ['sampleSearch'],
	filter: ['sampleFilter'],
	mode: ['mode']
};

function searchIsValid(s) {
	try {
		(0, _searchParser.parse)(s);
	} catch (e) {
		return false;
	}
	return true;
}

var isStringArray = function isStringArray(a) {
	return (0, _underscore_ext.isArray)(a) && a.every(function (s) {
		return (0, _underscore_ext.isString)(s);
	});
};

var arrayToSearch = function arrayToSearch(a) {
	return a.map(function (s) {
		return 'A:="' + s + '"';
	}).join(' OR ');
};

var invalid = {}; // use reference equality to tag invalid values.
var heatmapOptCleaner = {
	mode: function mode(s) {
		return s === 'heatmap' ? s : s === 'chart' ? s : invalid;
	},
	search: function search(s) {
		return searchIsValid(s) ? s : invalid;
	},
	searchSampleList: function searchSampleList(s) {
		return isStringArray(s) ? arrayToSearch(s) : invalid;
	},
	filter: function filter(s) {
		return searchIsValid(s) ? s : invalid;
	},
	showWelcome: function showWelcome(v) {
		return (0, _underscore_ext.isBoolean)(v) ? v : invalid;
	}
};
var heatmapOptClean = function heatmapOptClean(opt, v) {
	return (heatmapOptCleaner[opt] || _underscore_ext.identity)(v);
};
var heatmapOpts = (0, _keys2.default)(heatmapOptPaths);

var heatmapOptSetter = {
	mode: function mode(s) {
		return s ? s : invalid;
	},
	search: function search(s) {
		return searchIsValid(s) ? s : invalid;
	},
	searchSampleList: function searchSampleList() {
		return invalid;
	}, // don't try to save a sample highlight
	filter: function filter() {
		return invalid;
	}, // don't try to save a sample filter
	showWelcome: function showWelcome(v) {
		return v == null ? invalid : v;
	}
};
var heatmapOptSet = function heatmapOptSet(opt, v) {
	return (heatmapOptSetter[opt] || _underscore_ext.identity)(v);
};

var cleanOpts = function cleanOpts(c) {
	return (0, _underscore_ext.flatmap)((0, _underscore_ext.pick)(c, heatmapOpts), function (v, k) {
		return (0, _underscore_ext.Let)(function () {
			var cleaned = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : heatmapOptClean(k, v);
			return cleaned === invalid ? [] : [[heatmapOptPaths[k], cleaned]];
		});
	});
};

var heatmapSchema = function heatmapSchema(obj) {
	return (0, _underscore_ext.isObject)(obj);
};

function heatmapParam() {
	var _allParameters = (0, _util.allParameters)(),
	    heatmap = _allParameters.heatmap;

	if (heatmap) {
		try {
			var opts = cleanOpts(JSON.parse(heatmap[0]));
			if (heatmapSchema(opts)) {
				var _groupBy = (0, _underscore_ext.groupBy)(opts, function (_ref) {
					var _ref2 = (0, _slicedToArray3.default)(_ref, 1),
					    _ref2$ = (0, _slicedToArray3.default)(_ref2[0], 1),
					    k = _ref2$[0];

					return k === 'sampleFilter';
				}),
				    _groupBy$true = _groupBy.true;

				_groupBy$true = _groupBy$true === undefined ? [[]] : _groupBy$true;

				var _groupBy$true2 = (0, _slicedToArray3.default)(_groupBy$true, 1),
				    _groupBy$true2$ = (0, _slicedToArray3.default)(_groupBy$true2[0], 2),
				    filter = _groupBy$true2$[1],
				    other = _groupBy.false;

				return { heatmap: other, filter: filter };
			}
			console.log('Invalid heatmap parameter ' + opts);
		} catch (e) {
			console.log('Failed to parse heatmap ' + heatmap);
		}
	}
	return {};
}

var heatmapOptions = function heatmapOptions(heatmap) {
	return (0, _underscore_ext.pick)((0, _underscore_ext.mapObject)(heatmapOptPaths, function (path, key) {
		return heatmapOptSet(key, (0, _underscore_ext.getIn)(heatmap, path));
	}), function (v) {
		return v !== invalid;
	});
};

var encode = function encode(x) {
	return encodeURIComponent((0, _stringify2.default)(x));
};

var getHeatmap = exports.getHeatmap = function getHeatmap(state) {
	return encode(heatmapOptions((0, _underscore_ext.get)(state, 'spreadsheet')));
};