'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _underscore_ext = require('../underscore_ext');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Rx = require('../rx');
/*eslint-disable no-unused-vars*/

var _Rx$Observable = Rx.Observable,
    empty = _Rx$Observable.empty,
    from = _Rx$Observable.from,
    of = _Rx$Observable.of,
    defer = _Rx$Observable.defer;
var asap = Rx.Scheduler.asap;

var EMPTY = empty();

var fs = require('fs');

//function spy(msg, x) {
//	console.log(msg, `|${x}|`);
//	return x;
//}

var data = fs.readFileSync('/home/brian/xena/files/BRCA_clinicalMatrix2').toString();
var cutNewline = function cutNewline(str) {
	return str.replace(/[\n\r]+$/, '');
};

var chunkSize = 1000;

// lines including newlines
var lines = from(cutNewline(data).split(/(?=\n)/).slice(0, 3), asap);

var chunkExp = new RegExp('[\\s\\S]{' + chunkSize + '}', 'g');

var chunkBuffers = [].concat((0, _toConsumableArray3.default)(data.match(chunkExp) || []), [data.slice(-data.length % chunkSize)]);

function backPressure(done, next, len, bufferFn, sendFn) {
	var bufferCount,
	    stream = done() ? EMPTY : of(next(), asap).expand(function () {
		return done() ? EMPTY : bufferCount.find(function (c) {
			return c < len;
		}).concatMap(function () {
			return of(next(), asap);
		});
	}),
	    buffered = bufferFn(stream).share(),
	    sent = buffered.concatMap(sendFn).share();

	bufferCount = buffered.mapTo(1).merge(sent.mapTo(-1)).scan(function (acc, x) {
		return acc + x;
	}).startWith(0, asap).shareReplay(1, asap);
	return sent; // this is a bit weird. We have to subscribe to kick things
	// off. Shouldn't it be emitting something, instead? We have
	// to capture the sends, so we know when they finish.
	// Also need to return errors.
}

// Keep hitting the problem that I'm testing with small buffer sizes,
// which are smaller than the lines, which is the opposite of what we
// will actually experience: buffer will be much larger than lines.
// It's silly to try to handle lines longer than buffer, but I need a
// better test scenario. Maybe cut the demo file.
var toLines = function toLines(chunks) {
	var rem = '';
	return chunks.concatMap(function (chunk) {
		// unclear if this is going to do what we want, e.g.
		// split sometimes emits empty strings. Might need to
		// filter, or use a different split method.
		var lines = chunk.split(/(?=\n)/),
		    ret = [rem + lines[0]].concat((0, _toConsumableArray3.default)(lines.slice(1, -1)));
		rem = lines[lines.length - 1];
		return from(ret, asap);
	}).concat(defer(function () {
		return rem.length ? of(rem, asap) : EMPTY;
	}));
};

// Probably don't need this in practice, since buffer size will be much larger
// than line size.
var sliceOver = function sliceOver(lines) {
	return lines.concatMap(function (line) {
		return from([].concat((0, _toConsumableArray3.default)(line.match(chunkExp) || []), [line.slice(-line.length % chunkSize)]), asap);
	});
};

var fillBuffer = function fillBuffer(seq) {
	var buff = '';
	return seq.concatMap(function (s) {
		var over = buff.length + s.length - chunkSize,
		    ret = over > 0 ? of(buff + s.slice(0, -over), asap) : EMPTY;
		buff = over > 0 ? s.slice(-over) : buff + s;
		return ret;
	}).concat(defer(function () {
		return buff.length ? of(buff, asap) : EMPTY;
	}));
};

var fakeXHR = function fakeXHR(data) {
	return of(data, asap).do(function (d) {
		return process.stdout.write(d);
	}).delay(500);
};

var bufferFn = function bufferFn(stream) {
	return fillBuffer(sliceOver(toLines(stream)));
};

var buffersSent = (0, _underscore_ext.Let)(function () {
	var i = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
	return backPressure(function () {
		return i >= chunkBuffers.length;
	}, function () {
		return chunkBuffers[i++];
	}, 2, bufferFn, fakeXHR);
});

buffersSent.subscribe();

// To make a useable API, what should it look like?
// Streams:
//  chunks  -> calls expand & is gated by bufferCount
//  toLines -> collects chunks & emits lines
//  buffers -> collects lines into output buffers
//  bufferSent -> posts buffers
//  bufferCount -> diff of buffers & buffersSent
//
// Need a mechanism that's like "here's the reader, here's my
// processing, here's my writer, implement the back pressure"
//
// chunks expands from a fixed array, right now. In practice it would
// be emitting chunks from a file. We need a) boolean returning true
// if there are more buffers, b) fn to return the next buffer.
//
// reader looks a lot like an iterator.
// toLines is walking over the read chunks.
// buffers is more walking over the chunks.
// so, we end up with buffers, and we want to
// derive the count.
// So... pass (buffer, send)
// sent = buffers.concatMap(send).share
// count = blah blah
//
// sync(iterator, mapFn, sendFn)
// Can we combine mapFn and sendFn? No...we need to create a stream
// from mapFn, and count the before & after of sendFn.