'use strict';

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('underscore');

var min = Math.min,
    max = Math.max,
    floor = Math.floor;


function pxTransform1(layout, i, x) {
	var screen = layout.screen,
	    chrom = layout.chrom,
	    pos = chrom[i],
	    _pos = (0, _slicedToArray3.default)(pos, 2),
	    start = _pos[0],
	    end = _pos[1],
	    _screen$i = (0, _slicedToArray3.default)(screen[i], 2),
	    sstart = _screen$i[0],
	    send = _screen$i[1];

	return floor(sstart + (x - start + 1) * (send - sstart) / (end - start + 1));
}

// check for overlap. closed coords.
var overlapsRegion = function overlapsRegion(_ref) {
	var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
	    s = _ref2[0],
	    e = _ref2[1];

	return function (_ref3) {
		var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
		    cs = _ref4[0],
		    ce = _ref4[1];

		return cs <= e && ce >= s;
	};
};

// Using this due to poor performance of underscore findLastIndex. Also,
// there's no equivalent es6 method, though there is a [].findIndex.
function findLastIndex(arr, fn) {
	for (var i = arr.length - 1; i >= 0; --i) {
		if (fn(arr[i])) {
			return i;
		}
	}
	return -1;
}
// Find first and last overlapping draw regions
var regionIndxs = function regionIndxs(layout, intvl) {
	var overlaps = overlapsRegion(intvl);
	return [layout.chrom.findIndex(overlaps), findLastIndex(layout.chrom, overlaps)];
};

function flopIfIndexed(layout, _ref5, _ref6) {
	var _ref8 = (0, _slicedToArray3.default)(_ref5, 2),
	    s = _ref8[0],
	    e = _ref8[1];

	var _ref7 = (0, _slicedToArray3.default)(_ref6, 2),
	    si = _ref7[0],
	    ei = _ref7[1];

	var reversed = layout.reversed,
	    chrom = layout.chrom,
	    _chrom$si = (0, _slicedToArray3.default)(chrom[si], 2),
	    siStart = _chrom$si[0],
	    siEnd = _chrom$si[1],
	    _chrom$ei = (0, _slicedToArray3.default)(chrom[ei], 2),
	    eiStart = _chrom$ei[0],
	    eiEnd = _chrom$ei[1];

	return reversed ? [siEnd - e + siStart, eiEnd - s + eiStart] : [s, e];
}

function clipIndexed(layout, _ref9, _ref10) {
	var _ref12 = (0, _slicedToArray3.default)(_ref9, 2),
	    s = _ref12[0],
	    e = _ref12[1];

	var _ref11 = (0, _slicedToArray3.default)(_ref10, 2),
	    si = _ref11[0],
	    ei = _ref11[1];

	var _layout$chrom$si = (0, _slicedToArray3.default)(layout.chrom[si], 1),
	    start = _layout$chrom$si[0],
	    _layout$chrom$ei = (0, _slicedToArray3.default)(layout.chrom[ei], 2),
	    end = _layout$chrom$ei[1];

	return [max(s, start), min(e, end)];
}

var halfOpen = function halfOpen(_ref13) {
	var _ref14 = (0, _slicedToArray3.default)(_ref13, 2),
	    a = _ref14[0],
	    b = _ref14[1];

	return [a - 1, b];
};

// Project to screen coordinates the left-most and right-most
// visible chrom coords.
// flop;clip;halfOpen;map
function pxTransformInterval(layout, intvl) {
	var indxs = regionIndxs(layout, intvl),
	    _halfOpen = halfOpen(clipIndexed(layout, flopIfIndexed(layout, intvl, indxs), indxs)),
	    _halfOpen2 = (0, _slicedToArray3.default)(_halfOpen, 2),
	    s = _halfOpen2[0],
	    e = _halfOpen2[1],
	    _indxs = (0, _slicedToArray3.default)(indxs, 2),
	    si = _indxs[0],
	    ei = _indxs[1];

	return [pxTransform1(layout, si, s), pxTransform1(layout, ei, e)];
}

function flopIf(reversed, start, end) {
	return reversed ? function (_ref15) {
		var _ref16 = (0, _slicedToArray3.default)(_ref15, 2),
		    vstart = _ref16[0],
		    vend = _ref16[1];

		return [end - vend + start, end - vstart + start];
	} : _.identity;
}

function pxTransformI(layout, fn, i) {
	var screen = layout.screen,
	    chrom = layout.chrom,
	    reversed = layout.reversed,
	    pos = chrom[i];

	var _pos2 = (0, _slicedToArray3.default)(pos, 2),
	    start = _pos2[0],
	    end = _pos2[1];

	var _screen$i2 = (0, _slicedToArray3.default)(screen[i], 2),
	    sstart = _screen$i2[0],
	    send = _screen$i2[1];
	// If reversed, we mirror the coords in the exon, rather than swapping all the bounds. This might
	// be simpler.


	var flop = flopIf(reversed, start, end);
	var toPx = function toPx(x) {
		return floor(sstart + (x - start + 1) * (send - sstart) / (end - start + 1));
	};
	var clip = function clip(_ref17) {
		var _ref18 = (0, _slicedToArray3.default)(_ref17, 2),
		    s = _ref18[0],
		    e = _ref18[1];

		return [max(s, start), min(e, end)];
	};
	var intvlToPx = function intvlToPx(i) {
		return _.map(halfOpen(clip(flop(i))), toPx);
	};

	return fn(intvlToPx, pos, screen[i]);
}

function pxTransformEach(layout, fn) {
	var chrom = layout.chrom;


	_.times(chrom.length, function (i) {
		return pxTransformI(layout, fn, i);
	});
}

// This is hacky. Should really have started w/this, instead of
// with pxTransformEach.
function pxTransformFlatmap(layout, fn) {
	var res = [];
	pxTransformEach(layout, function (intvlToPx, pos, screen) {
		res = res.concat(fn(intvlToPx, pos, screen));
	});
	return res;
}

module.exports = {
	pxTransformInterval: pxTransformInterval,
	pxTransformEach: pxTransformEach,
	pxTransformFlatmap: pxTransformFlatmap
};