'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Desired behavior:
// When we change the state structure, add a migration.
// Sources of state:
//   sessionStorage
//   bookmarks
//   initial state
//
//

var version = 3; // XXX duplicated in store.js?

var _require = require('./underscore_ext'),
    assoc = _require.assoc,
    flatten = _require.flatten,
    get = _require.get,
    getIn = _require.getIn,
    Let = _require.Let,
    mapObject = _require.mapObject,
    merge = _require.merge,
    omit = _require.omit,
    pick = _require.pick,
    isString = _require.isString,
    updateIn = _require.updateIn,
    without = _require.without;

var _require2 = require('./defaultServers'),
    _require2$servers = _require2.servers,
    localHub = _require2$servers.localHub,
    oldLocalHub = _require2$servers.oldLocalHub;

var setVersion = function setVersion(state) {
	return assoc(state, 'version', version);
};
var getVersion = function getVersion(state) {
	return Let(function () {
		var s = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : get(state, 'version', 0);
		return isString(s) ? 0 : s;
	});
};

var noComposite = function noComposite(state) {
	return assoc(state, 'cohort', state.cohort[0], 'cohortSamples', state.cohortSamples[0]);
};

var spreadsheetProps = ['columnOrder', 'columns', 'mode', 'notifications', 'servers', 'showWelcome', 'wizardMode', 'zoom', 'defaultWidth', 'data', 'cohort', 'cohortSamples', 'km', 'survival', 'sampleSearch', 'samplesOver', 'editing', 'openVizSettings', 'chartState', 'hasPrivateSamples'];

var splitPages = function splitPages(state) {
	return {
		spreadsheet: pick(state, spreadsheetProps),
		page: 'heatmap'
	};
};

// This will break bookmarks with sample search and samples to the right
var samplesToLeft = function samplesToLeft(state) {
	return (
		// We don't want to create a blank spreadsheet object if there is none, so check
		// if there's a columnOrder before doing anything. indexOf could be -1 or 0
		getIn(state, ['spreadsheet', 'columnOrder'], []).indexOf('samples') > 0 ? updateIn(state, ['spreadsheet', 'columnOrder'], function (order) {
			return ['samples'].concat((0, _toConsumableArray3.default)(without(order, 'samples')));
		}) : state
	);
};

var noFieldSpec = function noFieldSpec(state) {
	return get(state, 'spreadsheet') ? updateIn(state, ['spreadsheet', 'columns'], function (columns) {
		return mapObject(columns, function (column) {
			return merge(omit(column, 'fieldSpecs'), getIn(column, ['fieldSpecs', 0], {}));
		});
	}) : state;
};

var noLocalCert = function noLocalCert(state) {
	return getIn(state, ['spreadsheet', 'servers', oldLocalHub]) ? updateIn(state, ['spreadsheet', 'servers'], function (servers) {
		return assoc(omit(servers, oldLocalHub), localHub, servers[oldLocalHub]);
	}) : state;
};

// This must be sorted, with later versions appearing last.
var migrations = [[noComposite, splitPages, samplesToLeft], // to v1
[noFieldSpec], // to v2
[noLocalCert] // to v3
];

function apply(state) {
	var v = getVersion(state),
	    toDo = flatten(migrations.slice(v));

	return setVersion(toDo.reduce(function (prev, fn) {
		return fn(prev);
	}, state));
}

module.exports = apply;