'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.computeSettings = exports.typeWidth = exports.matchDatasetFields = exports.guessFields = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _multi = require('../multi');

var _multi2 = _interopRequireDefault(_multi);

var _xenaQuery = require('../xenaQuery');

var xenaQuery = _interopRequireWildcard(_xenaQuery);

var _rx = require('../rx');

var _underscore_ext = require('../underscore_ext');

var _ = _interopRequireWildcard(_underscore_ext);

var _parsePos = require('../parsePos');

var _parsePos2 = _interopRequireDefault(_parsePos);

var _parseInput = require('../parseInput');

var _parseInput2 = _interopRequireDefault(_parseInput);

var _parseGeneSignature = require('../parseGeneSignature');

var _parseGeneSignature2 = _interopRequireDefault(_parseGeneSignature);

var _fieldSpec = require('../models/fieldSpec');

var _heatmapColors = require('../heatmapColors');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// XXX duplicated in VariableSelect.
var getAssembly = function getAssembly(datasets, dsID) {
	return _.getIn(datasets, [dsID, 'assembly'], _.getIn(datasets, [dsID, 'probemapMeta', 'assembly']));
};

var guessFields = exports.guessFields = function guessFields(text, assembly) {
	var value = text.trim(),
	    sig = (0, _parseGeneSignature2.default)(value),
	    pos = (0, _parsePos2.default)(value, assembly),
	    hasCoord = value.match(/^chr[0-9xyXY]+[pq]?:/),
	    fields = sig ? sig.genes : pos ? [value] : (0, _parseInput2.default)(value);

	return {
		value: value,
		fields: fields,
		sig: sig,
		pos: pos,
		hasCoord: hasCoord
	};
};

var matchDatasetFields = exports.matchDatasetFields = (0, _multi2.default)(function (datasets, dsID) {
	var meta = datasets[dsID];
	// Do field matching against probemap if we have one
	return meta.type === 'genomicMatrix' && meta.probemap ? 'genomicMatrix-probemap' : meta.type;
});

// XXX The error handling here isn't great, because it can leave us with a
// field of the wrong case, e.g. foxm1 vs. FOXM1, or treat a probe as a gene.
// However, it's better to handle it than to lose the observable, which wedges
// the widget. Better handling would warn the user and wait for the network
// error to clear.

// default to probes
matchDatasetFields.dflt = function (datasets, dsID, input) {
	var guess = guessFields(input, getAssembly(datasets, dsID)),
	    fields = guess.fields,
	    pos = guess.pos,
	    warning = pos ? 'position-unsupported' : undefined;

	return xenaQuery.matchFields(dsID, fields).map(function (fields) {
		return (0, _extends3.default)({}, guess, {
			type: 'probes',
			warning: warning,
			fields: fields
		});
	}).catch(function (err) {
		console.log(err);
		return _rx.Observable.of({ type: 'probes', warning: warning, fields: fields }, _rx.Scheduler.asap);
	});
};

var geneProbeMatch = function geneProbeMatch(host, dsID, probemap, guess) {
	return _rx.Observable.zip(xenaQuery.matchGenesWithProbes(dsID, guess.fields), xenaQuery.matchFields(dsID, guess.fields), function (genes, probes) {
		return _.filter(probes).length > _.filter(genes).length ? (0, _extends3.default)({}, guess, { type: 'probes', fields: probes }) : (0, _extends3.default)({}, guess, { type: 'genes', fields: genes });
	}).catch(function (err) {
		console.log(err);
		return _rx.Observable.of((0, _extends3.default)({}, guess, { type: 'genes' }), _rx.Scheduler.asap);
	});
};

var MAX_PROBES = 500;
var chromLimit = function chromLimit(host, probemap, guess) {
	return xenaQuery.maxRange(host, probemap, guess.pos.chrom, guess.pos.baseStart, guess.pos.baseEnd, MAX_PROBES).map(function (_ref) {
		var _ref2 = (0, _slicedToArray3.default)(_ref, 1),
		    end = _ref2[0];

		return (0, _extends3.default)({}, guess, {
			type: 'chrom'
		}, end != null ? {
			warning: 'too-many-probes',
			start: guess.pos.baseStart,
			end: end - 1 } : {});
	});
};

matchDatasetFields.add('genomicMatrix-probemap', function (datasets, dsID, input) {
	var guess = guessFields(input, getAssembly(datasets, dsID)),
	    _JSON$parse = JSON.parse(dsID),
	    host = _JSON$parse.host,
	    probemap = datasets[dsID].probemap;

	return guess.pos ? chromLimit(host, probemap, guess) : geneProbeMatch(host, dsID, probemap, guess);
});

var matchAnyPosition = function matchAnyPosition(guess) {
	return _rx.Observable.of((0, _extends3.default)({}, guess, { type: 'chrom' }), _rx.Scheduler.asap);
};

var normalizeGenes = function normalizeGenes(host, dsID, guess) {
	return xenaQuery.sparseDataMatchField(host, 'name2', dsID, guess.fields).map(function (fields) {
		return (0, _extends3.default)({}, guess, {
			type: 'genes',
			fields: fields
		});
	});
};

function matchWithAssembly(datasets, dsID, input) {
	var guess = guessFields(input, getAssembly(datasets, dsID)),
	    ref = xenaQuery.refGene[datasets[dsID].assembly];
	return (guess.pos ? matchAnyPosition(guess) : normalizeGenes(ref.host, ref.name, guess)).catch(function (err) {
		console.log(err);
		return _rx.Observable.of((0, _extends3.default)({}, guess, { type: 'genes' }), _rx.Scheduler.asap);
	});
}

matchDatasetFields.add('genomicSegment', matchWithAssembly);
matchDatasetFields.add('mutationVector', matchWithAssembly);

function matchPhenotype(datasets, dsID, input) {
	return _rx.Observable.of({ value: input, fields: [input] }, _rx.Scheduler.asap);
}

matchDatasetFields.add('clinicalMatrix', matchPhenotype);

// need to handle
// phenotypic,
// null field, null dataset
// sparse,
// dense with probemap,
// dense without probemap

var typeWidth = exports.typeWidth = {
	matrix: 136,
	chrom: 200
};

var typeClass = function typeClass(dataset) {
	return _.contains(['mutationVector', 'segmented'], dataset.type) ? 'chrom' : 'matrix';
};

// 'features' is a problem here, because they are not unique across datasets.
// How do we look up features w/o a dataset?
function getValueType(dataset, features, fields) {
	var type = dataset.type,
	    valuetype = _.getIn(features, [fields[0], 'valuetype']);

	return type === 'mutationVector' ? 'mutation' : type === 'genomicSegment' ? 'segmented' : type === 'clinicalMatrix' && valuetype === 'category' ? 'coded' : 'float';
}

function getFieldType(dataset, fields, matches, pos) {
	var sig = matches.sig,
	    type = matches.type;

	if (sig) {
		return type;
	}
	if (dataset.type === 'mutationVector') {
		return dataset.dataSubType.search(/SV|structural/i) !== -1 ? 'SV' : 'mutation';
	}
	if (dataset.type === 'genomicSegment') {
		return 'segmented';
	}
	if (dataset.type === 'clinicalMatrix') {
		return 'clinical';
	}
	// We treat probes in chrom view (pos) as geneProbes
	return type === 'probes' ? 'probes' : fields.length > 1 && !pos ? 'genes' : 'geneProbes';
}

function sigFields(fields, _ref3) {
	var genes = _ref3.genes,
	    weights = _ref3.weights;

	var matches = new _set2.default(fields.map(function (s) {
		return s.toLowerCase();
	}));
	return {
		missing: genes.filter(function (g) {
			return !matches.has(g.toLowerCase());
		}),
		genes: fields,
		weights: weights.filter(function (p, i) {
			return matches.has(genes[i].toLowerCase());
		})
	};
}

// use vizSettings set in .json metadata
var getDefaultVizSettings = function getDefaultVizSettings(meta) {
	return _.has(meta, 'min') && _.has(meta, 'max') ? { vizSettings: _.pick(meta, 'min', 'max', 'minstart', 'maxstart') } : _.has(meta, 'origin') && _.has(meta, 'thresh') && _.has(meta, 'max') ? { vizSettings: _.pick(meta, 'origin', 'max', 'thresh') } : {};
};

var xenaField = function xenaField(datasets, features, settings) {
	return (0, _extends3.default)({}, settings, {
		fetchType: 'xena'
	}, _.getIn(settings.dataset, ['probemapMeta', 'dataSubType']) === 'regulon' ? { clustering: 'probes' } : {}, getDefaultVizSettings(settings.dataset), {
		valueType: getValueType(settings.dataset, features[settings.dsID], settings.fields),
		assembly: getAssembly(datasets, settings.dsID)
	});
};

function columnSettings(datasets, features, dsID, matches) {
	var fields = matches.fields,
	    value = matches.value,
	    pos = matches.pos,
	    sig = matches.sig,
	    dataset = datasets[dsID],
	    fieldType = getFieldType(dataset, fields, matches, pos),
	    _ref4 = sig ? sigFields(fields, sig) : {},
	    missing = _ref4.missing,
	    genes = _ref4.genes,
	    weights = _ref4.weights,
	    normalizedFields = sig ? [value] : pos ? [pos.chrom + ':' + pos.baseStart + '-' + pos.baseEnd] : _.contains(['segmented', 'mutation', 'SV'], fieldType) ? [fields[0]] : fields,
	    columnLabel = !dataset.dataSubType ? dataset.label : dataset.dataSubType.match(/phenotype/i) ? '' : dataset.dataSubType + ' - ' + dataset.label,
	    fieldLabel = sig ? 'signature' + (_.isEmpty(missing) ? '' : ' (missing terms: ' + missing.join(', ') + ')') : dataset.type === 'clinicalMatrix' ? _.getIn(features, [dsID, fields[0], 'longtitle']) || fields[0] : normalizedFields.join(', '),
	    defaults = (0, _extends3.default)({}, ['geneProbes', 'segmented', 'SV'].indexOf(fieldType) !== -1 ? { showIntrons: true } : {}, {
		colorClass: _heatmapColors.defaultColorClass,
		columnLabel: columnLabel,
		dataset: dataset,
		defaultNormalization: dataset.colnormalization,
		dsID: dsID,
		fields: normalizedFields,
		fieldLabel: fieldLabel,
		fieldType: fieldType,
		user: { columnLabel: columnLabel, fieldLabel: fieldLabel },
		value: value,
		width: typeWidth[typeClass(dataset)]
	});

	return sig ? (0, _fieldSpec.signatureField)(fieldLabel, (0, _extends3.default)({}, defaults, {
		signature: ['geneSignature', dsID, genes, weights],
		missing: missing
	})) : xenaField(datasets, features, defaults);
}

var computeSettings = exports.computeSettings = _.curry(function (datasets, features, opts, dataset, matches) {
	var settings = columnSettings(datasets, features, dataset, matches);

	// XXX need a way to validate settings that depend on column type, i.e.
	// fieldType geneProbes only works for matrix with probemap.
	// Or, a possible refactor of the schema to make this simpler?
	return _.assocIn.apply(_, [settings].concat((0, _toConsumableArray3.default)((opts || []).flatten())));
});