'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('../underscore_ext');
var multi = require('../multi');

var _require = require('../colorScales'),
    colorScale = _require.colorScale;

var km = require('../km'); // move km down?

var _require2 = require('../color_helper'),
    RGBToHex = _require2.RGBToHex;
//var {segmentAverage} = require('./segmented');

var MAX = 10; // max number of groups to display.

var getSplits = function getSplits(splits) {
	return splits ? splits : 2;
};

var survivalOptions = {
	"osEv": {
		patient: 'patient',
		ev: 'osEv',
		tte: 'osTte',
		evFeature: 'OS',
		tteFeature: 'OS.time',
		label: 'Overall survival'
	},
	"dssEv": {
		patient: 'patient',
		ev: 'dssEv',
		tte: 'dssTte',
		evFeature: 'DSS',
		tteFeature: 'DSS.time',
		label: 'Disease specific survival'
	},
	"ddfsEv": {
		patient: 'patient',
		ev: 'ddfsEv',
		tte: 'ddfsTte',
		evFeature: 'DDFS',
		tteFeature: 'DDFS.time',
		label: 'Distant disease free survival'
	},
	"dmfsEv": {
		patient: 'patient',
		ev: 'dmfsEv',
		tte: 'dmfsTte',
		evFeature: 'DMFS',
		tteFeature: 'DMFS.time',
		label: 'Distant metastasis free survival'
	},
	"idfsEv": {
		patient: 'patient',
		ev: 'idfsEv',
		tte: 'idfsTte',
		evFeature: 'IDFS',
		tteFeature: 'IDFS.time',
		label: 'Invasive disease free survival'
	},
	"dfiEv": {
		patient: 'patient',
		ev: 'dfiEv',
		tte: 'dfiTte',
		evFeature: 'DFI',
		tteFeature: 'DFI.time',
		label: 'Disease free interval'
	},
	"pfiEv": {
		patient: 'patient',
		ev: 'pfiEv',
		tte: 'pfiTte',
		evFeature: 'PFI',
		tteFeature: 'PFI.time',
		label: 'Progression free interval'
	},
	"pfsEv": {
		patient: 'patient',
		ev: 'pfsEv',
		tte: 'pfsTte',
		evFeature: 'PFS',
		tteFeature: 'PFS.time',
		label: 'Progression free survival'
	},
	"lriEv": {
		patient: 'patient',
		ev: 'lriEv',
		tte: 'lriTte',
		evFeature: 'LRI',
		tteFeature: 'LRI.time',
		label: 'Local recurrence interval'
	},
	"rrEv": {
		patient: 'patient',
		ev: 'rrEv',
		tte: 'rrTte',
		evFeature: 'RR',
		tteFeature: 'RR.time',
		label: 'Regional recurrence'
	},
	"driEv": {
		patient: 'patient',
		ev: 'driEv',
		tte: 'driTte',
		evFeature: 'DRI',
		tteFeature: 'DRI.time',
		label: 'Distant recurrence interval'
	},
	"dmiEv": {
		patient: 'patient',
		ev: 'dmiEv',
		tte: 'dmiTte',
		evFeature: 'DMI',
		tteFeature: 'DMI.time',
		label: 'Distant metastasis interval'
	},
	"mEv": {
		patient: 'patient',
		ev: 'mEv',
		tte: 'mTte',
		evFeature: 'Metastasis',
		tteFeature: 'Metastasis.time',
		label: 'Metastasis'
	},
	"rEv": {
		patient: 'patient',
		ev: 'rEv',
		tte: 'rTte',
		evFeature: 'Relapse',
		tteFeature: 'Relapse.time',
		label: 'Relapse'
	},
	"ev": {
		patient: 'patient',
		ev: 'ev',
		tte: 'tte',
		evFeature: '_EVENT',
		tteFeature: '_TIME_TO_EVENT',
		label: 'Survival'
	}
};

function average(data) {
	return data[0].map(function (v, s) {
		return _.meannull(data.map(function (p) {
			return p[s];
		}));
	});
}

// Currently assumes we have only one field. Right now we only
// handle coded phenotype data, and limit it to one-per-column.
function codedVals(_ref) {
	var heatmap = _ref.heatmap,
	    colors = _ref.colors,
	    codes = _ref.codes;

	var groups = _.range(codes.length),
	    colorfn = _.first(colors.map(colorScale));

	return {
		groups: groups,
		colors: groups.map(colorfn),
		labels: codes,
		values: heatmap[0]
	};
}

var saveNull = function saveNull(fn) {
	return function (v) {
		return v == null ? v : fn(v);
	};
};

// XXX Here we include samples that might not have tte and ev, when
// picking the range. Ideal would be to partition not including
// samples w/o survival data.
function partitionedVals3(avg, uniq, colorfn) {
	//eslint-disable-line no-unused-vars
	var vals = _.without(avg, null, undefined).sort(function (a, b) {
		return a - b;
	}),
	    min = _.min(vals),
	    max = _.max(vals),
	    low = vals[Math.round(vals.length / 3)],
	    high = vals[Math.round(2 * vals.length / 3)],
	    labelLow = low.toPrecision(4),
	    labelHigh = high.toPrecision(4);
	return {
		values: _.map(avg, saveNull(function (v) {
			return v < low ? 'low' : v < high ? 'middle' : 'high';
		})),
		groups: ['low', 'middle', 'high'],
		colors: [colorfn(min), colorfn((min + max) / 2), colorfn(max)],
		labels: ['< ' + labelLow, labelLow + ' to ' + labelHigh, '>= ' + labelHigh]
	};
}

function partitionedValsQuartile(avg, uniq, colorfn) {
	var vals = _.without(avg, null, undefined).sort(function (a, b) {
		return a - b;
	}),
	    min = _.min(vals),
	    max = _.max(vals),
	    low = vals[Math.round(vals.length / 4)],
	    high = vals[Math.round(3 * vals.length / 4)],
	    labelLow = low.toPrecision(4),
	    labelHigh = high.toPrecision(4);
	return {
		values: _.map(avg, saveNull(function (v) {
			return v < low ? 'low' : v > high ? 'high' : null;
		})),
		groups: ['low', 'high'],
		colors: [colorfn(min), colorfn(max)],
		labels: ['< ' + labelLow, '> ' + labelHigh]
	};
}

function partitionedVals2(avg, uniq, colorfn) {
	var vals = _.without(avg, null, undefined).sort(function (a, b) {
		return a - b;
	}),
	    min = _.min(vals),
	    max = _.max(vals),
	    mid = vals[Math.round(vals.length / 2)],
	    labelMid = mid.toPrecision(4);
	return {
		values: _.map(avg, saveNull(function (v) {
			return v < mid ? 'low' : 'high';
		})),
		groups: ['low', 'high'],
		colors: [colorfn(min), colorfn(max)],
		labels: ['< ' + labelMid, '>= ' + labelMid]
	};
}

function floatVals(avg, uniq, colorfn) {
	return {
		values: avg,
		groups: uniq,
		colors: uniq.map(colorfn),
		labels: uniq.map(function (v) {
			return v.toPrecision(6);
		})
	};
}

// We use data.display so the values reflect normalization settings.
// We average 1st, then see how many unique values there are, then decide
// whether to partition or not.
function floatOrPartitionVals(_ref2, data, index, samples, splits) {
	var heatmap = _ref2.heatmap,
	    colors = _ref2.colors;

	var clarification = heatmap.length > 1 ? 'average' : undefined,
	    avg = average(heatmap),
	    uniq = _.without(_.uniq(avg), null, undefined),
	    colorfn = _.first(colors.map(colorScale)),
	    partFn = splits === -4 ? partitionedValsQuartile : splits === 3 ? partitionedVals3 : partitionedVals2,
	    maySplit = uniq.length > MAX;
	return (0, _extends3.default)({ clarification: clarification, maySplit: maySplit }, (maySplit ? partFn : floatVals)(avg, uniq, colorfn));
}

function mutationVals(column, data, _ref3, sortedSamples) {
	var bySample = _ref3.bySample;

	var mutCode = _.mapObject(bySample, function (vs) {
		return vs.length > 0 ? 1 : 0;
	});
	return {
		values: _.map(sortedSamples, function (s) {
			return mutCode[s];
		}),
		groups: [0, 1],
		colors: ["#ffffff", // white
		"#9467bd" // dark purple
		],
		labels: ['No Mutation', 'Has Mutation']
	};
}

//var avgOrNull = (rows, xzoom) => _.isEmpty(rows) ? null : segmentAverage(rows, xzoom);

function segmentedVals(column, data, index, samples, splits) {
	var color = column.color,
	    avg = _.getIn(data, ['avg', 'geneValues', 0]),
	    bySampleSortAvg = samples.map(function (sample) {
		return avg[sample];
	}),
	    uniq = _.without(_.uniq(avg), null, undefined),
	    scale = colorScale(color),
	    _color = (0, _slicedToArray3.default)(color, 5),
	    origin = _color[4],
	    colorfn = function colorfn(v) {
		return RGBToHex.apply(undefined, (0, _toConsumableArray3.default)(v < origin ? scale.lookup(0, origin - v) : scale.lookup(1, v - origin)));
	},
	    partFn = splits === -4 ? partitionedValsQuartile : splits === 3 ? partitionedVals3 : partitionedVals2;

	return (0, _extends3.default)({ maySplit: true }, partFn(bySampleSortAvg, uniq, colorfn));
}

var toCoded = multi(function (fs) {
	return fs.valueType;
});
toCoded.add('float', floatOrPartitionVals);
toCoded.add('coded', codedVals);
toCoded.add('mutation', mutationVals);
toCoded.add('segmented', segmentedVals);

var has = function has(obj, v) {
	return obj[v] != null;
};

// Give tte and ev for each subgroup, compute p-value.
// Not sure why we recombine the data after splitting by group.
function pValue(groupsTte, groupsEv) {
	var allTte = _.flatten(groupsTte),
	    allEv = _.flatten(groupsEv);

	return km.logranktest(
	// Only use points with an event
	km.compute(allTte, allEv).filter(function (point) {
		return point.e;
	}), groupsTte, groupsEv);
}

function warnDupPatients(usableSamples, samples, patient) {
	var getPatient = function getPatient(i) {
		return patient[samples[i]];
	},
	    havePatient = usableSamples.filter(getPatient),
	    dups = _.difference(havePatient, _.uniq(havePatient, false, getPatient));

	return dups.length ? 'Some individuals\' survival data are used more than once in the KM plot.\n\t\tThere are ' + dups.length + ' samples affected.\n\t\tFor more information and how to remove such duplications,\n\t\tsee https://ucsc-xena.gitbook.io/project/how-do-i/how-do-i-remove-duplicate-samples-from-a-km-plot .' : null;
}

function filterByGroups(feature, groupedIndices) {
	var labels = feature.labels,
	    colors = feature.colors,
	    groups = feature.groups,
	    clarification = feature.clarification,
	    notEmpty = _.range(groups.length).filter(function (i) {
		return _.has(groupedIndices, groups[i]);
	}),
	    useIndices = notEmpty.slice(0, MAX),
	    nlabels = useIndices.map(function (i) {
		return labels[i];
	}),
	    ncolors = useIndices.map(function (i) {
		return colors[i];
	}),
	    ngroups = useIndices.map(function (i) {
		return groups[i];
	});

	return {
		labels: nlabels,
		colors: ncolors,
		groups: ngroups,
		warning: notEmpty.length > MAX ? 'Limited drawing to ' + MAX + ' categories' : undefined,
		clarification: clarification
	};
}

var getFieldData = function getFieldData(survival) {
	return _.mapObject(survival, function (f) {
		return _.getIn(f, ['data', 'req', 'values', 0]);
	});
};

function cutoffData(survivalData, cutoff) {
	if (cutoff == null) {
		return survivalData;
	}

	var tte = survivalData.tte,
	    ev = survivalData.ev,
	    patient = survivalData.patient,
	    ctte = _.map(tte, function (tte) {
		return tte >= cutoff ? cutoff : tte;
	}),
	    cev = _.map(ev, function (ev, i) {
		return tte[i] >= cutoff ? 0 : ev;
	});

	return {
		tte: ctte,
		ev: cev,
		patient: patient
	};
}

function findSurvDataByType(survivalData, survivalType) {
	var eligibleSurv = _.keys(survivalOptions);

	survivalType = survivalType ? survivalType : _.intersection(_.keys(survivalData), eligibleSurv)[0];

	if (eligibleSurv.indexOf(survivalType) === -1) {
		return null;
	}

	return {
		patient: survivalData[survivalOptions[survivalType].patient],
		tte: survivalData[survivalOptions[survivalType].tte],
		ev: survivalData[survivalOptions[survivalType].ev]
	};
}

var bounds = function bounds(x) {
	return [_.minnull(x), _.maxnull(x)];
};

// After toCoded, we can still end up with empty groups if
// we don't have survival data for the samples in question.
// So, picking MAX groups after filtering for survival data.
// Order should be
// 1) convert to coded feature
// 2) filter usable samples
// 3) drop empty groups
// 4) pick at-most MAX groups
// 5) compute km

function makeGroups(column, data, index, cutoff, splits, survivalType, survival, samples) {
	var survivalData = findSurvDataByType(getFieldData(survival), survivalType),
	    domain = bounds(survivalData.tte),
	    _cutoffData = cutoffData(survivalData, cutoff),
	    tte = _cutoffData.tte,
	    ev = _cutoffData.ev,
	    patient = _cutoffData.patient,
	    codedFeat = toCoded(column, data, index, samples, getSplits(splits)),
	    values = codedFeat.values,
	    usableSamples = _.filterIndices(samples, function (s, i) {
		return has(tte, s) && has(ev, s) && has(values, i);
	}),
	    patientWarning = warnDupPatients(usableSamples, samples, patient),
	    groupedIndices = _.groupBy(usableSamples, function (i) {
		return values[i];
	}),
	    usableData = filterByGroups(codedFeat, groupedIndices),
	    groups = usableData.groups,
	    colors = usableData.colors,
	    labels = usableData.labels,
	    warning = usableData.warning,
	    clarification = usableData.clarification,
	    gtte = groups.map(function (g) {
		return groupedIndices[g].map(function (i) {
			return tte[samples[i]];
		});
	}),
	    gev = groups.map(function (g) {
		return groupedIndices[g].map(function (i) {
			return ev[samples[i]];
		});
	}),
	    curves = groups.map(function (g, i) {
		return km.compute(gtte[i], gev[i]);
	}),
	    pV = pValue(gtte, gev);


	return (0, _extends3.default)({
		colors: colors,
		labels: labels,
		curves: curves,
		warning: warning,
		clarification: clarification,
		patientWarning: patientWarning,
		domain: domain,
		maySplit: codedFeat.maySplit
	}, pV);
}

var toDsID = function toDsID(host, name) {
	return (0, _stringify2.default)({ host: host, name: name });
};

var allFeatures = function allFeatures(cohortFeatures) {
	return _.flatmap(cohortFeatures, function (datasets, server) {
		return _.flatmap(datasets, function (features, dataset) {
			return _.map(features, function (f) {
				return { dsID: toDsID(server, dataset), name: f.name };
			});
		});
	});
};

var featuresByName = function featuresByName(cohortFeatures) {
	return _.Let(function () {
		var all = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : allFeatures(cohortFeatures);
		return _.object(_.pluck(all, 'name'), all);
	});
};

// 'user' is passed the spreadsheet.km object, which
// holds user override of the survival columns.
// XXX This is all a bit wrong, because feature names are not unique
// across datasets.
function pickSurvivalVars(cohortFeatures, user) {
	var byName = featuresByName(cohortFeatures),
	    patient = byName._PATIENT || byName.sampleID,
	    featureMapping = _.flatmap(survivalOptions, function (option) {
		return [[option.ev, _.getIn(user, [option.ev], byName[option.evFeature])], [option.tte, _.getIn(user, [option.tte], byName[option.tteFeature])]];
	});

	return _.assoc(_.object(featureMapping), 'patient', _.getIn(user, ['patient'], patient));
}

module.exports = { makeGroups: makeGroups, pickSurvivalVars: pickSurvivalVars, survivalOptions: survivalOptions, getSplits: getSplits };