'use strict';

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Domain logic for mutation datasets.

var _ = require('../underscore_ext');
var widgets = require('../columnWidgets');
var xenaQuery = require('../xenaQuery');
var Rx = require('../rx');
var exonLayout = require('../exonLayout');
var intervalTree = require('static-interval-tree');

var _require = require('../layoutPlot'),
    pxTransformInterval = _require.pxTransformInterval;

var _require2 = require('../color_helper'),
    hexToRGB = _require2.hexToRGB,
    colorStr = _require2.colorStr;

var jStat = require('jStat').jStat;
var parsePos = require('../parsePos');

//function groupedLegend(colorMap, valsInData) { //eslint-disable-line no-unused-vars
//	var inData = new Set(valsInData),
//		groups = _.groupBy(
//			_.filter(_.keys(colorMap), val => inData.has(val)), k => colorMap[k]),
//		colors = _.keys(groups);
//	return {
//		colors,
//		labels: _.map(colors, c => groups[c].join(', ')),
//		align: 'left'
//	};
//}
//
//function sortedLegend(colorMap, valsInData) { //eslint-disable-line no-unused-vars
//	var inData = new Set(valsInData),
//		colorList = _.filter(_.pairs(colorMap), ([val]) => inData.has(val))
//			.sort(([, c0], [, c1]) => c0 > c1 ? -1 : 1);
//	return {
//		colors: _.pluck(colorList, 1),
//		labels: _.pluck(colorList, 0),
//		align: 'left'
//	};
//}

var colors = {
	categoryMutation: [["#9b9b9b", 'Intron/RNA'], // darker grey code: 0
	["#2CA02C", 'Silent'], // green  code:1
	["#1F77B4", 'Missense/Inframe'], // blue  code:2
	["#FF7F0E", 'Splice'], // orange  code:3
	["#D62728", 'Deleterious'], // red  code:4
	["#c5b0d5", "Complex/Other/Unannotated"] // light lavender missing/no code
	],
	af: { r: 255, g: 0, b: 0 },
	SVdefaultColor: "#c5b0d5", // light lavender
	missing: "#c5b0d5" // light lavender for no impact annotation or annotation outside of impact code
};

var getSNVEffect = function getSNVEffect(colorMap, effect) {
	// match case insensitive, beginning of word
	var effectKey = _.keys(colorMap).find(function (key) {
		var patt = new RegExp('^' + key, "i");
		return patt.test(effect);
	});
	return effectKey;
};

function impactLegend(colorMap, valsInData) {
	var missing = _.filter(valsInData, function (v) {
		return !getSNVEffect(colorMap, v);
	}),
	    colorList = _.filter(_.pairs(colorMap), function (_ref) {
		var _ref2 = (0, _slicedToArray3.default)(_ref, 1),
		    val = _ref2[0];

		var patt = new RegExp('^' + val, "i");
		return valsInData.find(function (effect) {
			return patt.test(effect);
		});
	}).reverse(); // Legend reverses

	if (missing.length !== 0) {
		colorList = _.merge(colorList, [["unannotated", colors.missing]]);
	}

	//groupBy color
	var colorGroups = _.mapObject(_.groupBy(colorList, function (_ref3) {
		var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
		    color = _ref4[1];

		return color;
	}), function (val) {
		return _.pluck(val, 0);
	}),
	    mutationColors = _.keys(colorGroups),
	    // color square
	titles = _.values(colorGroups).map(function (impactList) {
		return impactList.join("\n");
	}),
	    // mouse over text
	labels = mutationColors.map(function (color) {
		return _.find(colors.categoryMutation, function (_ref5) {
			var _ref6 = (0, _slicedToArray3.default)(_ref5, 1),
			    c = _ref6[0];

			return color === c;
		})[1];
	}); //visible text


	return {
		colors: mutationColors,
		titles: titles,
		labels: labels
	};
}

var impact = {
	//destroy protein
	'Nonsense': 4,
	'stop_gained': 4,
	'Frameshift': 4,
	'Frame_Shift': 4,
	'De_novo_Start_OutOfFrame': 4,

	//splice related
	'splice': 3,

	//modify protein
	'missense': 2,
	'NON_SYNONYMOUS': 2,
	'NONSYNONYMOUS': 2,
	'MultiAAMissense': 2,
	'start_lost': 2,
	'start_gained': 2,
	'STOP_LOST': 2,
	'Nonstop_Mutation': 2,
	'Translation_Start_Site': 2,
	'CdsStartSNV': 2,
	'De_novo_Start_InFrame': 2,
	'initiator_codon': 2,
	'5_prime_UTR_premature_start_codon_gain': 2,
	'disruptive_inframe_deletion': 2,
	'disruptive_inframe_insertion': 2,
	'inframe': 2,
	'In_Frame': 2,
	'CODON': 2,

	//do not modify protein
	'Synonymous': 1,
	'Silent': 1,
	'stop_retained': 1,
	'TF_BINDING_SITE': 1,

	//mutations effect we don't know
	'lincRNA': 0,
	'RNA': 0,
	'exon_variant': 0,
	'NON_CODING_EXON': 0,
	'upstream': 0,
	'downstream': 0,
	"5'Flank": 0,
	"3'Flank": 0,
	"3'UTR": 0,
	"5'UTR": 0,
	'UTR_3_PRIME': 0,
	'UTR_5_PRIME': 0,
	'5_prime_UTR': 0,
	'3_prime_UTR': 0,
	'intron': 0,
	'intergenic': 0,
	'IGR': 0,
	'INTRAGENIC': 0
},
    chromColorGB = { //genome browser chrom coloring
	"1": "#996600",
	"2": "#666600",
	"3": "#99991E",
	"4": "#CC0000",
	"5": "#FF0000",
	"6": "#FF00CC",
	"7": "#FFCCCC",
	"8": "#3FF990",
	"9": "#FFCC00",
	"10": "#FFFF00",
	"11": "#CCFF00",
	"12": "#00FF00",
	"13": "#358000",
	"14": "#0000CC",
	"15": "#6699FF",
	"16": "#99CCFF",
	"17": "#00FFFF",
	"18": "#CCFFFF",
	"19": "#9900CC",
	"20": "#CC33FF",
	"21": "#CC99FF",
	"22": "#666666",
	"X": "#999999",
	"Y": "#CCCCCC",
	"M": "#CCCC99"
},
    chromColorPCAWG = { // PCAWG chrom coloring
	"1": "#DE47AB",
	"2": "#72BE97",
	"3": "#F7F797",
	"4": "#7C749B",
	"5": "#E85726",
	"6": "#B395F8",
	"7": "#DC8747",
	"8": "#96D53D",
	"9": "#DC85EE",
	"10": "#7D32B3",
	"11": "#88DB68",
	"12": "#78AAF1",
	"13": "#D9C6CA",
	"14": "#336C80",
	"15": "#F7CA44",
	"16": "#32C7C7",
	"17": "#D4C5F2",
	"18": "#995493",
	"19": "#F88B78",
	"20": "#475ECC",
	"21": "#E0BD8C",
	"22": "#9E2800",
	"X": "#F2BBD2",
	"Y": "#B6EBEA"
},
    impactColor = _.mapObject(impact, function (i) {
	return colors.categoryMutation[i][0];
}),
    saveUndef = function saveUndef(f) {
	return function (v) {
		return v == null ? v : f(v);
	};
},
    round = Math.round,
    decimateFreq = saveUndef(function (v) {
	return round(v * 31) / 32;
}),
    // reduce to 32 vals
vafLegend = function vafLegend() {
	return {
		colors: [0, 0.5, 1].map(function (a) {
			return colorStr((0, _extends3.default)({}, colors.af, { a: a }));
		}),
		labels: ['0%', '50%', '100%'],
		align: 'center'
	};
},
    getSVLegend = function getSVLegend(chromColorMap) {
	return {
		// have to explicitly call hexToRGB to avoid map passing in index.
		colors: chromColorMap ? _.values(chromColorMap).map(function (h) {
			return hexToRGB(h);
		}).map(colorStr).reverse() : [colors.SVdefaultColor],
		labels: chromColorMap ? _.keys(chromColorMap).map(function (key) {
			return "chr" + key;
		}).reverse() : ['structural variant'],
		align: 'left'
	};
},
    features = {
	impact: {
		get: function get(v) {
			return v.effect;
		},
		color: function color(colorMap, v) {
			return colorMap[getSNVEffect(colorMap, v)] || colors.missing;
		},
		legend: impactLegend
	},
	// dna_vaf and rna_vaf need to be updated to reflect the call params.
	'dna_vaf': {
		get: function get(v) {
			return v.dna_vaf == null ? undefined : decimateFreq(v.dna_vaf);
		},
		color: function color(v) {
			return colorStr(v == null ? colors.missing : _.assoc(colors.af, 'a', v));
		},
		legend: vafLegend
	},
	'rna_vaf': {
		get: function get(v) {
			return v.rna_vaf == null ? undefined : decimateFreq(v.rna_vaf);
		},
		color: function color(v) {
			return colorStr(v == null ? colors.missing : _.assoc(colors.af, 'a', v));
		},
		legend: vafLegend
	}
};

function chromFromAlt(alt) {
	var start = alt.search(/[\[\]]/),
	    end = alt.search(":");
	return alt.slice(start + 1, end).replace(/chr/i, "");
}

function posFromAlt(alt) {
	var end = alt.search(/[\[\]\A\T\G\C]{1,2}$/),
	    start = alt.search(":");
	return alt.slice(start + 1, end);
}

function structuralVariantClass(alt) {
	var firstBase = _.first(alt),
	    lastBase = _.last(alt);
	return firstBase === '[' || firstBase === ']' ? 'left' : lastBase === '[' || lastBase === ']' ? 'right' : null;
}

function joinedVariantDirection(alt) {
	var firstBase = _.first(alt),
	    lastBase = _.last(alt);
	return firstBase === ']' || lastBase === ']' ? 'left' : firstBase === '[' || lastBase === '[' ? 'right' : null;
}

var getExonPadding = function getExonPadding(mutationDataType) {
	if (mutationDataType === "SV") {
		return {
			padTxStart: 2000,
			padTxEnd: 0
		};
	} else if (mutationDataType === "mutation") {
		return {
			padTxStart: 1000,
			padTxEnd: 1000
		};
	} else {
		return {
			padTxStart: 1000,
			padTxEnd: 1000
		};
	}
};

function evalMut(flip, mut) {
	return {
		//		impact: features.impact.get(mut),
		right: flip ? -mut.end : mut.start
	};
}

function cmpMut(mut1, mut2) {
	/*
 if (mut1.impact !== mut2.impact) {
 	if (mut1.impact === undefined){
 		return 1;
 	} else if (mut2.impact === undefined) {
 		return -1;
 	} else {
 		return mut2.impact - mut1.impact; // high impact sorts first
 	}
 }
 */
	return mut1.right - mut2.right; // low coord sorts first
}

function rowOrder(row1, row2, flip) {
	var row1a, row2a;

	// Native map is a lot faster than _.map. Need an es5 polyfill, or
	// perhaps lodash, or ramda.
	row1a = row1.map(function (m) {
		return evalMut(flip, m);
	});
	row2a = row2.map(function (m) {
		return evalMut(flip, m);
	});

	return cmpMut(_.maxWith(row1a, cmpMut), _.maxWith(row2a, cmpMut));
}

function cmpRowOrNoVariants(v1, v2, xzoom, flip) {
	var vf1 = v1.filter(function (v) {
		return v.start <= xzoom.end && v.end >= xzoom.start;
	}),
	    vf2 = v2.filter(function (v) {
		return v.start <= xzoom.end && v.end >= xzoom.start;
	});
	if (vf1.length === 0) {
		return vf2.length === 0 ? 0 : 1;
	}
	return vf2.length === 0 ? -1 : rowOrder(vf1, vf2, flip);
}

function cmpRowOrNull(v1, v2, xzoom, flip) {
	if (v1 == null) {
		return v2 == null ? 0 : 1;
	}
	return v2 == null ? -1 : cmpRowOrNoVariants(v1, v2, xzoom, flip);
}

function cmpSamples(xzoom, sample, flip, s1, s2) {
	return cmpRowOrNull(sample[s1], sample[s2], xzoom, flip);
}

// XXX Instead of checking strand here, it should be set as a column
// property as part of the user input: flip if user enters a gene on
// negative strand. Don't flip for genomic range view, or positive strand.
function cmp(column, data, index) {
	var _column$fields = (0, _slicedToArray3.default)(column.fields, 1),
	    field = _column$fields[0],
	    xzoom = column.xzoom,
	    sortVisible = column.sortVisible,
	    pos = parsePos(field),
	    appliedZoom = sortVisible && xzoom ? xzoom : { start: -Infinity, end: Infinity },
	    refGene = _.getIn(data, ['refGene']),
	    flip = !pos && refGene && _.getIn(_.values(refGene), [0, 'strand']) === '-',
	    samples = _.getIn(index, ['bySample']);

	return samples ? function (s1, s2) {
		return cmpSamples(appliedZoom, samples, flip, s1, s2);
	} : function () {
		return 0;
	};
}

var sparseDataRange = xenaQuery.sparseDataRange,
    refGeneRange = xenaQuery.refGeneRange;

// XXX Might want to optimize this before committing. We could mutate in-place
// without affecting anyone. This may be slow for large mutation datasets.
//
// Map sampleIDs to index into 'samples' array.

function mapSamples(samples, data) {
	var sampleMap = _.object(samples, _.range(samples.length));

	return _.updateIn(data, ['req', 'rows'], function (rows) {
		return _.map(rows, function (row) {
			return _.assoc(row, 'sample', sampleMap[row.sample]);
		});
	}, ['req', 'samplesInResp'], function (sIR) {
		return _.map(sIR, function (s) {
			return sampleMap[s];
		});
	});
}

//user input is a gene, get the mutations by the gene's genomic coordinates
function fetchGeneByCoordinate(_ref7, samples) {
	var dsID = _ref7.dsID,
	    fields = _ref7.fields,
	    fieldType = _ref7.fieldType,
	    assembly = _ref7.assembly;

	var _ref8 = xenaQuery.refGene[assembly] || {},
	    name = _ref8.name,
	    host = _ref8.host;

	return name ? xenaQuery.refGeneExons(host, name, fields).flatMap(function (refGene) {
		var coords = _.values(refGene)[0];
		if (!coords) {
			return Rx.Observable.of(null);
		}

		var txStart = coords.txStart,
		    txEnd = coords.txEnd,
		    chrom = coords.chrom,
		    _getExonPadding = getExonPadding(fieldType),
		    padTxStart = _getExonPadding.padTxStart,
		    padTxEnd = _getExonPadding.padTxEnd;

		return sparseDataRange(dsID, samples, chrom, txStart - padTxStart, txEnd + padTxEnd).map(function (req) {
			return mapSamples(samples, { req: req, refGene: refGene });
		});
	}) : Rx.Observable.of(null);
}

/*
//user input is a gene, get the mutations by gene annotation that comes with the data
var {sparseData} = xenaQuery;
function fetchGeneByAnnotation({dsID, fields, assembly}, [samples]) {
	var {name, host} = xenaQuery.refGene[assembly] || {};
	return Rx.Observable.zipArray(
		sparseData(dsID, samples, fields[0]),
		name ? xenaQuery.refGeneExonCase(host, name, fields) : Rx.Observable.of({})
	).map(resp => mapSamples(samples, _.object(['req', 'refGene'], resp)));
}*/

function fetchChrom(_ref9, samples, pos) {
	var dsID = _ref9.dsID,
	    assembly = _ref9.assembly;

	var _ref10 = xenaQuery.refGene[assembly] || {},
	    name = _ref10.name,
	    host = _ref10.host;

	return refGeneRange(host, name, pos.chrom, pos.baseStart, pos.baseEnd).flatMap(function (refGene) {
		return sparseDataRange(dsID, samples, pos.chrom, pos.baseStart, pos.baseEnd).map(function (req) {
			return mapSamples(samples, { req: req, refGene: refGene });
		});
	});
}

function fetch(column, cohortSamples) {
	var pos = parsePos(column.fields[0]),
	    method = pos ? fetchChrom : fetchGeneByCoordinate;
	return method(column, cohortSamples, pos);
}

// Group by, returning groups in sorted order. Scales O(n) vs.
// sort's O(n log n), if the number of values is much smaller than
// the number of elements.
function sortByGroup(arr, keyfn) {
	var grouped = _.groupBy(arr, keyfn);
	return _.map(_.sortBy(_.keys(grouped), _.identity), function (k) {
		return grouped[k];
	});
}

function findSNVNodes(byPosition, layout, colorMap, feature, samples) {
	var sindex = _.object(samples, _.range(samples.length)),
	    _features$feature = features[feature],
	    get = _features$feature.get,
	    color = _features$feature.color,
	    minSize = function minSize(_ref11) {
		var _ref12 = (0, _slicedToArray3.default)(_ref11, 2),
		    s = _ref12[0],
		    e = _ref12[1];

		return [s, e - s < 1 ? s + 1 : e];
	},
	    sortfn = function sortfn(coll, keyfn) {
		return _.flatten(sortByGroup(coll, keyfn), true);
	};

	// _.uniq is something like O(n^2). Using ES6 Set, which should be more like O(n).
	var matches = _.groupBy([].concat((0, _toConsumableArray3.default)(new _set2.default(_.flatmap(layout.chrom, function (_ref13) {
		var _ref14 = (0, _slicedToArray3.default)(_ref13, 2),
		    start = _ref14[0],
		    end = _ref14[1];

		return intervalTree.matches(byPosition, { start: start, end: end });
	})))), function (v) {
		return v.variant.sample;
	});

	return _.flatmap(matches, function (vars) {
		var count = vars.length;

		return sortfn(vars.map(function (v, i) {
			var _minSize = minSize(pxTransformInterval(layout, [v.start, v.end])),
			    _minSize2 = (0, _slicedToArray3.default)(_minSize, 2),
			    xStart = _minSize2[0],
			    xEnd = _minSize2[1];

			return {
				xStart: xStart,
				xEnd: xEnd,
				y: sindex[v.variant.sample],
				color: color(colorMap, get(v.variant)), // needed for sort, before drawing.
				subrow: i,
				rowCount: count,
				data: v.variant
			};
		}), function (v) {
			return v.color;
		});
	});
}

function findSVNodes(byPosition, layout, colorMap, samples) {
	var sindex = _.object(samples, _.range(samples.length)),
	    minSize = function minSize(_ref15) {
		var _ref16 = (0, _slicedToArray3.default)(_ref15, 2),
		    s = _ref16[0],
		    e = _ref16[1];

		return [s, e - s < 1 ? s + 1 : e];
	};

	// _.uniq is something like O(n^2). Using ES6 Set, which should be more like O(n).
	var matches = _.groupBy([].concat((0, _toConsumableArray3.default)(new _set2.default(_.flatmap(layout.chrom, function (_ref17) {
		var _ref18 = (0, _slicedToArray3.default)(_ref17, 2),
		    start = _ref18[0],
		    end = _ref18[1];

		return intervalTree.matches(byPosition, { start: start, end: end });
	})))), function (v) {
		return v.variant.sample;
	});

	return _.flatmap(matches, function (vars) {
		var count = vars.length;

		return vars.map(function (v, i) {
			var _minSize3 = minSize(pxTransformInterval(layout, [v.start, v.end])),
			    _minSize4 = (0, _slicedToArray3.default)(_minSize3, 2),
			    xStart = _minSize4[0],
			    xEnd = _minSize4[1],
			    y = sindex[v.variant.sample],
			    _v$variant = v.variant,
			    chr = _v$variant.chr,
			    alt = _v$variant.alt;

			return {
				xStart: xStart,
				xEnd: xEnd,
				y: y,
				color: colorMap ? colorMap[chromFromAlt(alt)] || colorMap[chr.replace(/chr/i, "")] || colors.SVdefaultColor : colors.SVdefaultColor,
				subrow: i,
				rowCount: count,
				data: v.variant
			};
		});
	});
}

var swapIf = function swapIf(strand, _ref19) {
	var _ref20 = (0, _slicedToArray3.default)(_ref19, 2),
	    x = _ref20[0],
	    y = _ref20[1];

	return strand === '-' ? [y, x] : [x, y];
};

function defaultXZoom(pos, refGene, type) {
	if (pos) {
		// User supplied chrom position
		return {
			start: pos.baseStart,
			end: pos.baseEnd
		};
	}

	var txStart = refGene.txStart,
	    txEnd = refGene.txEnd,
	    strand = refGene.strand,
	    _getExonPadding2 = getExonPadding(type),
	    padTxStart = _getExonPadding2.padTxStart,
	    padTxEnd = _getExonPadding2.padTxEnd,
	    _swapIf = swapIf(strand, [padTxStart, padTxEnd]),
	    _swapIf2 = (0, _slicedToArray3.default)(_swapIf, 2),
	    startPad = _swapIf2[0],
	    endPad = _swapIf2[1];

	return {
		start: txStart - startPad,
		end: txEnd + endPad
	};
}

var getCustomColor = function getCustomColor(dataset, type) {
	return _.getIn(dataset, ['customcolor', type], impactColor);
};

function svDataToDisplay(column, vizSettings, data, sortedSamples, index) {
	var pos = parsePos(column.fields[0]);
	if (_.isEmpty(data) || _.isEmpty(data.req) || !pos && _.isEmpty(data.refGene)) {
		return {};
	}
	var colorMapping = {
		"chromosomeGB": chromColorGB,
		"chromosomePCAWG": chromColorPCAWG
	};
	var getColorMapping = function getColorMapping(svColor) {
		return colorMapping[svColor];
	};

	var refGeneObj = _.values(data.refGene)[0],
	    maxXZoom = defaultXZoom(pos, refGeneObj, 'SV'),
	    width = column.width,
	    _column$showIntrons = column.showIntrons,
	    showIntrons = _column$showIntrons === undefined ? false : _column$showIntrons,
	    _column$xzoom = column.xzoom,
	    xzoom = _column$xzoom === undefined ? maxXZoom : _column$xzoom,
	    createLayout = pos ? exonLayout.chromLayout : showIntrons ? exonLayout.intronLayout : exonLayout.layout,
	    layout = createLayout(refGeneObj, width, xzoom, pos),
	    colorMap = vizSettings && vizSettings.svColor ? getColorMapping(vizSettings.svColor) : undefined,
	    nodes = findSVNodes(index.byPosition, layout, colorMap, sortedSamples);


	return {
		layout: layout,
		nodes: nodes,
		maxXZoom: maxXZoom,
		legend: getSVLegend(colorMap)
	};
}

function snvDataToDisplay(column, vizSettings, data, sortedSamples, index) {
	var pos = parsePos(column.fields[0]);
	if (_.isEmpty(data) || _.isEmpty(data.req) || !pos && _.isEmpty(data.refGene)) {
		return {};
	}
	var refGeneObj = _.values(data.refGene)[0],
	    maxXZoom = defaultXZoom(pos, refGeneObj, 'mutation'),
	    dataset = column.dataset,
	    width = column.width,
	    _column$showIntrons2 = column.showIntrons,
	    showIntrons = _column$showIntrons2 === undefined ? false : _column$showIntrons2,
	    _column$sFeature = column.sFeature,
	    sFeature = _column$sFeature === undefined ? 'impact' : _column$sFeature,
	    _column$xzoom2 = column.xzoom,
	    xzoom = _column$xzoom2 === undefined ? maxXZoom : _column$xzoom2,
	    allVals = _.uniq(data.req.rows.map(features[sFeature].get)),
	    createLayout = pos ? exonLayout.chromLayout : showIntrons ? exonLayout.intronLayout : exonLayout.layout,
	    layout = createLayout(refGeneObj, width, xzoom, pos),
	    colorMap = getCustomColor(dataset, 'SNV'),
	    nodes = findSNVNodes(index.byPosition, layout, colorMap, sFeature, sortedSamples);


	return {
		layout: layout,
		nodes: nodes,
		maxXZoom: maxXZoom,
		legend: features[sFeature].legend(colorMap, allVals)
	};
}

function index(fieldType, data) {
	if (!_.get(data, 'req')) {
		return null;
	}

	var _data$req = data.req,
	    rows = _data$req.rows,
	    samplesInResp = _data$req.samplesInResp,
	    bySample = _.groupBy(rows, 'sample'),
	    empty = []; // use a single empty object.

	rows = rows.map(function (row) {
		var alt = row.alt,
		    virtualStart = row.start,
		    virtualEnd = row.end;

		if (row.start === row.end) {
			// SV vcf starndard: start is equal to end position
			var vclass = structuralVariantClass(alt);
			if (vclass === 'left') {
				//SV: new segment to the left
				virtualStart = -Infinity;
			} else if (vclass === 'right') {
				//SV: new segment on the right
				virtualEnd = Infinity;
			}
		}
		return {
			start: virtualStart,
			end: virtualEnd,
			variant: row
		};
	});

	return {
		byPosition: intervalTree.index(rows),
		bySample: _.object(samplesInResp, samplesInResp.map(function (s) {
			return bySample[s] || empty;
		}))
	};
}

// SNV P value calculation is an approximation Jing's got from the internet, it is not tested against the exact calculation.
// it is ok to use in the mupit viz, but not confident it can be used to represent the true p value
// in addition, the K value should be the size of the gene coding region, or gene's exon region, or the size of the whole gene including introns, currently set as 1000.
function SNVPvalue(rows, total, k) {
	//total: instances, like total number of people in the experiments
	//k: possible variaty, like 365 days for birthday
	var newRows = _.map(rows, function (n) {
		return n.chr + ':' + n.start;
	});
	return _.values(_.mapObject(_.countBy(newRows, function (n) {
		return n;
	}), function (val, key) {
		// a classic birthday problem: https://en.wikipedia.org/wiki/Birthday_problem
		// a strong birthday problem: extend to trio (at least a trio) or Quadruple (at least four) etc
		// Journal of Statistical Planning and Inference 130 (2005) 377 – 389  https://www.math.ucdavis.edu/~tracy/courses/math135A/UsefullCourseMaterial/birthday.pdf
		// Poisson approximation
		//        http://math.stackexchange.com/questions/25876/probability-of-3-people-in-a-room-of-30-having-the-same-birthday/25880#25880
		//        no. 28
		// simulation: http://www.drmoron.org/3-birthday-problem/
		// total = 30;
		// val = 4;
		// k = 365;
		// result ≈0.028537
		// P(365,30,3)≃2,85%
		// P(365,30,4)≃0,0532%
		var T = jStat.combination(total, val) / Math.pow(k, val - 1),
		    pValue = 1 - (Math.exp(-T) + Math.exp(-(T / (1 + val * (total - val) / (2 * k))))) / 2,
		    _key$split = key.split(':'),
		    _key$split2 = (0, _slicedToArray3.default)(_key$split, 2),
		    chr = _key$split2[0],
		    start = _key$split2[1];


		return {
			chr: chr,
			start: start,
			total: total,
			class: k,
			count: val,
			pValue: pValue
		};
	}));
}

////////////////////////////////
// download of on-screen data

function makeRow(fields, sampleGroup, row) {
	var fieldValue = void 0;
	if (_.isArray(sampleGroup) && sampleGroup.length === 0) {
		fieldValue = 'no variant';
	}
	if (_.isEmpty(sampleGroup)) {
		sampleGroup = [row];
	}
	return _.flatmap(sampleGroup, function (row) {
		return _.map(fields, function (f) {
			return row && row[f] || fieldValue;
		});
	});
}

function getRowFields(rows, sampleGroups) {
	if (_.isEmpty(sampleGroups)) {
		return []; // When no samples exist
	} else if (!_.isEmpty(rows)) {
		return _.keys(rows[0]); // When samples have mutation(s)
	} else {
		return ['sample', 'result']; // default fields for mutation-less columns
	}
}

function formatSamples(sampleFormat, rows) {
	return rows ? _.map(rows, function (r) {
		return _.updateIn(r, ['sample'], sampleFormat);
	}) : rows;
}

function download(_ref21) {
	var rows = _ref21.data.req.rows,
	    samples = _ref21.samples,
	    index = _ref21.index,
	    sampleFormat = _ref21.sampleFormat;

	var groupedSamples = _.getIn(index, ['bySample']) || [],
	    rowFields = getRowFields(rows, groupedSamples),
	    allRows = _.map(samples, function (sId) {
		var alternateRow = { sample: sampleFormat(sId) }; // only used for mutation-less samples
		return makeRow(rowFields, formatSamples(sampleFormat, groupedSamples[sId]), alternateRow);
	});
	return [rowFields, allRows];
}

widgets.cmp.add('mutation', cmp);
widgets.index.add('mutation', index);
widgets.transform.add('mutation', snvDataToDisplay);
widgets.download.add('mutation', download);

widgets.cmp.add('SV', cmp);
widgets.index.add('SV', index);
widgets.transform.add('SV', svDataToDisplay);
widgets.download.add('SV', download);

module.exports = {
	features: features,
	chromFromAlt: chromFromAlt,
	posFromAlt: posFromAlt,
	structuralVariantClass: structuralVariantClass,
	joinedVariantDirection: joinedVariantDirection,
	chromColorGB: chromColorGB,
	SNVPvalue: SNVPvalue,
	fetch: fetch,
	impact: impact,
	getSNVEffect: getSNVEffect
};