'use strict';

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _toArray2 = require('babel-runtime/helpers/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('../underscore_ext');
var fieldFetch = require('../fieldFetch');
var Rx = require('../rx');

var _require = require('../xenaQuery'),
    datasetProbeSignature = _require.datasetProbeSignature,
    datasetGeneSignature = _require.datasetGeneSignature,
    nanstr = _require.nanstr;

var immediate = function immediate(x) {
	return Rx.Observable.of(x, Rx.Scheduler.asap);
};

// dispatch on variant type (data, not genetic, variant)
var m = function m(methods, exp, defaultMethod) {
	var _exp = (0, _toArray3.default)(exp),
	    type = _exp[0],
	    args = _exp.slice(1),
	    method = methods[type];

	return method ? method.apply(undefined, (0, _toConsumableArray3.default)(args)) : defaultMethod(exp);
};

// We have some expression in form
// [type, ...exps]
// To be recursive while supporting async, it needs CPS transform.
// So, an expression like
// ['add', ['signature', ds0, ...genes], ['signature', ds1, ...genes]]
// needs to look like
// Rx.zip(fetch(ds0, genes), fetch(ds1, genes)).map(add)
// This would allow signatures across different hubs or datasets,
// for example.

function evalIn(_ref, list) {
	var samples = _ref.samples;

	var set = _.object(list, list); // XXX use Set?
	return {
		req: {
			values: [_.map(samples, function (s) {
				return _.has(set, s) ? 1 : 0;
			})]
		},
		codes: ['false', 'true']
	};
}

// cross product of boolean terms, as text, e.g. for 2 terms,
// !a !b, a !b, !a b, a b
function booleanCross(terms) {
	var i = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
	var acc = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

	return i === terms.length ? acc : booleanCross(terms, i + 1, acc.length === 0 ? ['false', 'true'] : acc.map(function (t) {
		return t + ';false';
	}).concat(acc.map(function (t) {
		return t + ';true';
	})));
}

function evalCross(_ref2, lists, exprs) {
	var samples = _ref2.samples;

	var sets = lists.map(function (l) {
		return new _set2.default(l);
	}),
	    bits = _.times(lists.length, function (i) {
		return 1 << i;
	}); // 1, 2

	return {
		req: {
			values: [_.map(samples, function (s) {
				return _.sum(bits.filter(function (v, i) {
					return sets[i].has(s);
				}));
			})]
		},
		codes: booleanCross(exprs)
	};
}

function samplesAsData(_ref3) {
	var samples = _ref3.samples;

	return {
		req: {
			values: [_.range(samples.length)]
		},
		codes: samples
	};
}

var sigProbeResult = function sigProbeResult(dataValues) {
	var values = _.map(dataValues, nanstr),
	    mean = _.meannull(values);

	return {
		req: {
			values: [values],
			mean: [mean]
		}
	};
};

var sigGeneResult = function sigGeneResult(_ref4) {
	var scores = _ref4.scores;
	return sigProbeResult(scores);
};

var sigFetch = {
	probes: function probes() {
		return datasetProbeSignature.apply(undefined, arguments).map(sigProbeResult);
	},
	genes: function genes() {
		return datasetGeneSignature.apply(undefined, arguments).map(sigGeneResult);
	}
};

function evalexp(ctx, expression) {
	return m({
		'in': function _in(list) {
			return immediate(evalIn(ctx, list));
		},
		'cross': function cross(lists, exprs) {
			return immediate(evalCross(ctx, lists, exprs));
		},
		'samples': function samples() {
			return immediate(samplesAsData(ctx));
		},
		'geneSignature': function geneSignature(dsID, probes, weights) {
			return sigFetch[ctx.column.fieldType](dsID, ctx.samples, probes, weights);
		}
	}, expression);
}

function fetchSignature(column, samples) {
	return evalexp({ samples: samples, column: column }, column.signature);
}

fieldFetch.add('signature', fetchSignature);