'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var styles = require('./spreadsheetStyles');
var widgets = require('./columnWidgets');

require('./pdfMutationVector');
require('./pdfDenseMatrix');
require('./pdfSegmented');
require('./pdfSamples');
var _ = require('./underscore_ext');

var totalWidth = function totalWidth(cols) {
	return (cols.length - 1) * styles.column.margin + _.reduce(_.pluck(cols, 'width'), function (x, y) {
		return x + y;
	}, 0);
};

function getOffsets(cols) {
	var widths = _.pluck(cols, 'width'),
	    m = styles.column.margin;
	return _.scan(widths, function (acc, n) {
		return acc + n + m;
	}, 0);
}

var pdfImgHeight = 2000;

var download = function download(state) {
	require.ensure(['pdfkit', 'blob-stream', './vgpdf'], function () {
		var PDFDocument = require('pdfkit');
		var blobStream = require('blob-stream');
		var vgpdf = require('./vgpdf');
		var columns = state.columnOrder.map(function (id) {
			return state.columns[id];
		}),
		    width = totalWidth(columns),

		// pdfkit zlib is pathologically slow.
		doc = new PDFDocument({ compress: false, size: [width, state.zoom.height] }),
		    stream = doc.pipe(blobStream()),
		    vg = vgpdf(doc, pdfImgHeight * width / state.zoom.height, pdfImgHeight),
		    offsets = getOffsets(columns);

		columns.forEach(function (column, i) {
			return vg.translate(offsets[i], 0, function () {
				widgets.pdf(state.columnOrder[i], column, vg, state, i);
			});
		});
		doc.end();

		stream.on('finish', function () {
			var url = stream.toBlobURL('application/pdf');

			var a = document.createElement('a');
			var filename = 'xenaDownload.pdf';
			(0, _assign2.default)(a, { id: filename, download: filename, href: url });
			document.body.appendChild(a);
			a.click();
			document.body.removeChild(a);
		});
	});
};

exports.default = download;