'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _PureComponent3 = require('./PureComponent');

var _PureComponent4 = _interopRequireDefault(_PureComponent3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('./underscore_ext');
var Rx = require('./rx');
var widgets = require('./columnWidgets');
var colorScales = require('./colorScales');
var util = require('./util');
var Legend = require('./views/Legend');
var BandLegend = require('./views/BandLegend');

var React = require('react');
var CanvasDrawing = require('./CanvasDrawing');

var _require = require('./react-utils'),
    rxEvents = _require.rxEvents;

var _require2 = require('./drawHeatmap'),
    drawHeatmap = _require2.drawHeatmap;

// Since we don't set module.exports, but instead register ourselves
// with columWidgets, react-hot-loader can't handle the updates automatically.
// Accept hot loading here.


if (module.hot) {
	module.hot.accept();
}

// Since there are multiple components in the file we have to use makeHot
// explicitly.
function hotOrNot(component) {
	return module.makeHot ? module.makeHot(component) : component;
}

var colorFns = function colorFns(vs) {
	return _.map(vs, colorScales.colorScale);
};

//
// Tooltip
//

var prec = function () {
	var precision = 6,
	    factor = Math.pow(10, precision);
	return function (val) {
		return val == null ? 'NA' : Math.round(val * factor) / factor;
	};
}();

// We're getting events with coords < 0. Not sure if this
// is a side-effect of the react event system. This will
// restrict values to the given range.
function bounded(min, max, x) {
	return x < min ? min : x > max ? max : x;
}

var posString = function posString(p) {
	return p.chrom + ':' + util.addCommas(p.chromstart) + '-' + util.addCommas(p.chromend);
};
//gb position string of the segment with 15bp extra on each side, centered at segment
var posRegionString = function posRegionString(p) {
	var pad = Math.round((p.chromend - p.chromstart) / 2);
	return p.chrom + ':' + util.addCommas(p.chromstart - pad) + '-' + util.addCommas(p.chromend + pad);
};
var gbURL = function gbURL(assembly, pos, hgtCustomtext, hubUrl) {
	var assemblyString = encodeURIComponent(assembly),
	    positionString = encodeURIComponent(posString(pos)),
	    regionString = encodeURIComponent(posRegionString(pos));
	return 'http://genome.ucsc.edu/cgi-bin/hgTracks?db=' + assemblyString + '\n\t\t\t&highlight=' + assemblyString + '.' + positionString + '\n\t\t\t&position=' + regionString + '\n\t\t\t' + (hubUrl ? '&hubUrl=' + hubUrl : '') + '\n\t\t\t' + (hgtCustomtext ? '&hgt.customText=' + hgtCustomtext : '');
};

function sigTooltip(genes, missing, val) {
	var visibleCount = 2,
	    moreCount = genes.length - visibleCount,
	    visible = genes.slice(0, visibleCount),
	    moreLabel = moreCount > 0 ? ' + ' + moreCount + ' more' : '',
	    missingLabel = _.get(missing, 'length') ? '(missing terms: ' + missing.join(' ') + ') ' : '';
	return [['sig',
	// label on hover
	'signature ' + missingLabel + '(= ' + visible.join(' + ') + moreLabel + ')',
	// label on freeze
	'signature ' + missingLabel + '(= ' + genes.join(' + ') + ')', val]];
}

function tooltip(id, heatmap, avg, assembly, hgtCustomtext, hubUrl, fields, sampleFormat, fieldFormat, codes, position, width, zoom, samples, _ref, ev) {
	var genes = _ref.genes,
	    missing = _ref.missing;

	var coord = util.eventOffset(ev),
	    sampleIndex = bounded(0, samples.length, Math.floor(coord.y * zoom.count / zoom.height + zoom.index)),
	    sampleID = samples[sampleIndex],
	    fieldIndex = bounded(0, fields.length, Math.floor(coord.x * fields.length / width)),
	    pos = _.get(position, fieldIndex),
	    field = fields[fieldIndex],
	    sig = !!genes;

	var val = _.getIn(heatmap, [fieldIndex, sampleIndex]),
	    code = _.get(codes, val),
	    label = sig ? 'signature' : fieldFormat(field);

	val = code ? code : prec(val);
	var mean = avg && prec(avg.mean[fieldIndex]),
	    median = avg && prec(avg.median[fieldIndex]);

	return {
		sampleID: sampleFormat(sampleID),
		id: id,
		fieldIndex: fieldIndex,
		rows: [sig ? sigTooltip(genes, missing, val) : [['labelValue', label, val]]].concat((0, _toConsumableArray3.default)(pos && assembly ? [[['url', assembly + ' ' + posString(pos), gbURL(assembly, pos, hgtCustomtext, hubUrl)]]] : []), (0, _toConsumableArray3.default)(!codes && mean !== 'NA' && median !== 'NA' ? [[['label', 'Mean: ' + mean + ' Median: ' + median]]] : []))
	};
}

//
// Legends
//

function categoryLegend(dataIn, colorScale, codes) {
	if (!colorScale) {
		return { colors: [], labels: [] };
	}
	// only finds categories for the current data in the column
	var data = _.reject(_.uniq(dataIn), function (x) {
		return x == null;
	}).sort(function (v1, v2) {
		return v1 - v2;
	}),
	    colors = _.map(data, colorScale),
	    labels = _.map(data, function (d) {
		return codes[d];
	});

	return { colors: colors, labels: labels, titles: labels };
}

// might want to use <wbr> here, instead, so cut & paste work better, but that
// will require a recursive split/flatmap to inject the <wbr> elements.
var addWordBreaks = function addWordBreaks(str) {
	return str.replace(/([_/])/g, '\u200B$1\u200B');
};

function renderFloatLegend(props) {
	var units = props.units,
	    colors = props.colors,
	    data = props.data,
	    vizSettings = props.vizSettings;


	if (_.isEmpty(data)) {
		return null;
	}

	var colorSpec = _.max(colors, function (colorList) {
		return _.uniq(colorList.slice(Math.ceil(colorList.length / 2.0))).length;
	});

	if (colorSpec[0] === 'no-data') {
		return null;
	}

	var scale = colorScales.colorScale(colorSpec),
	    values = scale.domain(),
	    footnotes = units && units[0] ? [React.createElement(
		'span',
		{ title: units[0] },
		addWordBreaks(units[0])
	)] : null,
	    hasViz = !isNaN(_.getIn(vizSettings, ['min'])),
	    multiScaled = colors && colors.length > 1 && !hasViz;

	return React.createElement(BandLegend, {
		multiScaled: multiScaled,
		range: { min: _.first(values), max: _.last(values) },
		colorScale: scale,
		footnotes: footnotes,
		width: 50,
		height: 20 });
}

// Might have colorScale but no data (phenotype), no data & no colorScale,
// or data & colorScale, no colorScale &  data?
function renderCodedLegend(props) {
	var _props$data = props.data;
	_props$data = _props$data === undefined ? [] : _props$data;

	var _props$data2 = (0, _slicedToArray3.default)(_props$data, 1),
	    data = _props$data2[0],
	    codes = props.codes,
	    _props$colors = props.colors,
	    colors = _props$colors === undefined ? [] : _props$colors;

	var legendProps;
	var colorfn = _.first(colorFns(colors.slice(0, 1)));

	// We can use domain() for categorical, but we want to filter out
	// values not in the plot. Also, we build the categorical from all
	// values in the db (even those not in the plot) so that colors will
	// match in other datasets.
	if (data && colorfn) {
		// category
		legendProps = categoryLegend(data, colorfn, codes);
	} else {
		return React.createElement('span', null);
	}

	return React.createElement(Legend, legendProps);
}

var HeatmapLegend = hotOrNot(function (_PureComponent) {
	(0, _inherits3.default)(_class, _PureComponent);

	function _class() {
		(0, _classCallCheck3.default)(this, _class);
		return (0, _possibleConstructorReturn3.default)(this, (_class.__proto__ || (0, _getPrototypeOf2.default)(_class)).apply(this, arguments));
	}

	(0, _createClass3.default)(_class, [{
		key: 'render',
		value: function render() {
			var column = this.props.column,
			    units = column.units,
			    heatmap = column.heatmap,
			    colors = column.colors,
			    valueType = column.valueType,
			    vizSettings = column.vizSettings,
			    defaultNormalization = column.defaultNormalization,
			    codes = column.codes,
			    props = {
				units: units,
				colors: colors,
				vizSettings: vizSettings,
				defaultNormalization: defaultNormalization,
				data: heatmap,
				coded: valueType === 'coded',
				codes: codes
			};


			return (props.coded ? renderCodedLegend : renderFloatLegend)(props);
		}
	}]);
	return _class;
}(_PureComponent4.default));

//
// plot rendering
//


var HeatmapColumn = hotOrNot( //
// plot rendering
//


function (_PureComponent2) {
	(0, _inherits3.default)(_class3, _PureComponent2);

	function _class3() {
		var _ref2;

		var _temp, _this2, _ret;

		(0, _classCallCheck3.default)(this, _class3);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this2 = (0, _possibleConstructorReturn3.default)(this, (_ref2 = _class3.__proto__ || (0, _getPrototypeOf2.default)(_class3)).call.apply(_ref2, [this].concat(args))), _this2), _this2.tooltip = function (ev) {
			var _this2$props = _this2.props,
			    samples = _this2$props.samples,
			    data = _this2$props.data,
			    column = _this2$props.column,
			    zoom = _this2$props.zoom,
			    sampleFormat = _this2$props.sampleFormat,
			    fieldFormat = _this2$props.fieldFormat,
			    id = _this2$props.id,
			    codes = column.codes,
			    avg = column.avg,
			    position = column.position || _.getIn(data, ['req', 'position']),
			    assembly = column.assembly,
			    fields = column.fields,
			    heatmap = column.heatmap,
			    width = column.width,
			    dataset = column.dataset,
			    missing = column.missing,
			    signature = column.signature,
			    _ref3 = signature || [],
			    _ref4 = (0, _slicedToArray3.default)(_ref3, 3),
			    genes = _ref4[2],
			    hgtCustomtext = _.getIn(dataset, ['probemapMeta', 'hgt.customtext']),
			    hubUrl = _.getIn(dataset, ['probemapMeta', 'huburl']);

			return tooltip(id, heatmap, avg, assembly, hgtCustomtext, hubUrl, fields, sampleFormat, fieldFormat(id), codes, position, width, zoom, samples, { genes: genes, missing: missing }, ev);
		}, _temp), (0, _possibleConstructorReturn3.default)(_this2, _ret);
	}

	(0, _createClass3.default)(_class3, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			var _this3 = this;

			var events = rxEvents(this, 'mouseout', 'mousemove', 'mouseover');

			// Compute tooltip events from mouse events.
			this.ttevents = events.mouseover.filter(function (ev) {
				return util.hasClass(ev.currentTarget, 'Tooltip-target');
			}).flatMap(function () {
				return events.mousemove.takeUntil(events.mouseout).map(function (ev) {
					return {
						data: _this3.tooltip(ev),
						open: true
					};
				}) // look up current data
				.concat(Rx.Observable.of({ open: false }));
			}).subscribe(this.props.tooltip);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.ttevents.unsubscribe();
		}
	}, {
		key: 'render',


		// To reduce this set of properties, we could
		//    - Drop data & move codes into the 'display' obj, outside of data
		// Might also want to copy fields into 'display', so we can drop req probes
		value: function render() {
			var _props = this.props,
			    column = _props.column,
			    zoom = _props.zoom,
			    heatmap = column.heatmap,
			    colors = column.colors,
			    codes = column.codes;


			return React.createElement(CanvasDrawing, {
				ref: 'plot',
				draw: drawHeatmap,
				wrapperProps: {
					className: 'Tooltip-target',
					onMouseMove: this.on.mousemove,
					onMouseOut: this.on.mouseout,
					onMouseOver: this.on.mouseover,
					onClick: this.props.onClick
				},
				codes: codes,
				width: _.get(column, 'width'),
				zoom: zoom,
				colors: colors,
				heatmapData: heatmap });
		}
	}]);
	return _class3;
}(_PureComponent4.default));

var getColumn = function getColumn(props) {
	return React.createElement(HeatmapColumn, props);
};

widgets.column.add("probes", getColumn);
widgets.column.add("geneProbes", getColumn);
widgets.column.add("genes", getColumn);
widgets.column.add("clinical", getColumn);

var getLegend = function getLegend(props) {
	return React.createElement(HeatmapLegend, props);
};

widgets.legend.add('probes', getLegend);
widgets.legend.add('geneProbes', getLegend);
widgets.legend.add('genes', getLegend);
widgets.legend.add('clinical', getLegend);