'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _PureComponent2 = require('./PureComponent');

var _PureComponent3 = _interopRequireDefault(_PureComponent2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('./underscore_ext');
var Rx = require('./rx');

var React = require('react');
var Legend = require('./views/Legend');

var _require = require('./react-utils'),
    rxEvents = _require.rxEvents;

var widgets = require('./columnWidgets');
var util = require('./util');
var CanvasDrawing = require('./CanvasDrawing');
var mv = require('./models/mutationVector');

var _require2 = require('./drawMutations'),
    drawSV = _require2.drawSV,
    drawMutations = _require2.drawMutations,
    radius = _require2.radius,
    toYPx = _require2.toYPx,
    toYPxSubRow = _require2.toYPxSubRow,
    minVariantHeight = _require2.minVariantHeight,
    splitRows = _require2.splitRows;

var _require3 = require('./exonLayout'),
    chromPositionFromScreen = _require3.chromPositionFromScreen;

// Since we don't set module.exports, but instead register ourselves
// with columWidgets, react-hot-loader can't handle the updates automatically.
// Accept hot loading here.


if (module.hot) {
	module.hot.accept();
	module.hot.accept('./models/mutationVector', function () {
		mv = require('./models/mutationVector');
	});
}

// Since there are multiple components in the file we have to use makeHot
// explicitly.
function hotOrNot(component) {
	return module.makeHot ? module.makeHot(component) : component;
}

function drawMutationLegend(_ref) {
	var column = _ref.column;

	if (!column.legend) {
		return null;
	}
	var _column$legend = column.legend,
	    colors = _column$legend.colors,
	    labels = _column$legend.labels,
	    titles = _column$legend.titles,
	    labelheader = "Variant Impact";


	return React.createElement(Legend, {
		colors: colors,
		labels: labels,
		titles: titles,
		addNullNotation: 1,
		labelheader: labelheader });
}

function drawSVLegend(_ref2) {
	var column = _ref2.column;

	if (!column.legend) {
		return null;
	}
	var _column$legend2 = column.legend,
	    colors = _column$legend2.colors,
	    labels = _column$legend2.labels,
	    titles = _column$legend2.titles,
	    labelheader = "Variant Impact";


	return React.createElement(Legend, {
		colors: colors,
		labels: labels,
		titles: titles,
		addBreakend: 1,
		addNullNotation: 1,
		labelheader: labelheader });
}

function closestNodeSNV(nodes, zoom, x, y) {
	var cutoffX = radius,
	    index = zoom.index,
	    height = zoom.height,
	    count = zoom.count,
	    cutoffY = minVariantHeight(height / count) / 2,
	    end = index + count,
	    nearBy = _.filter(nodes, function (n) {
		return n.y >= index && n.y < end && Math.abs(y - toYPx(zoom, n).y) < cutoffY && x > n.xStart - cutoffX && x < n.xEnd + cutoffX;
	});


	var closest = _.max(nearBy, function (n) {
		return mv.impact[mv.getSNVEffect(mv.impact, n.data.effect)] || 0;
	}),

	//multiple records of the same location same sample
	allClosest = _.filter(nearBy, function (n) {
		return n.data.start === closest.data.start && n.data.end === closest.data.end && n.data.sample === closest.data.sample;
	});

	return allClosest;
}

function closestNodeSV(nodes, zoom, x, y) {
	var index = zoom.index,
	    height = zoom.height,
	    count = zoom.count,
	    end = index + count,
	    toY = splitRows(count, height) ? toYPxSubRow : toYPx,
	    underRow = function underRow(v) {
		var _toY = toY(zoom, v),
		    svHeight = _toY.svHeight,
		    suby = _toY.y;

		return Math.abs(y - suby) < svHeight / 2;
	},
	    underMouse = _.filter(nodes, function (n) {
		return n.y >= index && n.y < end && x >= n.xStart && x <= n.xEnd && underRow(n);
	});

	return underMouse;
}

var closestNode = {
	SV: closestNodeSV,
	mutation: closestNodeSNV
};

function formatAf(af) {
	return af === 'NA' || af === '' || af == null ? null : Math.round(af * 100) + '%';
}

var fmtIf = function fmtIf(x, fmt) {
	var d = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
	return _.isString(x) && x !== 'NaN' && x !== '' ? fmt(x) : d;
};
var dropNulls = function dropNulls(rows) {
	return rows.map(function (row) {
		return row.filter(function (col) {
			return col != null;
		});
	}) // drop empty cols
	.filter(function (row) {
		return row.length > 0;
	});
}; // drop empty rows
//gb position string of the segment with 15bp extra on each side, centered at segment
var posRegionString = function posRegionString(p) {
	return p.chr + ':' + util.addCommas(p.start - 15) + '-' + util.addCommas(p.end + 15);
};
//gb position string like chr3:178,936,070-178,936,070
var posDoubleString = function posDoubleString(p) {
	return p.chr + ':' + util.addCommas(p.start) + '-' + util.addCommas(p.end);
};
//gb position string like chr3:178,936,070
var posStartString = function posStartString(p) {
	return p.chr + ':' + util.addCommas(p.start);
};
var gbURL = function gbURL(assembly, pos, highlightPos, GBoptions) {
	// assembly : e.g. hg18
	// pos: e.g. chr3:178,936,070-178,936,070
	// highlight: e.g. chr3:178,936,070-178,936,070
	// GBoptions.hubUrl: To build a URL that will load the hub directly
	// GBoptions.fullTracks: full display mode track list
	var assemblyString = encodeURIComponent(assembly),
	    positionString = encodeURIComponent(pos),
	    highlightString = encodeURIComponent(highlightPos),
	    hubString = GBoptions && GBoptions.assembly === assembly && GBoptions.hubUrl ? "&hubUrl=" + encodeURIComponent(GBoptions.hubUrl) : '',
	    trackString = GBoptions && GBoptions.fullTracks ? '&hideTracks=1' + GBoptions.fullTracks.map(function (track) {
		return '&' + track + '=full';
	}).join('') : '',
	    GBurl = 'http://genome.ucsc.edu/cgi-bin/hgTracks?db=' + assemblyString + hubString + trackString + '&highlight=' + assemblyString + '.' + highlightString + '&position=' + positionString;
	return GBurl;
};
var gbMultiColorURL = function gbMultiColorURL(assembly, pos, posColorList, GBoptions) {
	// assembly : e.g. hg18
	// pos: e.g. chr3:178,936,070-178,936,070
	// posColorList: [[chr3:178,936,070-178,936,070, AA0000], ...]
	// GBoptions.hubUrl: To build a URL that will load the hub directly
	// GBoptions.fullTracks: full display mode track list
	var assemblyString = encodeURIComponent(assembly),
	    positionString = encodeURIComponent(pos),
	    highlightString = posColorList.map(function (p) {
		return assemblyString + '.' + p[0] + p[1];
	}).join('|'),
	    hubString = GBoptions && GBoptions.hubUrl ? "&hubUrl=" + encodeURIComponent(GBoptions.hubUrl) : '',
	    trackString = GBoptions && GBoptions.assembly === assembly && GBoptions.fullTracks ? '&hideTracks=1' + GBoptions.fullTracks.map(function (track) {
		return '&' + track + '=full';
	}).join('') : '',
	    GBurl = 'http://genome.ucsc.edu/cgi-bin/hgTracks?db=' + assemblyString + hubString + trackString + '&highlight=' + encodeURIComponent(highlightString) + '&position=' + positionString;
	return GBurl;
};
var defaultSNVSVGBsetting = function defaultSNVSVGBsetting(assembly) {
	if (assembly === 'hg19' || assembly === 'GRCh37') {
		return {
			assembly: assembly,
			hubUrl: 'http://hgwdev.soe.ucsc.edu/~max/immuno/track/hub/hub.txt', // Max Haussler's cancer genomics hub
			fullTracks: ['knownGene', 'refGene', 'wgEncodeGencodeV24lift37', // gene annotation
			'ucscGenePfam', 'spUniprot', // protein annotation
			//'pubs', // publication
			'wgEncodeRegDnaseClustered', 'wgEncodeAwgSegmentation', // encode regulation
			'hub_29889_dienstmann', 'hub_29889_civic', 'hub_29889_oncokb' // cancer genomics knowledgebase
			]
		};
	} else if (assembly === 'hg38' || assembly === 'GRCh38') {
		return {
			assembly: assembly,
			fullTracks: ['knownGene', 'refGene', // gene annotation
			'ucscGenePfam', 'spUniprot', // protein annotation
			//'pubs', // publication
			'wgEncodeRegDnaseClustered']
		};
	} else {
		return {};
	}
};

function sampleTooltip(sampleFormat, dataList, assembly, fields) {
	var perRowTip = function perRowTip(data) {
		var dnaVaf = data.dnaVaf == null ? null : ['labelValue', 'DNA variant allele freq', formatAf(data.dnaVaf)],
		    rnaVaf = data.rnaVaf == null ? null : ['labelValue', 'RNA variant allele freq', formatAf(data.rnaVaf)],
		    ref = data.reference && ['label', ' ' + data.reference + ' to '],


		//alt
		altDirection = data.alt && mv.joinedVariantDirection(data.alt),
		    altStart = altDirection && parseInt(mv.posFromAlt(data.alt)),
		    altPos = altDirection && 'chr' + mv.chromFromAlt(data.alt) + ':' + altStart + '-' + altStart,
		    altRegion = altDirection && altDirection === 'left' ? 'chr' + mv.chromFromAlt(data.alt) + ':' + (altStart - 100) + '-' + (altStart - 1) : 'chr' + mv.chromFromAlt(data.alt) + ':' + (altStart + 1) + '-' + (altStart + 100),
		    altDisplayRegion = altDirection && 'chr' + mv.chromFromAlt(data.alt) + ':' + (altStart - 150) + '-' + (altStart + 150),


		//variant
		variantDirection = data.alt && mv.structuralVariantClass(data.alt),
		    start = data.start,
		    dataRegion = variantDirection && variantDirection === 'left' ? data.chr + ':' + (start - 100) + '-' + (start - 1) : data.chr + ':' + (start + 1) + '-' + (start + 100),
		    dataDisplayRegion = altDirection && data.chr + ':' + (start - 150) + '-' + (start + 150),


		//alt link
		alt = data.alt && (mv.structuralVariantClass(data.alt) ? ['url', '' + data.alt, gbMultiColorURL(assembly, altDisplayRegion, [[altPos, '#AA0000'], [altRegion, '#aec7e8']], defaultSNVSVGBsetting(assembly))] : ['label', '' + data.alt]),


		//variant link
		posDisplay = data && data.start === data.end ? posStartString(data) : posDoubleString(data),
		    posURL = ['url', assembly + ' ' + posDisplay, altDirection ? gbMultiColorURL(assembly, dataDisplayRegion, [[posDoubleString(data), '#AA0000'], [dataRegion, '#aec7e8']], defaultSNVSVGBsetting(assembly)) : gbURL(assembly, posRegionString(data), posDoubleString(data), defaultSNVSVGBsetting(assembly))],
		    effect = ['value', fmtIf(data.effect, function (x) {
			return x + ', ';
		}, 'unannotated') + //eslint-disable-line comma-spacing
		fmtIf(data.gene, function (x) {
			return '' + x;
		}) + fmtIf(data.aminoAcid, function (x) {
			return ' (' + x + ')';
		}) + fmtIf(data.altGene, function (x) {
			return '--' + x + ' ';
		})];
		return dropNulls([[effect], [posURL, ref, alt], [dnaVaf], [rnaVaf]]);
	};

	//sort dataList by fields[0], put variants annoated with fields[0] in dataset in front
	dataList = _.sortBy(dataList, function (obj) {
		return obj.gene === fields[0];
	}).reverse();

	var rows = _.reduce(_.map(dataList /*.slice(0, 3)*/, perRowTip), function (a, b) {
		return a.concat(b);
	}, []);

	//	if (dataList.length > 3) {
	//		var allRows = rows.concat(_.reduce(_.map(dataList.slice(3), perRowTip), function(a, b) { return b.concat(a); }, []));
	//		rows.push([["popOver", dataList.length - 3 + " more ...", allRows]]);
	//	}
	return {
		rows: rows,
		sampleID: sampleFormat(dataList[0].sample)
	};
}

function posTooltip(layout, samples, sampleFormat, pixPerRow, index, assembly, x, y) {
	var yIndex = Math.round((y - pixPerRow / 2) / pixPerRow + index),
	    pos = Math.floor(chromPositionFromScreen(layout, x)),
	    coordinate = {
		chr: layout.chromName,
		start: pos,
		end: pos
	};
	return {
		sampleID: sampleFormat(samples[yIndex]),
		rows: [[['url', assembly + ' ' + posStartString(coordinate), gbURL(assembly, posRegionString(coordinate), posDoubleString(coordinate))]]] };
}

function tooltip(id, fieldType, fields, layout, nodes, samples, sampleFormat, zoom, assembly, ev) {
	var _util$eventOffset = util.eventOffset(ev),
	    x = _util$eventOffset.x,
	    y = _util$eventOffset.y,
	    height = zoom.height,
	    count = zoom.count,
	    index = zoom.index,
	    pixPerRow = height / count,
	    lo = _.updateIn(layout, ['chromName'], function (c) {
		return c || _.getIn(nodes, [0, 'data', 'chr']);
	}),
	    closestNodes = closestNode[fieldType](nodes, zoom, x, y);

	return (0, _extends3.default)({
		x: x,
		id: id
	}, closestNodes.length > 0 ? sampleTooltip(sampleFormat, _.pluck(closestNodes, 'data'), assembly, fields) : posTooltip(lo, samples, sampleFormat, pixPerRow, index, assembly, x, y));
}

var MutationColumn = hotOrNot(function (_PureComponent) {
	(0, _inherits3.default)(_class2, _PureComponent);

	function _class2() {
		var _ref3;

		var _temp, _this, _ret;

		(0, _classCallCheck3.default)(this, _class2);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref3 = _class2.__proto__ || (0, _getPrototypeOf2.default)(_class2)).call.apply(_ref3, [this].concat(args))), _this), _this.tooltip = function (ev) {
			var _this$props = _this.props,
			    _this$props$column = _this$props.column,
			    fieldType = _this$props$column.fieldType,
			    fields = _this$props$column.fields,
			    layout = _this$props$column.layout,
			    nodes = _this$props$column.nodes,
			    assembly = _this$props$column.assembly,
			    samples = _this$props.samples,
			    sampleFormat = _this$props.sampleFormat,
			    zoom = _this$props.zoom,
			    id = _this$props.id;

			return tooltip(id, fieldType, fields, layout, nodes, samples, sampleFormat, zoom, assembly, ev);
		}, _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
	}

	(0, _createClass3.default)(_class2, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			var _this2 = this;

			var events = rxEvents(this, 'mouseout', 'mousemove', 'mouseover');

			// Compute tooltip events from mouse events.
			this.ttevents = events.mouseover.filter(function (ev) {
				return util.hasClass(ev.currentTarget, 'Tooltip-target');
			}).flatMap(function () {
				return events.mousemove.takeUntil(events.mouseout).map(function (ev) {
					return {
						data: _this2.tooltip(ev),
						open: true
					};
				}) // look up current data
				.concat(Rx.Observable.of({ open: false }));
			}).subscribe(this.props.tooltip);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.ttevents.unsubscribe();
		}
	}, {
		key: 'render',
		value: function render() {
			var _props = this.props,
			    column = _props.column,
			    samples = _props.samples,
			    zoom = _props.zoom,
			    index = _props.index,
			    draw = _props.draw;


			return React.createElement(CanvasDrawing, {
				ref: 'plot',
				draw: draw,
				wrapperProps: {
					className: 'Tooltip-target',
					onMouseMove: this.on.mousemove,
					onMouseOut: this.on.mouseout,
					onMouseOver: this.on.mouseover,
					onClick: this.props.onClick
				},
				nodes: column.nodes,
				strand: column.strand,
				width: column.width,
				index: index,
				samples: samples,
				xzoom: column.zoom,
				zoom: zoom });
		}
	}]);
	return _class2;
}(_PureComponent3.default));

widgets.column.add('mutation', function (props) {
	return React.createElement(MutationColumn, (0, _extends3.default)({ draw: drawMutations }, props));
});
widgets.column.add('SV', function (props) {
	return React.createElement(MutationColumn, (0, _extends3.default)({ draw: drawSV }, props));
});

widgets.legend.add('mutation', drawMutationLegend);
widgets.legend.add('SV', drawSVLegend);