'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _PureComponent2 = require('./PureComponent');

var _PureComponent3 = _interopRequireDefault(_PureComponent2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('./underscore_ext');
var Rx = require('./rx');

var React = require('react');
var BandLegend = require('./views/BandLegend');

var _require = require('./react-utils'),
    rxEvents = _require.rxEvents;

var widgets = require('./columnWidgets');
var util = require('./util');
var CanvasDrawing = require('./CanvasDrawing');

var _require2 = require('./drawSegmented'),
    drawSegmented = _require2.drawSegmented,
    toYPx = _require2.toYPx;

var _require3 = require('./exonLayout'),
    chromPositionFromScreen = _require3.chromPositionFromScreen;

var colorScales = require('./colorScales');

// Since we don't set module.exports, but instead register ourselves
// with columWidgets, react-hot-loader can't handle the updates automatically.
// Accept hot loading here.
if (module.hot) {
	module.hot.accept();
}

// Since there are multiple components in the file we have to use makeHot
// explicitly.
function hotOrNot(component) {
	return module.makeHot ? module.makeHot(component) : component;
}

// might want to use <wbr> here, instead, so cut & paste work better, but that
// will require a recursive split/flatmap to inject the <wbr> elements.
var addWordBreaks = function addWordBreaks(str) {
	return str.replace(/([_/])/g, '\u200B$1\u200B');
};

function renderFloatLegend(props) {
	var units = props.units,
	    color = props.color;


	if (color[0] === 'no-data') {
		return null;
	}
	var unitText = (units || [''])[0] || '',
	    footnotes = [React.createElement(
		'span',
		{ title: unitText },
		addWordBreaks(unitText)
	)];

	var _color$slice = color.slice(4),
	    _color$slice2 = (0, _slicedToArray3.default)(_color$slice, 3),
	    origin = _color$slice2[0],
	    max = _color$slice2[2],
	    powerScale = colorScales.colorScale(color).lookup,
	    scale = function scale(v) {
		return v < origin ? powerScale(0, origin - v) : powerScale(1, v - origin);
	},
	    min = origin - (max - origin);

	return React.createElement(BandLegend, {
		range: { min: min, max: max },
		colorScale: scale,
		footnotes: footnotes,
		width: 50,
		height: 20 });
}

function drawLegend(props) {
	var column = props.column,
	    units = column.units,
	    color = column.color,
	    vizSettings = column.vizSettings,
	    defaultNormalization = column.defaultNormalization,
	    legendProps = {
		units: units,
		color: color,
		vizSettings: vizSettings,
		defaultNormalization: defaultNormalization
	};

	return renderFloatLegend(legendProps);
}

function closestNode(nodes, zoom, x, y) {
	var index = zoom.index,
	    count = zoom.count,
	    end = index + count,
	    underRow = function underRow(v) {
		var _toYPx = toYPx(zoom, v),
		    svHeight = _toYPx.svHeight,
		    suby = _toYPx.y;

		return Math.abs(y - suby) < svHeight / 2;
	},
	    underMouse = _.filter(nodes, function (n) {
		return n.y >= index && n.y < end && x >= n.xStart && x <= n.xEnd && underRow(n);
	});

	return underMouse[0];
}

//var fmtIf = (x, fmt, d = '' ) => x ? fmt(x) : d;
var dropNulls = function dropNulls(rows) {
	return rows.map(function (row) {
		return row.filter(function (col) {
			return col != null;
		});
	}) // drop empty cols
	.filter(function (row) {
		return row.length > 0;
	});
}; // drop empty rows
//gb position string for 1.5 x segment, centered at segment
var posRegionString = function posRegionString(chrom, p) {
	return chrom + ':' + util.addCommas(p.start - Math.round((p.end - p.start) / 4)) + '-' + util.addCommas(p.end + Math.round((p.end - p.start) / 4));
};
//gb position string like chr3:178,936,070-178,936,070
var posDoubleString = function posDoubleString(chrom, p) {
	return chrom + ':' + util.addCommas(p.start) + '-' + util.addCommas(p.end);
};
//gb position string like chr3:178,936,070
var posStartString = function posStartString(chrom, p) {
	return chrom + ':' + util.addCommas(p.start);
};
// gb url link with highlight
var gbURL = function gbURL(assembly, pos, highlightPos) {
	// assembly : e.g. hg18
	// pos: e.g. chr3:178,936,070-178,936,070
	// highlight: e.g. chr3:178,936,070-178,936,070
	var assemblyString = encodeURIComponent(assembly),
	    positionString = encodeURIComponent(pos),
	    highlightString = encodeURIComponent(highlightPos);
	return 'http://genome.ucsc.edu/cgi-bin/hgTracks?db=' + assemblyString + '&highlight=' + assemblyString + '.' + highlightString + '&position=' + positionString;
};

function sampleTooltip(chrom, sampleFormat, data, gene, assembly) {
	var posDisplay = data && data.start === data.end ? posStartString(chrom, data) : posDoubleString(chrom, data),
	    posURL = ['url', assembly + ' ' + posDisplay, gbURL(assembly, posRegionString(chrom, data), posDoubleString(chrom, data))],
	    value = ['labelValue', 'value', '' + data.value];

	return {
		rows: dropNulls([[value], [posURL]]),
		sampleID: sampleFormat(data.sample)
	};
}

function posTooltip(layout, samples, sampleFormat, pixPerRow, index, assembly, x, y) {
	var chrom = layout.chromName,
	    yIndex = Math.round((y - pixPerRow / 2) / pixPerRow + index),
	    pos = Math.floor(chromPositionFromScreen(layout, x)),
	    coordinate = {
		chr: chrom,
		start: pos,
		end: pos
	};
	return {
		sampleID: sampleFormat(samples[yIndex]),
		rows: [[['url', assembly + ' ' + posStartString(chrom, coordinate), gbURL(assembly, posRegionString(chrom, coordinate), posDoubleString(chrom, coordinate))]]] };
}

function tooltip(id, fieldType, layout, nodes, samples, sampleFormat, zoom, gene, assembly, ev) {
	var _util$eventOffset = util.eventOffset(ev),
	    x = _util$eventOffset.x,
	    y = _util$eventOffset.y,
	    height = zoom.height,
	    count = zoom.count,
	    index = zoom.index,
	    pixPerRow = height / count,
	    lo = _.updateIn(layout, ['chromName'], function (c) {
		return c || _.getIn(nodes, [0, 'data', 'chr']);
	}),
	    node = closestNode(nodes, zoom, x, y);

	return (0, _extends3.default)({
		x: x,
		id: id }, node ? sampleTooltip(lo.chromName, sampleFormat, node.data, gene, assembly) : posTooltip(lo, samples, sampleFormat, pixPerRow, index, assembly, x, y));
}

var SegmentedColumn = hotOrNot(function (_PureComponent) {
	(0, _inherits3.default)(_class2, _PureComponent);

	function _class2() {
		var _ref;

		var _temp, _this, _ret;

		(0, _classCallCheck3.default)(this, _class2);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref = _class2.__proto__ || (0, _getPrototypeOf2.default)(_class2)).call.apply(_ref, [this].concat(args))), _this), _this.tooltip = function (ev) {
			var _this$props = _this.props,
			    _this$props$column = _this$props.column,
			    fieldType = _this$props$column.fieldType,
			    layout = _this$props$column.layout,
			    nodes = _this$props$column.nodes,
			    fields = _this$props$column.fields,
			    assembly = _this$props$column.assembly,
			    samples = _this$props.samples,
			    sampleFormat = _this$props.sampleFormat,
			    zoom = _this$props.zoom,
			    id = _this$props.id;

			return tooltip(id, fieldType, layout, nodes, samples, sampleFormat, zoom, fields[0], assembly, ev);
		}, _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
	}

	(0, _createClass3.default)(_class2, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			var _this2 = this;

			var events = rxEvents(this, 'mouseout', 'mousemove', 'mouseover');

			// Compute tooltip events from mouse events.
			this.ttevents = events.mouseover.filter(function (ev) {
				return util.hasClass(ev.currentTarget, 'Tooltip-target');
			}).flatMap(function () {
				return events.mousemove.takeUntil(events.mouseout).map(function (ev) {
					return {
						data: _this2.tooltip(ev),
						open: true
					};
				}) // look up current data
				.concat(Rx.Observable.of({ open: false }));
			}).subscribe(this.props.tooltip);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.ttevents.unsubscribe();
		}
	}, {
		key: 'render',
		value: function render() {
			var _props = this.props,
			    column = _props.column,
			    samples = _props.samples,
			    zoom = _props.zoom,
			    index = _props.index;


			return React.createElement(CanvasDrawing, {
				ref: 'plot',
				draw: drawSegmented,
				wrapperProps: {
					className: 'Tooltip-target',
					onMouseMove: this.on.mousemove,
					onMouseOut: this.on.mouseout,
					onMouseOver: this.on.mouseover,
					onClick: this.props.onClick
				},
				color: column.color,
				nodes: column.nodes,
				strand: column.strand,
				width: column.width,
				index: index,
				samples: samples,
				xzoom: column.zoom,
				zoom: zoom });
		}
	}]);
	return _class2;
}(_PureComponent3.default));

widgets.column.add('segmented', function (props) {
	return React.createElement(SegmentedColumn, props);
});

widgets.legend.add('segmented', drawLegend);