'use strict';

var _require = require('../underscore_ext'),
    sortBy = _require.sortBy;

var Minhash = require('minhash/src/minhash');
var rawProbemapHashes = require('json-loader!./probemapMinHashes');
var rawCohortHashes = require('json-loader!./cohortMinHashes');

var inflate = function inflate(list) {
	return list.map(function (_ref) {
		var name = _ref.name,
		    hash = _ref.hash;

		var mh = new Minhash();
		mh.hashvalues = hash;
		return { name: name, hash: mh };
	});
};

var probemapHashes = inflate(rawProbemapHashes);
var cohortHashes = inflate(rawCohortHashes);

function minHashFromList(list) {
	var mh = new Minhash();
	list.forEach(function (p) {
		return mh.update(p);
	});
	return mh;
}

function recommend(list, hashes, thresh) {
	var mh = minHashFromList(list),
	    scores = hashes.map(function (_ref2) {
		var name = _ref2.name,
		    hash = _ref2.hash;
		return { name: name, score: hash.jaccard(mh) };
	});
	return sortBy(scores, 'score').reverse().filter(function (x) {
		return x.score > thresh;
	});
}

module.exports = {
	probemap: function probemap(probes) {
		return recommend(probes, probemapHashes, 0.001);
	},
	cohort: function cohort(samples) {
		return recommend(samples, cohortHashes, 0.001);
	}
};