'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var React = require('react');
var _ = require('../underscore_ext');
var styles = require('./DensityPlot.module.css');
var sc = require('science');

var plotHeight = 35;
var plotWidth = 200;
var zoomFactor = 3;
var topColor = "#BAA0CF";
var bottomColor = "#79C6C5";

function calculateHeights(data, max, min, plotWidth) {
	var pxWidth = (max - min) / plotWidth;
	var aHeights = data.studyA.map(function (exp) {
		return exp.expA.length ? sc.stats.kde().sample(exp.expA)(_.range(min, max + pxWidth, pxWidth)).map(function (kdep) {
			return kdep[1];
		}) : [];
	});
	var bHeights = data.studyB.map(function (exp) {
		return exp.expB.length ? sc.stats.kde().sample(exp.expB)(_.range(min, max + pxWidth, pxWidth)).map(function (kdep) {
			return kdep[1];
		}) : [];
	});

	return {
		aHeights: aHeights,
		bHeights: bHeights
	};
}

function densitySvg(yHeights, height, totalWidth, vscale, A) {
	var binWidth = plotWidth / yHeights.length,
	    polylinePoints = ['20,' + height].concat((0, _toConsumableArray3.default)(yHeights.map(function (y, i) {
		return i * binWidth + 20 + ',' + (1 - y / vscale) * height;
	})), [plotWidth + 20 + ',' + height]).join(' ');
	return React.createElement(
		'svg',
		{ width: totalWidth, height: height },
		React.createElement('polyline', { points: polylinePoints, fill: A ? topColor : bottomColor })
	);
}

var drawDensityPlot = function drawDensityPlot(min, max, totalWidth, getNameZoom) {
	return function (aHeight, bHeight, nameAndZoom) {
		var height = nameAndZoom.zoom ? plotHeight * zoomFactor : plotHeight,
		    rowClass = nameAndZoom.zoom ? "densityPlot--row--zoom" : "densityPlot--row",
		    vscale = Math.max.apply(Math, (0, _toConsumableArray3.default)(aHeight.filter(function (x) {
			return !isNaN(x);
		})).concat((0, _toConsumableArray3.default)(bHeight.filter(function (x) {
			return !isNaN(x);
		}))));

		return React.createElement(
			'div',
			{ key: nameAndZoom.name, className: styles[rowClass], style: { width: totalWidth + 'px' }, onClick: function onClick() {
					return getNameZoom(nameAndZoom.name);
				} },
			React.createElement('div', { className: styles["densityPlot--scale--Axis"], style: { height: 2 * height - 2, left: "18px" } }),
			React.createElement('div', { className: styles["densityPlot--row--xAxis"], style: { width: plotWidth * 100 / totalWidth + '%', left: "20px" } }),
			isFinite(vscale) ? React.createElement(
				'div',
				{ className: styles["densityPlot--vscal--label"], style: { left: "15px" } },
				vscale.toFixed(2)
			) : null,
			isFinite(vscale) ? React.createElement('div', { className: styles["densityPlot--vscal--tick"],
				style: { left: "15px", top: '1px' } }) : null,
			isFinite(vscale) ? React.createElement('div', { className: styles["densityPlot--vscal--tick"],
				style: { left: "15px", top: 2 * height - 2 + 'px' } }) : null,
			React.createElement(
				'div',
				{ className: styles["densityPlot--row--studyA"], style: { width: totalWidth } },
				densitySvg(aHeight, height, totalWidth, vscale, true)
			),
			React.createElement(
				'div',
				{ className: styles["densityPlot--row--studyB"], style: { width: totalWidth } },
				densitySvg(bHeight, height, totalWidth, vscale, false)
			)
		);
	};
};

var DensityPlot = function (_React$PureComponent) {
	(0, _inherits3.default)(DensityPlot, _React$PureComponent);

	function DensityPlot() {
		(0, _classCallCheck3.default)(this, DensityPlot);
		return (0, _possibleConstructorReturn3.default)(this, (DensityPlot.__proto__ || (0, _getPrototypeOf2.default)(DensityPlot)).apply(this, arguments));
	}

	(0, _createClass3.default)(DensityPlot, [{
		key: 'render',
		value: function render() {
			var _props = this.props,
			    getNameZoom = _props.getNameZoom,
			    min = _props.min,
			    max = _props.max,
			    totalWidth = plotWidth + 20,
			    data = this.props.data ? this.props.data : null,
			    _calculateHeights = calculateHeights(data, max, min, plotWidth),
			    aHeights = _calculateHeights.aHeights,
			    bHeights = _calculateHeights.bHeights,
			    drawPlot = drawDensityPlot(min, max, totalWidth, getNameZoom),
			    rows = _.mmap(aHeights, bHeights, data.nameAndZoom, drawPlot);

			return React.createElement(
				'div',
				{ className: styles.densityPlot },
				rows
			);
		}
	}]);
	return DensityPlot;
}(React.PureComponent);

module.exports = { DensityPlot: DensityPlot, bottomColor: bottomColor, topColor: topColor, plotWidth: plotWidth };