'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var React = require('react');
var _ = require('../underscore_ext');

var styles = require('./Exons.module.css');

var width = 700;

function box(type, startsAt, width, multiplyingFactor, strand) {
	var pad = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0;
	var zoom = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : false;
	var label = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : "";

	var exon = {
		type: type,
		label: label,
		width: width * multiplyingFactor + "px",
		origin: strand === '-' ? "right" : "left",
		position: startsAt * multiplyingFactor + pad + "px",
		zoom: zoom
	};
	return exon;
}

function renderExon(exon, key) {
	var style = (0, _defineProperty3.default)({
		width: exon.width
	}, exon.origin, exon.position);
	if (exon.type === 'small') {
		var boxClass = exon.zoom ? "exons--row--item-small--zoom" : "exons--row--item-small";
		return React.createElement(
			'div',
			{ key: key, className: styles[boxClass],
				style: style },
			React.createElement(
				'span',
				null,
				exon.label
			)
		);
	} else if (exon.type === 'big') {
		var _boxClass = exon.zoom ? "exons--row--item-big--zoom" : "exons--row--item-big";
		return React.createElement(
			'div',
			{ key: key, className: styles[_boxClass],
				style: style },
			React.createElement(
				'span',
				null,
				exon.label
			)
		);
	}
}

function exonShape(data, exonStarts, exonEnds, cdsStart, cdsEnd, multiplyingFactor, strand, origin) {

	var exonWidth = exonEnds - exonStarts;
	var startsAt = exonStarts - origin;
	if (cdsStart === cdsEnd) {
		return [box('small', startsAt, exonWidth, multiplyingFactor, strand)];
	} else if (cdsStart > exonEnds) {
		return [box('small', startsAt, exonWidth, multiplyingFactor, strand)];
	} else if (exonStarts < cdsStart && cdsStart < exonEnds) {
		var exonWidth1 = cdsStart - exonStarts;
		var exonWidth2 = exonEnds - cdsStart;
		return [box('small', startsAt, exonWidth1, multiplyingFactor, strand), box('big', cdsStart - origin, exonWidth2, multiplyingFactor, strand)];
	} else if (exonStarts < cdsEnd && cdsEnd < exonEnds) {
		var _exonWidth = cdsEnd - exonStarts;
		var _exonWidth2 = exonEnds - cdsEnd;
		return [box('big', startsAt, _exonWidth, multiplyingFactor, strand), box('small', cdsEnd - origin, _exonWidth2, multiplyingFactor, strand)];
	} else if (cdsEnd < exonStarts) {
		return [box('small', startsAt, exonWidth, multiplyingFactor, strand)];
	} else {
		return [box('big', startsAt, exonWidth, multiplyingFactor, strand)];
	}
}

var Exons = function (_React$PureComponent) {
	(0, _inherits3.default)(Exons, _React$PureComponent);

	function Exons() {
		var _ref;

		var _temp, _this, _ret;

		(0, _classCallCheck3.default)(this, Exons);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref = Exons.__proto__ || (0, _getPrototypeOf2.default)(Exons)).call.apply(_ref, [this].concat(args))), _this), _this.row = function (data, multiplyingFactor, origin) {

			return data.map(function (d, index) {

				var style = (0, _defineProperty3.default)({
					width: (d.txEnd - d.txStart) * multiplyingFactor + "px"
				}, d.strand === '-' ? 'right' : 'left', (d.txStart - origin) * multiplyingFactor + "px");

				return React.createElement(
					'div',
					{ className: styles["exons--row"], id: index },
					React.createElement('div', { className: styles["exons--row--axis"],
						style: style }),
					_.flatten(_.mmap(d.exonStarts, d.exonEnds, function (exonStarts, exonEnds) {
						return _.map(exonShape(data, exonStarts, exonEnds, d.cdsStart, d.cdsEnd, multiplyingFactor, d.strand, origin), renderExon);
					}))
				);
			});
		}, _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
	}

	(0, _createClass3.default)(Exons, [{
		key: 'render',
		value: function render() {
			var data = this.props.data ? this.props.data : null;
			var origin = Math.min.apply(Math, _.pluck(data, 'txStart'));
			//multiplying factor used to calculate position and width
			var multiplyingFactor = width / (Math.max.apply(Math, _.pluck(data, 'txEnd')) - origin);

			var rows = this.row(data, multiplyingFactor, origin);

			return React.createElement(
				'div',
				{ className: styles.exons },
				rows
			);
		}
	}]);
	return Exons;
}(React.PureComponent);

module.exports = {
	Exons: Exons,
	box: box,
	renderExon: renderExon
};