'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var React = require('react');
var _ = require('../underscore_ext');

var _require = require('../findIntrons'),
    allExons = _require.allExons,
    exonGroups = _require.exonGroups,
    intronRegions = _require.intronRegions;

var _require2 = require('./Exons'),
    box = _require2.box,
    renderExon = _require2.renderExon;

var styles = require('./Exons.module.css');

var width = 700;
var padding = 5;
var suffixList = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";

var suffix = function suffix(len, i, neg) {
	return len === 1 ? "" : neg ? suffixList.charAt(len - i - 1) : suffixList.charAt(i);
};

var exonLabel = function exonLabel(len, i, neg) {
	return (neg ? len - i : i + 1).toString();
};

var isBefore = function isBefore(start) {
	return function (_ref) {
		var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
		    end = _ref2[1];

		return end <= start;
	};
};
var calcLength = function calcLength(_ref3) {
	var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
	    s = _ref4[0],
	    e = _ref4[1];

	return e - s;
};
var subtract = function subtract(x, y) {
	return x - y;
};
var intronLengthBefore = _.curry(function (introns, x) {
	return _.sum(introns.filter(isBefore(x)).map(calcLength));
});

// intronRegions is a 2D array like so: [[10,30], [70,100]]
function newCoordinates(data, intronRegions, labelsAndPad) {
	var offsets = data.exonStarts.map(intronLengthBefore(intronRegions)),
	    exonStartsCopy = _.mmap(data.exonStarts, offsets, subtract),
	    exonEndsCopy = _.mmap(data.exonEnds, offsets, subtract),
	    cdsStartCopy = data.cdsStart - intronLengthBefore(intronRegions, data.cdsStart),
	    cdsEndCopy = data.cdsEnd - intronLengthBefore(intronRegions, data.cdsEnd),
	    exonLabelsAndPad = _.mmap(data.exonStarts, data.exonEnds, function (start, end) {
		return labelsAndPad[start][end];
	});

	return _.assoc(data, 'exonStarts', exonStartsCopy, 'exonEnds', exonEndsCopy, 'txStart', exonStartsCopy[0], 'txEnd', exonEndsCopy[data.exonCount - 1], 'cdsStart', cdsStartCopy, 'cdsEnd', cdsEndCopy, 'labelsAndPad', exonLabelsAndPad);
}

var cmpExons = function cmpExons(negativeStrand) {
	if (negativeStrand) {
		return function (e1, e2) {
			return e1.end > e2.end ? 1 : e1.end < e2.end ? -1 : e1.start > e2.start ? 1 : e1.start < e2.start ? -1 : 0;
		};
	} else {
		return function (e1, e2) {
			return e1.start > e2.start ? 1 : e1.start < e2.start ? -1 : e1.end > e2.end ? 1 : e1.end < e2.end ? -1 : 0;
		};
	}
};

function exonLabelsAndPad(exonGroups, neg) {
	var uniqExonGroups = exonGroups.map(function (exonGroup) {
		return _.uniq(exonGroup.exons.map(function (exon) {
			return (0, _stringify2.default)(exon);
		})).map(function (es) {
			return JSON.parse(es);
		}).sort(cmpExons(neg));
	}),
	    labels = _.apply(_.merge)(_.flatmap(uniqExonGroups, function (exonGroup, i) {
		return exonGroup.map(function (exon, j) {
			return [exon.start, exon.end, i * padding, exonLabel(exonGroups.length, i, neg) + suffix(exonGroup.length, j, neg)];
		}).reduce(function (acc, _ref5) {
			var _ref6 = (0, _slicedToArray3.default)(_ref5, 4),
			    start = _ref6[0],
			    end = _ref6[1],
			    pad = _ref6[2],
			    label = _ref6[3];

			return _.assocIn(acc, [start, end], { pad: pad, label: label });
		}, {});
	}));
	return labels;
}

var exonShape = function exonShape(cdsStart, cdsEnd, multiplyingFactor, strand, origin, zoom) {
	return function (exonStarts, exonEnds, _ref7) {
		var label = _ref7.label,
		    pad = _ref7.pad;

		var exonWidth = exonEnds - exonStarts,
		    startsAt = exonStarts - origin;
		if (cdsStart === cdsEnd) {
			return [box('small', startsAt, exonWidth, multiplyingFactor, strand, pad, zoom, label)];
		} else if (cdsStart > exonEnds) {
			return [box('small', startsAt, exonWidth, multiplyingFactor, strand, pad, zoom, label)];
		} else if (exonStarts < cdsStart && cdsStart < exonEnds) {
			var exonWidth1 = cdsStart - exonStarts;
			var exonWidth2 = exonEnds - cdsStart;
			// here if-else is only for identifying a suitable box for labeling.
			// labeling is done on the longest of the two boxes.
			if (exonWidth1 < exonWidth2) {
				return [box('small', startsAt, exonWidth1, multiplyingFactor, strand, pad, zoom), box('big', cdsStart - origin, exonWidth2, multiplyingFactor, strand, pad, zoom, label)];
			} else {
				return [box('small', startsAt, exonWidth1, multiplyingFactor, strand, pad, zoom, label), box('big', cdsStart - origin, exonWidth2, multiplyingFactor, strand, pad, zoom)];
			}
		} else if (exonStarts < cdsEnd && cdsEnd < exonEnds) {
			var _exonWidth = cdsEnd - exonStarts;
			var _exonWidth2 = exonEnds - cdsEnd;
			// here if-else is only for identifying a suitable box for labeling.
			// labeling is done on the longest of the two boxes.
			if (_exonWidth > _exonWidth2) {
				return [box('big', startsAt, _exonWidth, multiplyingFactor, strand, pad, zoom, label), box('small', cdsEnd - origin, _exonWidth2, multiplyingFactor, strand, pad, zoom)];
			} else {
				return [box('big', startsAt, _exonWidth, multiplyingFactor, strand, pad, zoom), box('small', cdsEnd - origin, _exonWidth2, multiplyingFactor, strand, pad, zoom, label)];
			}
		} else if (cdsEnd < exonStarts) {
			return [box('small', startsAt, exonWidth, multiplyingFactor, strand, pad, zoom, label)];
		} else {
			return [box('big', startsAt, exonWidth, multiplyingFactor, strand, pad, zoom, label)];
		}
	};
};

function drawRows(data, multiplyingFactor, origin, getNameZoom) {
	return data.map(function (d, index) {
		var firstPad = _.first(d.labelsAndPad).pad,
		    lastPad = _.last(d.labelsAndPad).pad,
		    extraAxisWidth = lastPad - firstPad,
		    style = (0, _defineProperty3.default)({
			width: (d.txEnd - d.txStart) * multiplyingFactor + extraAxisWidth + "px"
		}, d.strand === '-' ? 'right' : 'left', (d.txStart - origin) * multiplyingFactor + firstPad + "px"),
		    rowClass = d.zoom ? "exons--row--zoom" : "exons--row",
		    transcriptExonShape = exonShape(d.cdsStart, d.cdsEnd, multiplyingFactor, d.strand, origin, d.zoom),
		    allBoxes = _.flatten(_.mmap(d.exonStarts, d.exonEnds, d.labelsAndPad, transcriptExonShape));
		return React.createElement(
			'div',
			{ key: d.name, className: styles[rowClass], id: index, onClick: function onClick() {
					return getNameZoom(d.name);
				} },
			React.createElement('div', { className: styles["exons--row--axis"], style: style }),
			allBoxes.map(renderExon)
		);
	});
}

var ExonsOnly = function (_React$PureComponent) {
	(0, _inherits3.default)(ExonsOnly, _React$PureComponent);

	function ExonsOnly() {
		(0, _classCallCheck3.default)(this, ExonsOnly);
		return (0, _possibleConstructorReturn3.default)(this, (ExonsOnly.__proto__ || (0, _getPrototypeOf2.default)(ExonsOnly)).apply(this, arguments));
	}

	(0, _createClass3.default)(ExonsOnly, [{
		key: 'render',
		value: function render() {
			var data = this.props.data ? this.props.data : [],
			    exonGroupsList = exonGroups(allExons(data)),
			    intronRegionsList = intronRegions(exonGroupsList),
			    labelsAndPad = exonLabelsAndPad(exonGroupsList, _.getIn(data, [0, 'strand']) === '-'),
			    exonGroupsWidth = width - padding * (exonGroupsList.length - 1),
			    newData = data.map(function (d) {
				return newCoordinates(d, intronRegionsList, labelsAndPad);
			}),
			    origin = _.min(newData, 'txStart').txStart,
			    maxEnd = _.max(newData, 'txEnd').txEnd,
			    multiplyingFactor = exonGroupsWidth / (maxEnd - origin),
			    rows = drawRows(newData, multiplyingFactor, origin, this.props.getNameZoom);
			return React.createElement(
				'div',
				{ className: styles.exons },
				rows
			);
		}
	}]);
	return ExonsOnly;
}(React.PureComponent);

module.exports = ExonsOnly;