'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _iterator = require('babel-runtime/core-js/symbol/iterator');

var _iterator2 = _interopRequireDefault(_iterator);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = /*#__PURE__*/_regenerator2.default.mark(repeat);

var _ = require('underscore');
var ehmutable = require('ehmutable');
var defer = require('./defer');

var slice = Array.prototype.slice;

_.mixin({ defer: defer }); // override underscore defer
_.mixin(ehmutable.init(_)); // add immutable methods

function fmap(m, fn) {
	var x = {};
	_.each(m, function (v, k) {
		x[k] = fn(v, k);
	});
	return x;
}

// Return function that takes an array
function apply(fn, self) {
	return fn.apply.bind(fn, self);
}

// Create array from arguments
function array() {
	return _.toArray(arguments);
}

// Concat arrays
function concat() {
	var _ref;

	return (_ref = []).concat.apply(_ref, arguments);
}

function pluckPaths(paths, obj) {
	return _.fmap(paths, _.partial(_.getIn, obj));
}

function pluckPathsArray(paths, obj) {
	return _.map(paths, _.partial(_.getIn, obj));
}

function partitionN(arr, n, step, pad) {
	var i,
	    last,
	    len,
	    ret = [];

	step = step || n;
	for (i = 0; i < arr.length; i += step) {
		ret.push(arr.slice(i, i + n));
	}

	last = ret[ret.length - 1];
	if (last) {
		len = last.length;
		if (pad && len < n) {
			ret[ret.length - 1] = last.concat(pad.slice(0, n - len));
		}
	}
	return ret;
}

function objectFn(keys, fn) {
	return _.reduce(keys, function (acc, k) {
		acc[k] = fn(k);
		return acc;
	}, {});
}

/* Like _.find, but return the result of the predicate */
function findValue(obj, predicate, context) {
	var result;
	_.any(obj, function (value, index, list) {
		result = predicate.call(context, value, index, list);
		return result;
	});
	return result;
}

// XXX Drop this when upgrading underscore
function negate(predicate) {
	return function () {
		return !predicate.apply(this, slice.call(arguments, 0));
	};
}

function spy(msg, x) {
	if (console) {
		console.log(msg, x);
	}
	return x;
}

// Find max using a cmp function. Same as arr.slice[0].sort(cmp)[0],
// but O(n) instead of O(n log n).
function maxWith(arr, cmp) {
	return _.reduce(arr, function (x, y) {
		return cmp(x, y) < 0 ? x : y;
	});
}

function memoize1(fn) {
	var last = null,
	    lastParams = null;
	return function () {
		if (!_.isEqual(arguments, lastParams)) {
			last = fn.apply(this, arguments);
			lastParams = arguments;
		}
		return last;
	};
}

function meannull(values) {
	var count = 0,
	    sum = 0;
	if (!values) {
		return null;
	}
	sum = _.reduce(values, function (sum, v) {
		if (v != null) {
			count += 1;
			return sum + v;
		}
		return sum;
	}, 0);
	if (count > 0) {
		return sum / count;
	}
	return null;
}

// Version that works with iterables. Not using it for now
// due to babel generator performance. See below.
//function meannull(values) {
//	var count = 0, sum = 0;
//	if (!values) {
//		return null;
//	}
//	for (let v of values) {
//		if (v != null) {
//			count += 1;
//			sum += v;
//		}
//	}
//	return (count > 0) ? sum / count : null;
//}

function cmpNumberOrNull(v1, v2) {
	if (v1 == null && v2 == null) {
		return 0;
	} else if (v1 == null) {
		return 1;
	} else if (v2 == null) {
		return -1;
	}
	return v2 - v1;
}

function medianNull(values) {
	if (!values) {
		return null;
	}
	var sorted = values.slice(0).sort(cmpNumberOrNull),
	    firstNull = sorted.indexOf(null),
	    notNull = firstNull !== -1 ? sorted.slice(0, firstNull) : sorted,
	    n = notNull.length;

	return (notNull[Math.ceil(n / 2)] + notNull[Math.floor(n / 2)]) / 2;
}

function fmapMemoize1(fn) {
	var prevObj = {},
	    cache = {};
	return function memoized(obj) {
		// Preserve reference equality when input is the same.
		if (_.isEqual(obj, prevObj)) {
			return cache;
		}
		var next = _.mapObject(obj, function (val, key) {
			return _.isEqual(prevObj[key], val) ? cache[key] : fn(val, key);
		});

		// If all values are the same, preserve reference equality.
		// XXX this is weird.
		cache = _.isEqual(next, cache) ? cache : next;
		prevObj = obj;
		return cache;
	};
}

function curry2(fn) {
	return function (a, b) {
		switch (arguments.length) {
			case 1:
				return function (b) {
					return fn(a, b);
				};
			// use 'default', not 2, to catch & discard extra params.
			default:
				return fn(a, b);
		}
	};
}

function curry3(fn) {
	return function (a, b, c) {
		switch (arguments.length) {
			case 1:
				return function (b, c) {
					switch (arguments.length) {
						case 1:
							return function (c) {
								return fn(a, b, c);
							};
						// use 'default', not 2, to catch & discard extra params.
						default:
							return fn(a, b, c);
					}
				};
			case 2:
				return function (c) {
					return fn(a, b, c);
				};
			// use 'default', not 3, to catch & discard extra params.
			default:
				return fn(a, b, c);
		}
	};
}

function curryArgs(args, accN, fn) {
	var n = args.length;
	return function () {
		var m = arguments.length,
		    sum = accN + m;
		for (var i = 0; i < m; ++i) {
			args[accN + i] = arguments[i];
		}
		return sum >= n ? fn.apply(undefined, (0, _toConsumableArray3.default)(args)) :
		// every new curry must have its own copy of the args array.
		curryArgs(args.slice(0), sum, fn);
	};
}

// this no longer handles extra args passed to the fn. Should we care?
function curryN(n, fn) {
	// Unroll the most common cases.
	switch (n) {
		case 1:
			return fn;
		case 2:
			return curry2(fn);
		case 3:
			return curry3(fn);
	}
	return curryArgs(new Array(n), 0, fn);
}

function mmapper(cols, n, fn) {
	// Unroll the most common cases.
	switch (n) {
		// case 1 looks like a noop, i.e. we could return fn. However _.map also passes 'this', which
		// messes us up.
		case 1:
			return function (v0, i) {
				return fn(v0, i);
			};
		case 2:
			return function (v0, i) {
				return fn(v0, cols[1][i], i);
			};
		case 3:
			return function (v0, i) {
				return fn(v0, cols[1][i], cols[2][i], i);
			};
		case 4:
			return function (v0, i) {
				return fn(v0, cols[1][i], cols[2][i], cols[3][i], i);
			};
	}
	var buff = new Array(n + 1); // reuse buffer to avoid allocation
	return function (v0, i) {
		for (var j = 0; j < n; ++j) {
			buff[j] = cols[j][i];
		}
		buff[j] = i;
		return fn.apply(undefined, buff);
	};
}

function mmap() {
	for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
		args[_key] = arguments[_key];
	}

	var fn = args[args.length - 1];
	return (args[0] || []).map(mmapper(args, args.length - 1, fn));
}

function scanI(arr, fn, acc, i, out) {
	var _repeat = true;

	var _arr, _fn, _acc, _i, _out;

	while (_repeat) {
		_repeat = false;

		if (i >= arr.length) {
			return out;
		}
		var next = fn(acc, arr[i]);
		out.push(next);
		_arr = arr;
		_fn = fn;
		_acc = next;
		_i = i + 1;
		_out = out;
		arr = _arr;
		fn = _fn;
		acc = _acc;
		i = _i;
		out = _out;
		_repeat = true;
		continue;
	}
}

function scan(arr, fn, acc) {
	if (arguments.length < 3) {
		if (arr.length === 0) {
			throw new Error("scan of empty array with no initial value");
		}
		return scanI(arr, fn, arr[0], 1, [arr[0]]);
	}
	return scanI(arr, fn, acc, 0, [acc]);
}

var curry = function curry(fn) {
	return curryN(fn.length, fn);
};

var withoutIndex = function withoutIndex(arr, i) {
	return arr.slice(0, i).concat(arr.slice(i + 1));
};

// Return indices of arr for which fn is true. fn is passed the value and index.
var filterIndices = function filterIndices(arr, fn) {
	return _.range(arr.length).filter(function (i) {
		return fn(arr[i], i);
	});
};

// non-destructive reverse
var reverse = function reverse(arr) {
	return arr.slice(0).reverse();
};

// Group by consecutive matches, perserving order.
function groupByConsec(sortedArray, prop, ctx) {
	var cb = _.iteratee(prop, ctx);
	var last = {},
	    current; // init 'last' with a sentinel, !== to everything
	return _.reduce(sortedArray, function (acc, el) {
		var key = cb(el);
		if (key !== last) {
			current = [];
			last = key;
			acc.push(current);
		}
		current.push(el);
		return acc;
	}, []);
}

function findIndexDefault(arr, pred, def) {
	var i = _.findIndex(arr, pred);
	return i < 0 ? def : i;
}

function findLastIndexDefault(arr, pred, def) {
	var i = _.findLastIndex(arr, pred);
	return i < 0 ? def : i;
}

// underscore uniq is absurdly slow
var uUniq = _.uniq;
function unique(arr) {
	for (var _len2 = arguments.length, rest = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
		rest[_key2 - 1] = arguments[_key2];
	}

	if (rest.length > 0) {
		return uUniq.apply(undefined, [arr].concat(rest));
	}
	return [].concat((0, _toConsumableArray3.default)(new _set2.default(arr)));
}

function duplicates(arr) {
	var sorted = arr.sort(),
	    result = [];

	for (var i = 1; i < sorted.length; i++) {
		if (arr[i - 1] === arr[i]) {
			result.push(arr[i]);
		}
	}
	return result;
}

// underscore union is slow, due to n^2 algorithm using _.contains.
// Perserves order by using unique(), which preserves order by use of
// Set.
function union() {
	for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
		args[_key3] = arguments[_key3];
	}

	var flattened = args.reduce(function (a, b) {
		return a.concat(b);
	}, []);
	return unique(flattened);
}

// make unique by appending numbered suffix
function uniquify(strs) {
	var counts = {};
	return strs.map(function (s) {
		var c = counts[s] = _.has(counts, s) ? counts[s] + 1 : 0;
		return c > 0 ? s + ' (' + c + ')' : s;
	});
}

var insert = function insert(arr, i, vals) {
	return [].concat((0, _toConsumableArray3.default)(arr.slice(0, i)), (0, _toConsumableArray3.default)(vals), (0, _toConsumableArray3.default)(arr.slice(i)));
};
// immutable splice
var splice = function splice(arr, i, c) {
	for (var _len4 = arguments.length, vals = Array(_len4 > 3 ? _len4 - 3 : 0), _key4 = 3; _key4 < _len4; _key4++) {
		vals[_key4 - 3] = arguments[_key4];
	}

	var ret = arr.slice(0);
	ret.splice.apply(ret, [i, c].concat(vals));
	return ret;
};

function listSetsEqual(l1, l2) {
	if (l1.length !== l2.length) {
		return false;
	}
	var s1 = new _set2.default(l1);
	return _.every(l2, function (v) {
		return s1.has(v);
	});
}

// Return paths in obj which match path, where path is a path array
// of form [key | matchKeys.any, key | matchKeys.any, ...].
// key will match a literal string key. any will match any key.
// E.g. ['a', matchKey.any] will match ['a', 'b'], ['a', 'c'] in
// {a: {b: 0, c: 1}}
var any = {};
var matchKeys = function matchKeys(obj, path) {
	var i = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
	return i === path.length ? [path] : path[i] === any ? (0, _keys2.default)(obj).map(function (k) {
		return matchKeys(obj[k], splice(path, i, 1, k), i + 1);
	}).flatten() : !obj.hasOwnProperty(path[i]) ? [] : matchKeys(obj[path[i]], path, i + 1);
};

matchKeys.any = any;

function anyRange(coll, start, end) {
	var pred = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : _.identity;
	var i = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : start;
	var _repeat2 = true;

	var _coll, _start, _end, _pred, _i2;

	while (_repeat2) {
		_repeat2 = false;

		if (i === end) {
			return false;
		} else {
			if (pred(coll[i])) {
				return true;
			} else {
				_coll = coll;
				_start = start;
				_end = end;
				_pred = pred;
				_i2 = i + 1;
				coll = _coll;
				start = _start;
				end = _end;
				pred = _pred;
				i = _i2;
				_repeat2 = true;
				continue;
			}
		}
	}
}

// In theory one could apply zip, but 'apply' will blow the
// stack on chrome.
var transpose = function transpose(coll) {
	return !coll || !coll.hasOwnProperty('length') ? [] : coll.length === 0 ? coll : coll[0].map(function (v, i) {
		return coll.map(function (line) {
			return line[i];
		});
	});
};

// mutating 'push' that returns the array
var push = function push(arr, v) {
	return arr.push(v), arr;
};

// Starting some iterator methods here, but there are some performance
// concerns. babel generators are slow, possibly due to injecting a try/catch.
//
//function* imap(arr, fn) {
//	for (let v of arr) {
//		yield fn(v);
//	}
//}
//

// Iterable version. Wow, this is ugly.
function imap(iterable, fn) {
	return (0, _defineProperty3.default)({}, _iterator2.default, function () {
		var iter = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _getIterator3.default)(iterable);
		return {
			next: function next() {
				var n = iter.next();
				return n.done ? { done: true } : { value: fn(n.value) };
			}
		};
	});
}

// repeat an iterable n times.
function repeat(n, iterable) {
	return _regenerator2.default.wrap(function repeat$(_context) {
		while (1) {
			switch (_context.prev = _context.next) {
				case 0:
					if (!n--) {
						_context.next = 4;
						break;
					}

					return _context.delegateYield((0, _getIterator3.default)(iterable), 't0', 2);

				case 2:
					_context.next = 0;
					break;

				case 4:
				case 'end':
					return _context.stop();
			}
		}
	}, _marked, this);
}

function valToStr(v) {
	return !isNaN(v) && v !== null && v !== undefined ? "" + v : "";
}

// string slice() will hold a copy of the original string, which
// will run us out of memory when processing buffers. So, force
// a mem copy.
var copyStr = function copyStr(str) {
	return (' ' + str).slice(1);
};

//
//function* irange(n) {
//	for (let i = 0; i < n; ++i) {
//		yield i;
//	}
//}
//
//// tack on an iterator namespace. We should probably find a better
//// functional methods library that supports iterators.
_.iterable = {
	map: imap,
	repeat: repeat
};

_.mixin({
	anyRange: anyRange,
	apply: apply,
	array: array,
	concat: concat,
	copyStr: copyStr,
	cmpNumberOrNull: cmpNumberOrNull,
	curry: curry,
	curryN: curryN, // useful if the fn as multiple arities.
	duplicates: duplicates,
	filterIndices: filterIndices,
	findIndexDefault: findIndexDefault,
	findLastIndexDefault: findLastIndexDefault,
	findValue: findValue,
	flatmap: _.compose(_.partial(_.flatten, _, 1), _.map),
	fmap: fmap,
	fmapMemoize1: fmapMemoize1,
	groupByConsec: groupByConsec,
	insert: insert,
	listSetsEqual: listSetsEqual,
	matchKeys: matchKeys,
	maxWith: maxWith,
	maxnull: function maxnull(arr) {
		return _.max(arr, function (v) {
			return v == null || isNaN(v) ? -Infinity : v;
		});
	},
	meannull: meannull,
	medianNull: medianNull,
	memoize1: memoize1,
	merge: function merge() {
		for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
			args[_key5] = arguments[_key5];
		}

		return _.extend.apply(null, [{}].concat(args));
	},
	minnull: function minnull(arr) {
		return _.min(arr, function (v) {
			return v == null || isNaN(v) ? Infinity : v;
		});
	},
	mmap: mmap,
	negate: negate,
	objectFn: objectFn,
	partitionN: partitionN,
	pluckPaths: pluckPaths,
	pluckPathsArray: pluckPathsArray,
	push: push,
	reverse: reverse,
	scan: scan,
	splice: splice,
	spy: spy,
	sum: function sum(arr) {
		return _.reduce(arr, function (x, y) {
			return x + y;
		}, 0);
	},
	transpose: transpose,
	union: union,
	uniq: unique,
	unique: unique,
	uniquify: uniquify,
	withoutIndex: withoutIndex,
	valToStr: valToStr,
	// This inscrutable method allows one to write a 'let' expression via
	// es6 default arguments, e.g. _.Let((x = 5, y = x + 2) => x + y) === 12
	Let: function Let(f) {
		return f();
	}
});

module.exports = _;