'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.defaultHost = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _columnsParam = require('./columnsParam');

var _heatmapParam = require('./heatmapParam');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('./underscore_ext'),
    get = _require.get,
    getIn = _require.getIn,
    Let = _require.Let,
    merge = _require.merge,
    mapObject = _require.mapObject,
    pick = _require.pick,
    pluck = _require.pluck,
    uniq = _require.uniq,
    updateIn = _require.updateIn;

var _require2 = require('./bookmark'),
    hasBookmark = _require2.hasBookmark,
    resetBookmarkLocation = _require2.resetBookmarkLocation,
    getBookmark = _require2.getBookmark;

var _require3 = require('./inlineState'),
    hasInlineState = _require3.hasInlineState,
    resetInlineStateLocation = _require3.resetInlineStateLocation;

var _require4 = require('./hubParams'),
    getHubParams = _require4.hubParams;

var _require5 = require('./util'),
    allParameters = _require5.allParameters;

// This is all really wonky & needs refactor.

function bookmarkParam() {
	var ret = {};
	if (hasBookmark()) {
		ret = { 'bookmark': getBookmark() };
		resetBookmarkLocation();
	}
	return ret;
}

function inlineStateParam() {
	var ret = {};
	if (hasInlineState()) {
		ret = { 'inlineState': true };
		resetInlineStateLocation();
	}
	return ret;
}

// XXX Deprecating these, in favor of hubParams2.
function hubParams() {
	var hubs = getHubParams();
	return hubs.length ? { hubs: hubs } : {};
}

var hubParams2 = pick(allParameters(), 'addHub', 'removeHub');

function fixLocalhost(obj) {
	return get(obj, 'host') ? updateIn(obj, ['host'], function (host) {
		return host === 'https://local.xena.ucsc.edu:7223' ? 'http://127.0.0.1:7222' : host;
	}) : obj;
}

function datasetParams() {
	// only take the first of these
	return mapObject(pick(allParameters(), 'cohort', 'dataset', 'host', 'allIdentifiers', 'markdown'), function (l) {
		return l[0];
	});
}

function manifest() {
	// only take the first of these
	return mapObject(pick(allParameters(), 'manifest'), function (l) {
		return l[0];
	});
}

function getParams() {
	var columns = (0, _columnsParam.columnsParam)(),
	    hasCols = getIn(columns, ['columns', 'length'], 0) > 0,

	// ignore heatmap param w/o column param.
	heatmap = hasCols ? (0, _heatmapParam.heatmapParam)() : {},
	    hub2 = hasCols ? updateIn(hubParams2, ['addHub'], function () {
		var hubs = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
		return uniq(hubs.concat(pluck(columns.columns, 'host')));
	}) : hubParams2;
	return merge(hub2, bookmarkParam(), inlineStateParam(), hubParams(), fixLocalhost(datasetParams()), manifest(), columns, heatmap);
}

// Our handling of parameters 'hub' and 'host', is somewhat confusing. 'host'
// means "show the hub page for this url". 'hub' means "add this url to the
// active hub list, and, if in /datapages/ show the hub page for this url".
// The 'hub' parameter can be repeated, which adds each hub to the active hub
// list. Only the first one will be displayed when linking to /datapages/.
// Needs refactor.
var defaultHost = exports.defaultHost = function defaultHost(params) {
	return Let(function () {
		var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : params,
		    host = _ref.host,
		    hubs = _ref.hubs;

		return !host && hubs ? (0, _extends3.default)({}, params, { host: hubs[0] }) : params;
	});
};

exports.default = getParams;