"use strict";

var _slicedToArray2 = require("babel-runtime/helpers/slicedToArray");

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('./underscore_ext'),
    flatmap = _require.flatmap,
    apply = _require.apply,
    map = _require.map,
    first = _require.first,
    zip = _require.zip,
    times = _require.times,
    isArray = _require.isArray,
    constant = _require.constant,
    mapObject = _require.mapObject,
    groupBy = _require.groupBy;

function expandArrays(v, k) {
	if (isArray(v)) {
		return zip(v, times(v.length, constant(k)));
	}
	return [[v, k]];
}

var encodeParam = function encodeParam(v, k) {
	return k + "=" + encodeURIComponent(v);
};

var encodeObject = function encodeObject(obj) {
	return map(flatmap(obj, expandArrays), apply(encodeParam)).join("&");
};

var searchParams = function searchParams(search) {
	return search.length > 1 ? mapObject(groupBy(search.slice(1).split('&').map(function (exp) {
		return exp.split('=').map(decodeURIComponent);
	}), first), function (arr) {
		return arr.map(function (_ref) {
			var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
			    val = _ref2[1];

			return val;
		});
	}) : {};
};

module.exports = {
	encodeObject: encodeObject,
	getParameterByName: function getParameterByName(name) {
		// TODO duplicates galaxy.js, so extract into common file
		// see http://stackoverflow.com/questions/901115/how-can-i-get-query-string-values/901144#901144
		var match = new RegExp('[?&]' + name + '=([^&]*)').exec(window.location.search);
		return match && decodeURIComponent(match[1].replace(/\+/g, ' '));
	},
	urlParams: function urlParams(url) {
		var i = url.indexOf('?');
		return i === -1 ? {} : searchParams(url.slice(i));
	},
	allParameters: function allParameters() {
		return searchParams(location.search);
	},
	eventOffset: function eventOffset(ev) {
		var _ev$currentTarget$get = ev.currentTarget.getBoundingClientRect(),
		    top = _ev$currentTarget$get.top,
		    left = _ev$currentTarget$get.left;

		return {
			x: ev.pageX - (left + window.pageXOffset),
			y: ev.pageY - (top + window.pageYOffset)
		};
	},

	// utility fxn to add commas to a number str
	// found at: http://www.mredkj.com/javascript/numberFormat.html
	addCommas: function addCommas(nStr) {
		nStr += '';
		var x = nStr.split('.'),
		    x1 = x[0],
		    x2 = x.length > 1 ? '.' + x[1] : '',
		    rgx = /(\d+)(\d{3})/;
		while (rgx.test(x1)) {
			x1 = x1.replace(rgx, '$1' + ',' + '$2');
		}
		return x1 + x2;
	},

	hasClass: function hasClass(el, c) {
		return el.className.split(/ +/).indexOf(c) !== -1;
	},

	caseInsensitiveSort: function caseInsensitiveSort(a, b) {
		if (a.toUpperCase() < b.toUpperCase()) {
			return -1;
		} else if (a.toUpperCase() > b.toUpperCase()) {
			return 1;
		} else {
			return 0;
		}
	}
};