'use strict';

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// This is a mixed-mode vg renderer, using canvas for paint directives, and DOM
// for labels. It is *very* simplistic, and doesn't expose a coherent interface,
// due to the complexity, and lack of need for something so sophisticated.
//
// In particular, it makes the following assumptions:
// vg.textCenteredPushRight is never called following translate or scale. I.e. units
//   are relative to the canvas target.
// vg.clip is only used for clipping text, and here is redundant due to clipping of
//   the label itself, via overflow 'hidden'.
//
// We introduce a new 'labels' call that allocates a context for a set of textCenteredPushRight
// calls, and render via React when the context exits.
var vgcanvas = require('./vgcanvas');
var React = require('react');
var ReactDOM = require('react-dom');

module.exports = function (el, vgw, vgh, labelEl) {
	return (0, _extends3.default)({}, vgcanvas(el, vgw, vgh), {
		clip: function clip(x, y, width, height, cb) {
			cb();
		},
		labels: function labels(cb) {
			this.labelCtx = [];
			cb();
			// Below, we set 'hyphens' to 'none' if there's no room for a 2nd line, so we can fit more
			// of the text on-screen (instead of hyphenating & wrapping to the 2nd line that we can't see).
			//
			// Also, we clip the height to an integer multiple of lineHeight, so we don't, for example, render
			// the top pixel of the 2nd line, which looks distracting.
			var style = { textAlign: 'center' },
			    labels = this.labelCtx.map(function (_ref, i) {
				var _ref2 = (0, _slicedToArray3.default)(_ref, 7),
				    left = _ref2[0],
				    top = _ref2[1],
				    width = _ref2[2],
				    h = _ref2[3],
				    color = _ref2[4],
				    f = _ref2[5],
				    txt = _ref2[6];

				var lineHeight = 1.2 * f,
				    hyphens = h < 2 * lineHeight ? 'none' : 'auto',
				    // Don't hyphenate on a single line
				height = Math.floor(h / lineHeight) * lineHeight; // Render integer number of lines
				return React.createElement(
					'div',
					{ key: i, className: 'ColumnLabels',
						style: {
							left: left,
							top: top,
							width: width,
							color: color,
							lineHeight: lineHeight + 'px',
							height: height } },
					React.createElement(
						'p',
						{ className: 'ColumnLabel', style: {
								fontSize: f + 'px',
								width: width,
								hyphens: hyphens } },
						'' + txt
					)
				);
			});
			ReactDOM.render(React.createElement(
				'div',
				{ style: style },
				labels
			), labelEl);
			this.labelCtx = undefined;
		},
		textCenteredPushRight: function textCenteredPushRight() {
			if (this.labelCtx === undefined) {
				console.trace('textCenteredPushRight');
				return;
			}

			for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
				args[_key] = arguments[_key];
			}

			this.labelCtx.push(args);
		}
	});
};