'use strict';

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _button = require('react-toolbox/lib/button');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * UCSC Xena Client
 * http://xena.ucsc.edu
 *
 * Standard Xena help box, hand-rolled outside of RTB as we may require more than one help box (ie dialog) open
 * at a time, and we want to be able to control the overlay or lack thereof. RTB currently does not allow multiple
 * dialogs open at the same time, or theming to control overlay styles.
 *
 * There are two possible formats of help boxes.
 * 1. A help box positioned to the right of the item being highlighted. The help box is drawn with a triangle on the
 *   left-hand side of the help box dialog, and a dotted line is drawn from the left-hand side of the viewport to the
 *   triangle.
 * 2. A help box positioned below the item being highlighted. The help box is drawn with a triangle on the top side of
 *    dialog help box.
 *
 * State
 * -----
 * - x: Position of left hand side of help box.
 * - y: Position of top of help box.
 * - w: Width of help box.
 * - o: Orientation of help box (to determine where to place/point help box triangle), 'Right' or 'Below'.
 *
 * Actions
 * -------
 * - onClose: Called when "GOT IT" button clicked to hide help box.
 *
 * Children should be in the format:
 * <p>Lorem ipsum dolor sit amet, consectetur adipiscing elit.</p>
 *
 * For example,
 * <HelpBox x={882} y={271} w={280} o={'Right'}><p>Each row contains data from a single sample.</p><p>Row order is determined by sorting the rows by their column values.</p></HelpBox>
 */

// Core dependencies, components
var React = require('react');
var classNames = require('classnames');


// Styles
var compStyles = require('./HelpBox.module.css');

// Setup for help box displayed to right of item being highlighted (with row marker). 27.5 used in width calculations
// of marker comes from 16px margin (standard MD width) plus 11.5px for triangle. This smaller width creates the
// necessary break between the marker and the right triangle.
var HelpBoxR = function HelpBoxR(props) {
	var w = props.w,
	    children = props.children;

	return React.createElement(
		'div',
		{ className: classNames(compStyles.HelpBox, compStyles.withMarker, compStyles.helpRight) },
		React.createElement('div', { className: compStyles.rowMarker }),
		React.createElement(
			'div',
			{ style: { width: w }, className: compStyles.helpBoxBounds },
			children
		)
	);
};

// Setup for help box displayed below item being highlighted.
var HelpBoxB = function HelpBoxB(props) {
	var w = props.w,
	    children = props.children;

	return React.createElement(
		'div',
		{ style: { width: w }, className: classNames(compStyles.HelpBox, compStyles.helpBoxBounds, compStyles.helpBelow) },
		children
	);
};

var HelpBox = function (_React$Component) {
	(0, _inherits3.default)(HelpBox, _React$Component);

	function HelpBox() {
		var _ref;

		var _temp, _this, _ret;

		(0, _classCallCheck3.default)(this, HelpBox);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref = HelpBox.__proto__ || (0, _getPrototypeOf2.default)(HelpBox)).call.apply(_ref, [this].concat(args))), _this), _this.onClose = function () {
			_this.props.onClose();
		}, _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
	}

	(0, _createClass3.default)(HelpBox, [{
		key: 'render',
		value: function render() {
			var _props = this.props,
			    children = _props.children,
			    o = _props.o,
			    boxProps = (0, _objectWithoutProperties3.default)(_props, ['children', 'o']);

			var Box = o === 'Right' ? HelpBoxR : HelpBoxB;
			return React.createElement(
				Box,
				boxProps,
				React.createElement(
					'div',
					{ className: compStyles.helpBoxContent },
					children
				),
				React.createElement(
					'div',
					{ className: compStyles.buttonContainer },
					React.createElement(
						_button.Button,
						{ accent: true, onClick: this.onClose },
						'GOT IT'
					)
				)
			);
		}
	}]);
	return HelpBox;
}(React.Component);

module.exports = HelpBox;