'use strict';

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _PureComponent2 = require('../PureComponent');

var _PureComponent3 = _interopRequireDefault(_PureComponent2);

var _columns = require('../models/columns');

var _rx = require('../rx');

var _columnsParam = require('../columnsParam');

var _defaultServers = require('../defaultServers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var React = require('react');
var _ = require('../underscore_ext');
var XCheckboxGroup = require('./XCheckboxGroup');
var XRadioGroup = require('./XRadioGroup');
var WizardCard = require('./WizardCard');
var GeneSuggest = require('./GeneSuggest');
var PhenotypeSuggest = require('./PhenotypeSuggest');

var _require = require('../react-utils'),
    rxEvents = _require.rxEvents;

var parsePos = require('../parsePos');

var _require2 = require('../models/dataType'),
    ignoredType = _require2.ignoredType;

var LOCAL_DOMAIN = _defaultServers.servers.localHub;
var LOCAL_DOMAIN_LABEL = 'My Computer Hub';

var ignoredClinical = function ignoredClinical(type, subtype) {
	return type === 'clinicalMatrix' && (!subtype || subtype.match(/^phenotype/i));
}; // match ../controllers/wizard.js definition of phenotype data

var notIgnored = function notIgnored(_ref) {
	var type = _ref.type,
	    dataSubType = _ref.dataSubType;
	return !_.contains(ignoredType, type) && !ignoredClinical(type, dataSubType);
};

var category = function category(_ref2) {
	var dsID = _ref2.dsID,
	    dataSubType = _ref2.dataSubType;
	return dsID.includes(LOCAL_DOMAIN) ? LOCAL_DOMAIN_LABEL : dataSubType ? dataSubType : 'others';
};

function createLabels(datasets) {
	var sorted = _.sortBy(datasets, function (ds) {
		return ds.label.toLowerCase();
	}),
	    labels = _.uniquify(_.pluck(sorted, 'label'));
	return _.mmap(sorted, labels, function (_ref3, label) {
		var dsID = _ref3.dsID;
		return { value: dsID, label: label };
	});
}

// Create dataset list. Sorts by category and label, and enforces unique labels by
// appending a suffix.
function datasetList(datasets) {
	var groups = _.fmap(_.groupBy(_.values(datasets).filter(notIgnored), category), createLabels);
	return _.sortBy(_.keys(groups), function (g) {
		return g.toLowerCase();
	}).map(function (group) {
		return {
			label: group,
			options: groups[group]
		};
	});
}

var preferredList = function preferredList(preferred) {
	return [{
		options: preferred.map(function (_ref4) {
			var dsID = _ref4.dsID,
			    label = _ref4.label;
			return { value: dsID, label: label };
		})
	}];
};

var RETURN = 13;
var returnPressed = function returnPressed(cb) {
	return cb ? function (ev) {
		return ev.keyCode === RETURN && cb();
	} : undefined;
};

function selectedOptions(selected, options) {
	var smap = new _set2.default(selected);
	return options.map(function (group) {
		return _.updateIn(group, ['options'], function (options) {
			return options.map(function (opt) {
				return smap.has(opt.value) ? _.assoc(opt, 'checked', true) : opt;
			});
		});
	});
}

var assemblyColors = {
	hg18: '#527DA4',
	hg19: '#ff5722',
	hg38: '#77ADA7',
	default: '#999999'
};

var assemblyColor = function assemblyColor(assembly) {
	return _.get(assemblyColors, assembly, assemblyColors.default);
};

var getAssembly = _.curry(function (datasets, dsID) {
	return _.getIn(datasets, [dsID, 'assembly'], _.getIn(datasets, [dsID, 'probemapMeta', 'assembly']));
});

var setBadge = function setBadge(datasets) {
	return function (ds) {
		return _.Let(function () {
			var assembly = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getAssembly(datasets, ds.value);
			return assembly ? (0, _extends3.default)({}, ds, {
				badge: { label: assembly, style: { color: assemblyColor(assembly) } }
			}) : ds;
		});
	};
};

var setAssembly = function setAssembly(datasets, groups) {
	return groups.map(function (group) {
		return _.updateIn(group, ['options'], function (list) {
			return list.map(setBadge(datasets));
		});
	});
};

var defaultAssembly = 'hg38';
// For gene lookup, use the first assembly we find. If no datasets
// are selected, use a default so the user can get gene suggestions before
// selecting a dataset.
var firstAssembly = function firstAssembly(datasets, selected) {
	return selected.length === 0 ? defaultAssembly : _.findValue(selected, getAssembly(datasets));
};

var GenotypicForm = function GenotypicForm(props) {
	return React.createElement(
		'div',
		null,
		React.createElement(GeneSuggest, {
			dataset: props.selected.length === 1 && _.indexOf(['genomicMatrix', 'clinicalMatrix'], props.datasets[props.selected[0]].type) !== -1 ? props.selected[0] : undefined,
			assembly: firstAssembly(props.datasets, props.selected),
			error: props.error,
			value: props.value,
			onKeyDown: returnPressed(props.onReturn),
			onChange: props.onFieldChange,
			type: 'text' }),
		React.createElement(XCheckboxGroup, {
			label: 'Dataset',
			additionalAction: !_.isEmpty(props.preferred) && (props.advanced ? 'Show Basic' : 'Show Advanced'),
			onAdditionalAction: props.onAdvancedClick,
			onChange: props.onChange,
			hideBadge: props.hideAssembly,
			options: selectedOptions(props.selected, setAssembly(props.datasets, props.advanced ? datasetList(props.datasets) : preferredList(props.preferred))) })
	);
};

var basicFeatureLabels = function basicFeatureLabels(features, basicFeatures) {
	return basicFeatures.map(function (i) {
		return { value: i.toString(), label: features[i].label };
	});
};

var allFeatureLabels = function allFeatureLabels(features) {
	return features.map(function (f, i) {
		return { value: i.toString(), label: f.label };
	});
};

var PhenotypicForm = function PhenotypicForm(props) {
	var options = (props.advanced ? allFeatureLabels : basicFeatureLabels)(props.features, _.union(props.basicFeatures, props.selected));
	return React.createElement(
		'div',
		null,
		React.createElement(XCheckboxGroup, {
			label: 'Phenotype',
			additionalAction: !_.isEmpty(props.preferred) && (props.advanced ? 'Show Basic' : 'Show All'),
			onAdditionalAction: props.onAdvancedClick,
			onChange: props.onChange,
			options: selectedOptions(props.selected, [{ options: options }]) }),
		props.advanced ? null : React.createElement(PhenotypeSuggest, {
			error: props.error,
			value: props.value,
			features: props.features,
			onSuggestionSelected: function onSuggestionSelected(ev, _ref5) {
				var suggestion = _ref5.suggestion;
				return props.onAddFeature(suggestion);
			},
			onKeyDown: returnPressed(props.onAddFeature),
			onChange: props.onFieldChange, type: 'text' })
	);
};

var AnalyticForm = function AnalyticForm(props) {
	var options = props.analytic.map(function (_ref6, i) {
		var label = _ref6.label;
		return { value: i.toString(), label: label };
	});
	return React.createElement(
		'div',
		null,
		React.createElement(XCheckboxGroup, {
			label: 'Variable',
			onChange: props.onChange,
			options: selectedOptions(props.selected, [{ options: options }]) })
	);
};

var getModeFields = {
	Genotypic: GenotypicForm,
	Phenotypic: PhenotypicForm,
	Analytic: AnalyticForm
};

var applyInitialState = {
	Genotypic: function Genotypic(text, fields, dataset, datasets, features, preferred, defaults) {
		var mode = 'Genotypic',
		    isPreferred = _.contains(_.pluck(preferred, 'dsID'), dataset),

		// old bookmarks may not have a 'text' property
		value = text || fields.join(' '),
		    selected = [dataset];

		return _.assocIn(defaults, ['mode'], mode, ['advanced', mode], !isPreferred, ['value', mode], value, ['selected', mode, !isPreferred], selected);
	},
	Phenotypic: function Phenotypic(text, fields, dataset, datasets, features, preferred, defaults) {
		var mode = 'Phenotypic',
		    i = _.findIndex(features, _.matcher({ dsID: dataset, name: fields[0] })).toString(),
		    selected = [i];

		return i === '-1' ? _.assocIn(defaults, ['unavailable'], true) : _.assocIn(defaults, ['mode'], mode, ['basicFeatures'], defaults.basicFeatures, ['selected', mode, false], selected);
	},
	'undefined': function undefined(text, fields, dataset, datasets, features, preferred, defaults) {
		return _.assocIn(defaults, ['unavailable'], true);
	}
};

var datasetMode = function datasetMode(datasets, dataset) {
	return datasets[dataset] ? notIgnored(datasets[dataset]) ? 'Genotypic' : 'Phenotypic' : undefined;
};

var pluralDataset = function pluralDataset(i) {
	return i === 1 ? 'A dataset' : 'Some datasets';
};
var pluralDo = function pluralDo(i) {
	return i === 1 ? 'does' : 'do';
};
//var pluralHas = i => i === 1 ? 'has' : 'have';

function getWarningText(matches, datasets, selected, topWarnings, value) {
	var pos = parsePos(value),
	    warnings = _.groupBy(matches, function (m) {
		return m.warning;
	}),
	    unsupported = _.getIn(warnings, ['position-unsupported', 'length'], 0),
	    uwarn = unsupported ? [pluralDataset(unsupported) + ' in your selection ' + pluralDo(unsupported) + ' not support a chromosome view.'] : [],
	    probes = _.getIn(warnings, ['too-many-probes', 'length'], 0),
	    max = _.min(warnings['too-many-probes'], function (m) {
		return m.end;
	}),
	    pwarn = probes && pos ? ['There are too many data points to display. Please try a smaller region like ' + pos.chrom + ':' + max.start + '-' + max.end + '.'] : [];

	return [].concat((0, _toConsumableArray3.default)(topWarnings), uwarn, pwarn);
}

var featureIndexes = function featureIndexes(features, list) {
	return list.map(function (f) {
		return _.findIndex(features, _.matcher(f)).toString();
	}).filter(function (x) {
		return x !== "-1";
	});
};

var toDsID = function toDsID(_ref7) {
	var host = _ref7.host,
	    name = _ref7.name;
	return (0, _stringify2.default)({ host: host, name: name });
};

var doMatch = function doMatch(datasets, dsID, field) {
	var opts = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
	return (0, _columns.matchDatasetFields)(datasets, dsID, field).map(function (r) {
		return (0, _extends3.default)({}, r, { dataset: datasets[dsID], opts: opts });
	});
};

var assemblyError = 'Your dataset selections include two different assemblies. For chromosome coordinates, the assembly must be unique.';
var fieldError = 'None of these fields are available on all selected datasets.';
var sigError = 'Unable to parse signature.';

function intersectFields(matches) {
	if (matches.length === 0) {
		return matches;
	}
	var intersection = _.filterIndices(matches[0].fields, function (f, i) {
		return _.every(matches, function (m) {
			return m.fields[i];
		});
	});
	return _.map(matches, function (m) {
		return _.updateIn(m, ['fields'], function (fields) {
			return intersection.map(function (i) {
				return fields[i];
			});
		});
	});
}

var fieldAssembly = function fieldAssembly(datasets) {
	return function (match) {
		return getAssembly(datasets, match.dataset.dsID);
	};
};

var genomicMatches = function genomicMatches(datasets, text) {
	return function (matchesIn) {
		var matches = intersectFields(matchesIn),
		    _ref8 = parsePos(text) || {},
		    hasCoord = _ref8.hasCoord,
		    assemblies = _.uniq(_.map(matches, fieldAssembly(datasets)).filter(function (x) {
			return x;
		})),
		    assembly = hasCoord && assemblies.length > 1 ? [assemblyError] : [],
		    nomatch = matches.length && matches[0].fields.length === 0 ? [fieldError] : [],
		    sig = text.trim()[0] === '=' && !_.getIn(matches, [0, 'sig']),
		    warnings = sig ? [sigError] : [].concat(assembly, nomatch);


		return {
			matches: matches,
			hasCoord: hasCoord,
			warnings: warnings,
			valid: !_.any(matches, function (m) {
				return m.warning;
			}) && _.isEmpty(warnings)
		};
	};
};

// This is still kinda wonky, dispatching on mode before dispatching on type.
var matchFields = {
	Phenotypic: function Phenotypic(_ref9, selected) {
		var datasets = _ref9.datasets,
		    features = _ref9.features;
		return _rx.Observable.zipArray.apply(_rx.Observable, (0, _toConsumableArray3.default)(selected.map(function (i) {
			return doMatch(datasets, features[i].dsID, features[i].name);
		}))).map(function (matches) {
			return { matches: matches, valid: selected.length > 0 };
		});
	},

	Genotypic: function Genotypic(_ref10, selected, text) {
		var datasets = _ref10.datasets;
		return text.trim().length === 0 || !selected.length ? _rx.Observable.of({ valid: false }, _rx.Scheduler.asap) : _rx.Observable.zipArray.apply(_rx.Observable, (0, _toConsumableArray3.default)(selected.map(function (dsID) {
			return doMatch(datasets, dsID, text);
		}))).map(genomicMatches(datasets, text));
	},

	Analytic: function Analytic(_ref11, selected) {
		var datasets = _ref11.datasets,
		    analytic = _ref11.analytic;
		return _rx.Observable.zipArray.apply(_rx.Observable, (0, _toConsumableArray3.default)(selected.map(function (i) {
			return doMatch(datasets, toDsID(analytic[i]), analytic[i].fields, (0, _columnsParam.getOpts)(analytic[i]));
		}))).map(function (matches) {
			return { matches: matches, valid: selected.length > 0 };
		});
	}
};

var VariableSelect = function (_PureComponent) {
	(0, _inherits3.default)(VariableSelect, _PureComponent);

	function VariableSelect(props) {
		(0, _classCallCheck3.default)(this, VariableSelect);

		var _this = (0, _possibleConstructorReturn3.default)(this, (VariableSelect.__proto__ || (0, _getPrototypeOf2.default)(VariableSelect)).call(this, props));

		_initialiseProps.call(_this);

		var text = props.text,
		    fields = props.fields,
		    dataset = props.dataset,
		    datasets = props.datasets,
		    features = props.features,
		    preferred = props.preferred,
		    basicFeatures = props.basicFeatures,
		    _props$mode = props.mode,
		    mode = _props$mode === undefined ? 'Genotypic' : _props$mode;

		var defaults = {
			mode: mode,
			advanced: {
				Genotypic: _.isEmpty(preferred),
				Phenotypic: _.isEmpty(preferred),
				Analytic: false
			},
			basicFeatures: featureIndexes(features, basicFeatures),
			selected: {
				Genotypic: {
					true: [], // advanced
					false: [] // !advanced
				},
				Phenotypic: {
					true: [], // advanced
					false: [] // !advanced
				},
				Analytic: {
					false: []
				}
			},
			value: {
				Genotypic: '',
				Phenotypic: '',
				Analytic: ''
			},
			valid: false,
			hasCoord: false,
			warnings: []
		};

		_this.state = fields && dataset ? applyInitialState[datasetMode(datasets, dataset)](text, fields, dataset, datasets, features, preferred, defaults) : defaults;
		return _this;
	}

	(0, _createClass3.default)(VariableSelect, [{
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(_ref12) {
			var features = _ref12.features,
			    basicFeatures = _ref12.basicFeatures;

			this.setState({
				basicFeatures: featureIndexes(features, basicFeatures)
			});
		}

		// Sharing these streams, to avoid recompute, is complicated by the
		// startWith() operator, which does not subscribe to base observable until
		// the queued value is emitted. The result of this is that the
		// withLatestFrom() operators do not subscribe to their upstream sources,
		// and miss the startWith() of those. The workaround here is to use replay
		// subjects. This is all much too complex.

	}, {
		key: 'componentWillMount',
		value: function componentWillMount() {
			var _this2 = this;

			var events = rxEvents(this, 'mode', 'advanced', 'field', 'select');
			var mode = events.mode.startWith(this.state.mode).publishReplay(1).refCount(),
			    advanced = events.advanced.withLatestFrom(mode, function (advanced, mode) {
				return mode;
			}).scan(function (advanced, mode) {
				return _.updateIn(advanced, [mode], function (a) {
					return !a;
				});
			}, this.state.advanced).startWith(this.state.advanced).publishReplay(1).refCount(),
			    selected = events.select.withLatestFrom(advanced, mode, function (dataset, advanced, mode) {
				return [dataset, mode, advanced[mode]];
			}).scan(function (selected, _ref13) {
				var _ref14 = (0, _slicedToArray3.default)(_ref13, 3),
				    _ref14$ = _ref14[0],
				    selectValue = _ref14$.selectValue,
				    isOn = _ref14$.isOn,
				    mode = _ref14[1],
				    advanced = _ref14[2];

				return _.updateIn(selected, [mode, advanced], function (selected) {
					return _.uniq((isOn ? _.conj : _.without)(selected, selectValue));
				});
			}, this.state.selected).startWith(this.state.selected).publishReplay(1).refCount(),
			    value = events.field.withLatestFrom(mode, function (field, mode) {
				return [field, mode];
			}).scan(function (value, _ref15) {
				var _ref16 = (0, _slicedToArray3.default)(_ref15, 2),
				    field = _ref16[0],
				    mode = _ref16[1];

				return _.assoc(value, mode, field);
			}, this.state.value).startWith(this.state.value).publishReplay(1).refCount();

			this.modeSub = mode.subscribe(function (mode) {
				return _this2.setState({ mode: mode, error: false });
			});
			this.advancedSub = advanced.subscribe(function (advanced) {
				return _this2.setState({ advanced: advanced });
			});
			this.selectedSub = selected.subscribe(function (selected) {
				return _this2.setState({ selected: selected });
			});
			this.valueSub = value.subscribe(function (value) {
				return _this2.setState({ value: value, error: false });
			});

			// valid should only be set true after assessing disposition, but should be set false immediately on
			// user input.
			this.validSub = mode.combineLatest(advanced, selected, value, function (mode, advanced, selected, value) {
				return [mode, selected[mode][advanced[mode]], value[mode]];
			}).do(function () {
				return _this2.setState({ valid: false, loading: true });
			}) // XXX side-effects
			.debounceTime(200).switchMap(function (_ref17) {
				var _ref18 = (0, _slicedToArray3.default)(_ref17, 3),
				    mode = _ref18[0],
				    selected = _ref18[1],
				    value = _ref18[2];

				return matchFields[mode](_this2.props, selected, value);
			}).subscribe(function (valid) {
				return _this2.setState((0, _extends3.default)({ loading: false, warnings: [], matches: [] }, valid));
			}, function (err) {
				console.log(err);_this2.setState({ valid: false, loading: false });
			});
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.modeSub.unsubscribe();
			this.advancedSub.unsubscribe();
			this.selectedSub.unsubscribe();
			this.valueSub.unsubscribe();
			this.validSub.unsubscribe();
		}
	}, {
		key: 'render',
		value: function render() {
			var _state = this.state,
			    mode = _state.mode,
			    matches = _state.matches,
			    hasCoord = _state.hasCoord,
			    advanced = _state.advanced,
			    valid = _state.valid,
			    warnings = _state.warnings,
			    loading = _state.loading,
			    error = _state.error,
			    unavailable = _state.unavailable,
			    basicFeatures = _state.basicFeatures,
			    value = this.state.value[mode],
			    selected = this.state.selected[mode][advanced[mode]],
			    _props = this.props,
			    colId = _props.colId,
			    controls = _props.controls,
			    datasets = _props.datasets,
			    features = _props.features,
			    preferred = _props.preferred,
			    analytic = _props.analytic,
			    title = _props.title,
			    helpText = _props.helpText,
			    width = _props.width,
			    formError = getWarningText(matches, datasets, selected, warnings, value).join(' ') || error,
			    subtitle = unavailable ? 'This variable is currently unavailable. You may choose a different variable, or cancel to continue viewing the cached data.' : undefined,
			    contentSpecificHelp = _.getIn(helpText, [mode]),
			    ModeForm = getModeFields[mode],
			    wizardProps = {
				colId: colId,
				controls: controls,
				subtitle: subtitle,
				title: title,
				contentSpecificHelp: contentSpecificHelp,
				onDone: this.onDone,
				onDoneInvalid: this.onDoneInvalid,
				valid: valid,
				loading: loading,
				width: width
			},
			    dataTypeProps = {
				label: 'Select Data Type',
				value: mode,
				onChange: this.on.mode,
				options: [{ label: 'Genomic', value: 'Genotypic' }, { label: 'Phenotypic', value: 'Phenotypic' }].concat((0, _toConsumableArray3.default)(!_.isEmpty(analytic) ? [{ label: 'Analytic', value: 'Analytic' }] : []))
			};

			return React.createElement(
				WizardCard,
				wizardProps,
				React.createElement(XRadioGroup, dataTypeProps),
				React.createElement(ModeForm, {
					error: formError,
					onChange: this.onChange,
					onReturn: valid ? this.onDone : undefined,
					onFieldChange: this.on.field,
					hideAssembly: !hasCoord,
					datasets: datasets,
					selected: selected,
					value: value,
					features: features,
					preferred: preferred,
					analytic: analytic,
					basicFeatures: basicFeatures,
					onAddFeature: this.onAddFeature,
					onAdvancedClick: this.on.advanced,
					advanced: advanced[mode] })
			);
		}
	}]);
	return VariableSelect;
}(_PureComponent3.default);

// Render a load warning in a wrapper component so we don't
// try to initialize state when we're not holding the cohort
// data. There might be a better state model that avoids this.
//
// We are passed data incrementally as it arrives from the servers,
// so use a timeout to decide if we should keep waiting.


var _initialiseProps = function _initialiseProps() {
	var _this5 = this;

	this.onChange = function (selectValue, isOn) {
		_this5.on.select({ selectValue: selectValue, isOn: isOn });
	};

	this.onDone = function () {
		var _props4 = _this5.props,
		    pos = _props4.pos,
		    onSelect = _props4.onSelect,
		    matches = _this5.state.matches;


		onSelect(pos, matches);
	};

	this.onDoneInvalid = function () {
		var mode = _this5.state.mode,
		    value = _this5.state.value[mode];

		// Highlight the input field, since the user has forgotten it.
		// Might want to also scroll it into position, and also
		// scroll it into position if there's another error, like
		// assembly mismatch.
		if (mode === 'Genotypic' && value.trim().length === 0) {
			_this5.setState({ error: true });
		}
	};

	this.onAddFeature = function (featureIn) {
		var features = _this5.props.features,
		    _state2 = _this5.state,
		    basicFeatures = _state2.basicFeatures,
		    value = _state2.value,
		    mode = _state2.mode,
		    i = (featureIn ? features.indexOf(featureIn) : _.findIndex(features, _.matcher({ label: value[mode] }))).toString();

		if (i !== "-1") {
			_this5.setState({ basicFeatures: _.uniq([].concat((0, _toConsumableArray3.default)(basicFeatures), [i])) });
			_this5.on.select({ selectValue: i, isOn: true });
		}
		_this5.on.field("");
	};
};

var LoadingNotice = function (_React$Component) {
	(0, _inherits3.default)(LoadingNotice, _React$Component);

	function LoadingNotice() {
		var _ref19;

		var _temp, _this3, _ret;

		(0, _classCallCheck3.default)(this, LoadingNotice);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this3 = (0, _possibleConstructorReturn3.default)(this, (_ref19 = LoadingNotice.__proto__ || (0, _getPrototypeOf2.default)(LoadingNotice)).call.apply(_ref19, [this].concat(args))), _this3), _this3.state = { wait: true }, _temp), (0, _possibleConstructorReturn3.default)(_this3, _ret);
	}

	(0, _createClass3.default)(LoadingNotice, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			var _this4 = this;

			this.timeout = setTimeout(function () {
				return _this4.setState({ wait: false });
			}, 3000);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			clearTimeout(this.timeout);
		}
	}, {
		key: 'render',
		value: function render() {
			var _props2 = this.props,
			    analytic = _props2.analytic,
			    preferred = _props2.preferred,
			    datasets = _props2.datasets,
			    features = _props2.features,
			    basicFeatures = _props2.basicFeatures,
			    wait = this.state.wait;

			if (wait && (!preferred || _.isEmpty(datasets) || _.isEmpty(features) || !basicFeatures || !analytic)) {
				var _props3 = this.props,
				    colId = _props3.colId,
				    controls = _props3.controls,
				    title = _props3.title,
				    width = _props3.width,
				    wizardProps = {
					colId: colId,
					controls: controls,
					loading: true,
					loadingCohort: true,
					title: title,
					width: width
				};

				return React.createElement(WizardCard, wizardProps);
			}
			return React.createElement(VariableSelect, this.props);
		}
	}]);
	return LoadingNotice;
}(React.Component);

module.exports = LoadingNotice;