'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * UCSC Xena Client
 * http://xena.ucsc.edu
 *
 * Zoom help tag displayed on mousedown and drag over gene or samples zoom.
 */

// Core dependencies, components
var React = require('react');

var _require = require('react-overlays'),
    Portal = _require.Portal;

var columnZoom = require('../columnZoom');

// Styles
var compStyles = require('./ZoomHelpTag.module.css');

// Return zoom text when user has pressed mouse down but has not yet moved the mouse
var defaultText = function defaultText() {
	return 'Drag to zoom';
};

// Return zoom text when user is currently executing a horizontal zoom
var geneZoomText = function geneZoomText(annotationPct) {
	return 'Zoom to ' + (annotationPct < 1 ? '< 1' : annotationPct) + '%';
};

// Return zoom text when user is currently executing a vertical zoom
var sampleZoomText = function sampleZoomText(zoomTo) {
	var sampleRowFrom = zoomTo.index + 1,
	    sampleRowTo = zoomTo.index + zoomTo.count;
	return zoomTo.count === 1 ? 'Zoom to row ' + sampleRowFrom : 'Zoom to rows ' + sampleRowFrom + ' - ' + sampleRowTo;
};

var ZoomHelpTag = function (_React$Component) {
	(0, _inherits3.default)(ZoomHelpTag, _React$Component);

	function ZoomHelpTag() {
		(0, _classCallCheck3.default)(this, ZoomHelpTag);
		return (0, _possibleConstructorReturn3.default)(this, (ZoomHelpTag.__proto__ || (0, _getPrototypeOf2.default)(ZoomHelpTag)).apply(this, arguments));
	}

	(0, _createClass3.default)(ZoomHelpTag, [{
		key: 'render',
		value: function render() {
			var _props = this.props,
			    column = _props.column,
			    selection = _props.selection;

			if (selection) {
				var crosshair = selection.crosshair,
				    direction = selection.direction,
				    offset = selection.offset,
				    overlay = selection.overlay,
				    zone = selection.zone,
				    zoomTo = selection.zoomTo,
				    fieldType = column.fieldType,
				    zoomToLength = columnZoom.zoomToLength({ fieldType: fieldType, zoomTo: zoomTo }),
				    geneZoomPct = Math.round(zoomToLength / columnZoom.maxGeneZoomLength(column) * 100),
				    annotationZoom = direction === 'h',
				    noZoom = overlay.sstart === overlay.send,
				    tagInstruction = noZoom ? defaultText() : annotationZoom ? geneZoomText(geneZoomPct) : sampleZoomText(zoomTo),
				    crosshairVFromRightBounds = document.body.offsetWidth - crosshair.x,
				    tagPosLeft = zone === 'a' ? crosshair.x : annotationZoom ? offset.x + overlay.sstart === crosshair.x ? 'unset' : crosshair.x : crosshair.x,
				    tagPosRight = annotationZoom && zone === 's' && offset.x + overlay.sstart === crosshair.x ? crosshairVFromRightBounds : 'unset',
				    tagPosTop = zone === 'a' ? offset.y - 48 : annotationZoom ? crosshair.y : offset.y + overlay.sstart === crosshair.y ? crosshair.y - 40 : crosshair.y;
			}
			return React.createElement(
				Portal,
				{ container: document.body },
				selection ? React.createElement(
					'div',
					{ className: compStyles.zoomHelpTag,
						style: { left: tagPosLeft, right: tagPosRight, top: tagPosTop } },
					React.createElement(
						'div',
						{ className: compStyles.zoomInstructions },
						tagInstruction
					)
				) : null
			);
		}
	}]);
	return ZoomHelpTag;
}(React.Component);

module.exports = ZoomHelpTag;